@@ -42,11 +42,7 @@ struct ztdeth_header {
 
 /* We take the raw message, put it in an ethernet frame, and add a
    two byte addressing header at the top for future use */
-#ifdef DEFINE_SPINLOCK
 static DEFINE_SPINLOCK(zlock);
-#else
-static spinlock_t zlock = SPIN_LOCK_UNLOCKED;
-#endif
 
 static struct sk_buff_head skbs;
 
@@ -144,7 +140,7 @@ static int ztdeth_notifier(struct notifier_block *block, unsigned long event, vo
 	return 0;
 }
 
-static int ztdeth_transmit(void *pvt, unsigned char *msg, int msglen)
+static void ztdeth_transmit(struct dahdi_dynamic *dyn, u8 *msg, size_t msglen)
 {
 	struct ztdeth *z;
 	struct sk_buff *skb;
@@ -155,7 +151,7 @@ static int ztdeth_transmit(void *pvt, unsigned char *msg, int msglen)
 	unsigned short subaddr; /* Network byte order */
 
 	spin_lock_irqsave(&zlock, flags);
-	z = pvt;
+	z = dyn->pvt;
 	if (z->dev) {
 		/* Copy fields to local variables to remove spinlock ASAP */
 		dev = z->dev;
@@ -193,7 +189,6 @@ static int ztdeth_transmit(void *pvt, unsigned char *msg, int msglen)
 	}
 	else
 		spin_unlock_irqrestore(&zlock, flags);
-	return 0;
 }
 
 
@@ -271,9 +266,9 @@ static int hex2int(char *s)
 	return tmp;
 }
 
-static void ztdeth_destroy(void *pvt)
+static void ztdeth_destroy(struct dahdi_dynamic *dyn)
 {
-	struct ztdeth *z = pvt;
+	struct ztdeth *z = dyn->pvt;
 	unsigned long flags;
 	struct ztdeth *prev=NULL, *cur;
 	spin_lock_irqsave(&zlock, flags);
@@ -293,17 +288,17 @@ static void ztdeth_destroy(void *pvt)
 	if (cur == z) {	/* Successfully removed */
 		printk(KERN_INFO "TDMoE: Removed interface for %s\n", z->span->name);
 		kfree(z);
-		module_put(THIS_MODULE);
 	}
 }
 
-static void *ztdeth_create(struct dahdi_span *span, char *addr)
+static int ztdeth_create(struct dahdi_dynamic *dyn, const char *addr)
 {
 	struct ztdeth *z;
 	char src[256];
 	char tmp[256], *tmp2, *tmp3, *tmp4 = NULL;
 	int res,x;
 	unsigned long flags;
+	struct dahdi_span *const span = &dyn->span;
 
 	z = kmalloc(sizeof(struct ztdeth), GFP_KERNEL);
 	if (z) {
@@ -311,16 +306,16 @@ static void *ztdeth_create(struct dahdi_span *span, char *addr)
 		memset(z, 0, sizeof(struct ztdeth));
 
 		/* Address should be <dev>/<macaddr>[/subaddr] */
-		dahdi_copy_string(tmp, addr, sizeof(tmp));
+		strlcpy(tmp, addr, sizeof(tmp));
 		tmp2 = strchr(tmp, '/');
 		if (tmp2) {
 			*tmp2 = '\0';
 			tmp2++;
-			dahdi_copy_string(z->ethdev, tmp, sizeof(z->ethdev));
+			strlcpy(z->ethdev, tmp, sizeof(z->ethdev));
 		} else {
 			printk(KERN_NOTICE "Invalid TDMoE address (no device) '%s'\n", addr);
 			kfree(z);
-			return NULL;
+			return -EINVAL;
 		}
 		if (tmp2) {
 			tmp4 = strchr(tmp2+1, '/');
@@ -348,12 +343,12 @@ static void *ztdeth_create(struct dahdi_span *span, char *addr)
 			if (x != 6) {
 				printk(KERN_NOTICE "TDMoE: Invalid MAC address in: %s\n", addr);
 				kfree(z);
-				return NULL;
+				return -EINVAL;
 			}
 		} else {
 			printk(KERN_NOTICE "TDMoE: Missing MAC address\n");
 			kfree(z);
-			return NULL;
+			return -EINVAL;
 		}
 		if (tmp4) {
 			int sub = 0;
@@ -367,7 +362,7 @@ static void *ztdeth_create(struct dahdi_span *span, char *addr)
 				} else {
 					printk(KERN_NOTICE "TDMoE: Invalid subaddress\n");
 					kfree(z);
-					return NULL;
+					return -EINVAL;
 				}
 				mul *= 10;
 				tmp3--;
@@ -382,7 +377,7 @@ static void *ztdeth_create(struct dahdi_span *span, char *addr)
 		if (!z->dev) {
 			printk(KERN_NOTICE "TDMoE: Invalid device '%s'\n", z->ethdev);
 			kfree(z);
-			return NULL;
+			return -EINVAL;
 		}
 		z->span = span;
 		src[0] ='\0';
@@ -394,20 +389,20 @@ static void *ztdeth_create(struct dahdi_span *span, char *addr)
 		spin_lock_irqsave(&zlock, flags);
 		z->next = zdevs;
 		zdevs = z;
+		dyn->pvt = z;
 		spin_unlock_irqrestore(&zlock, flags);
-		if(!try_module_get(THIS_MODULE))
-			printk(KERN_DEBUG "TDMoE: Unable to increment module use count\n");
 	}
-	return z;
+	return (z) ? 0 : -ENOMEM;
 }
 
 static struct dahdi_dynamic_driver ztd_eth = {
-	"eth",
-	"Ethernet",
-	ztdeth_create,
-	ztdeth_destroy,
-	ztdeth_transmit,
-	ztdeth_flush
+	.owner = THIS_MODULE,
+	.name = "eth",
+	.desc = "Ethernet",
+	.create = ztdeth_create,
+	.destroy = ztdeth_destroy,
+	.transmit = ztdeth_transmit,
+	.flush = ztdeth_flush,
 };
 
 static struct notifier_block ztdeth_nblock = {
@@ -418,7 +413,7 @@ static int __init ztdeth_init(void)
 {
 	dev_add_pack(&ztdeth_ptype);
 	register_netdevice_notifier(&ztdeth_nblock);
-	dahdi_dynamic_register(&ztd_eth);
+	dahdi_dynamic_register_driver(&ztd_eth);
 
 	skb_queue_head_init(&skbs);
 
@@ -429,7 +424,7 @@ static void __exit ztdeth_exit(void)
 {
 	dev_remove_pack(&ztdeth_ptype);
 	unregister_netdevice_notifier(&ztdeth_nblock);
-	dahdi_dynamic_unregister(&ztd_eth);
+	dahdi_dynamic_unregister_driver(&ztd_eth);
 }
 
 MODULE_DESCRIPTION("DAHDI Dynamic TDMoE Support");
