@@ -46,7 +46,7 @@ static LIST_HEAD(registration_list);
  * is used as a simplistic way to spread the load amongst the different hardware
  * transcoders in the system. */
 static LIST_HEAD(active_list);
-static spinlock_t translock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(translock);
 
 EXPORT_SYMBOL(dahdi_transcoder_register);
 EXPORT_SYMBOL(dahdi_transcoder_unregister);
@@ -328,7 +328,7 @@ static long dahdi_tc_getinfo(unsigned long data)
 		return -ENOSYS;
 	}
 
-	dahdi_copy_string(info.name, tc->name, sizeof(info.name));
+	strlcpy(info.name, tc->name, sizeof(info.name));
 	info.numchannels = tc->numchannels;
 	info.srcfmts = tc->srcfmts;
 	info.dstfmts = tc->dstfmts;
@@ -403,12 +403,6 @@ static int dahdi_tc_ioctl(struct inode *inode, struct file *file, unsigned int c
 }
 #endif
 
-static int dahdi_tc_mmap(struct file *file, struct vm_area_struct *vma)
-{
-	printk(KERN_ERR "%s: mmap interface deprecated.\n", THIS_MODULE->name);
-	return -ENOSYS;
-}
-
 static unsigned int dahdi_tc_poll(struct file *file, struct poll_table_struct *wait_table)
 {
 	int ret;
@@ -440,12 +434,11 @@ static struct file_operations __dahdi_transcode_fops = {
 	.read =    dahdi_tc_read,
 	.write =   dahdi_tc_write,
 	.poll =    dahdi_tc_poll,
-	.mmap =    dahdi_tc_mmap,
 };
 
 static struct dahdi_chardev transcode_chardev = {
 	.name = "transcode",
-	.minor = 250,
+	.minor = DAHDI_TRANSCODE,
 };
 
 static int dahdi_transcode_init(void)
