@@ -49,7 +49,7 @@
 #include "voicebus.h"
 #include "vpmadtreg.h"
 
-static rwlock_t ifacelock;
+static DEFINE_SPINLOCK(ifacelock);
 static struct vpmadt032 *ifaces[MAX_DSP_CORES];
 
 #define vpm_info(vpm, format, arg...)         \
@@ -59,13 +59,13 @@ static inline struct vpmadt032 *find_iface(const unsigned short dspid)
 {
 	struct vpmadt032 *ret;
 
-	read_lock(&ifacelock);
+	spin_lock(&ifacelock);
 	if (ifaces[dspid]) {
 		ret = ifaces[dspid];
 	} else {
 		ret = NULL;
 	}
-	read_unlock(&ifacelock);
+	spin_unlock(&ifacelock);
 	return ret;
 }
 
@@ -492,30 +492,17 @@ void vpmadt032_echocan_free(struct vpmadt032 *vpm, int channo,
 EXPORT_SYMBOL(vpmadt032_echocan_free);
 
 struct vpmadt032 *
-vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
+vpmadt032_alloc(struct vpmadt032_options *options)
 {
 	struct vpmadt032 *vpm;
 	int i;
-	const char *suffix = "-vpm";
-	size_t length;
 
 	might_sleep();
 
-	length = strlen(board_name) + strlen(suffix) + 1;
-
-	/* Add a little extra to store the wq_name. */
-	vpm = kzalloc(sizeof(*vpm) + length, GFP_KERNEL);
+	vpm = kzalloc(sizeof(*vpm), GFP_KERNEL);
 	if (!vpm)
 		return NULL;
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
-	/* There is a limit on the length of the name of the workqueue. */
-	strcpy(vpm->wq_name, "vpmadt032");
-#else
-	strcpy(vpm->wq_name, board_name);
-	strcat(vpm->wq_name, suffix);
-#endif
-
 	/* Init our vpmadt032 struct */
 	memcpy(&vpm->options, options, sizeof(*options));
 	spin_lock_init(&vpm->list_lock);
@@ -526,12 +513,17 @@ vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
 	sema_init(&vpm->sem, 1);
 	vpm->curpage = 0x80;
 	vpm->dspid = -1;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
+	INIT_WORK(&vpm->work, vpmadt032_bh, vpm);
+#else
+	INIT_WORK(&vpm->work, vpmadt032_bh);
+#endif
 
 	/* Do not use the global workqueue for processing these events.  Some of
 	 * the operations can take 100s of ms, most of that time spent sleeping.
 	 * On single CPU systems, this unduly serializes operations accross
 	 * multiple vpmadt032 instances. */
-	vpm->wq = create_singlethread_workqueue(vpm->wq_name);
+	vpm->wq = create_singlethread_workqueue("vpmadt032");
 	if (!vpm->wq) {
 		kfree(vpm);
 		return NULL;
@@ -539,7 +531,7 @@ vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
 
 	/* Place this structure in the ifaces array so that the DspId from the
 	 * Gpak Library can be used to locate it. */
-	write_lock(&ifacelock);
+	spin_lock(&ifacelock);
 	for (i=0; i<MAX_DSP_CORES; ++i) {
 		if (NULL == ifaces[i]) {
 			ifaces[i] = vpm;
@@ -547,7 +539,7 @@ vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
 			break;
 		}
 	}
-	write_unlock(&ifacelock);
+	spin_unlock(&ifacelock);
 
 	if (-1 == vpm->dspid) {
 		kfree(vpm);
@@ -618,36 +610,26 @@ int vpmadt032_reset(struct vpmadt032 *vpm)
 }
 EXPORT_SYMBOL(vpmadt032_reset);
 
-int
-vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
+/**
+ * vpmadt032_test - Check if there is a VPMADT032 present on voicebus device.
+ * @vpm:	Allocated with vpmadt032_alloc previously.
+ * @vb:		Voicebus structure to test on.
+ *
+ * Returns 0 if there is a device, otherwise -ENODEV.
+ *
+ */
+int vpmadt032_test(struct vpmadt032 *vpm, struct voicebus *vb)
 {
-	int i;
-	u16 reg;
-	int res = -EFAULT;
-	unsigned long stoptime;
-	struct device *dev;
-	gpakPingDspStat_t pingstatus;
-
-	BUG_ON(!vpm->setchanconfig_from_state);
-	BUG_ON(!vpm->wq);
-	BUG_ON(!vb);
+	u8 reg;
+	int i, x;
+	struct device *dev = &vb->pdev->dev;
 
 	vpm->vb = vb;
 
-	might_sleep();
-
-	dev = &vpm->vb->pdev->dev;
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
-	INIT_WORK(&vpm->work, vpmadt032_bh, vpm);
-#else
-	INIT_WORK(&vpm->work, vpmadt032_bh);
-#endif
 	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
 		dev_info(dev, "VPMADT032 Testing page access: ");
 
 	for (i = 0; i < 0xf; i++) {
-		int x;
 		for (x = 0; x < 3; x++) {
 			vpmadt032_setpage(vpm, i);
 			reg = vpmadt032_getpage(vpm);
@@ -658,14 +640,42 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 						"VPMADT032 Failed HI page " \
 						"test\n", i, reg);
 				}
-				res = -ENODEV;
-				goto failed_exit;
+				return -ENODEV;
 			}
 		}
 	}
 
 	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
-		dev_info(&vpm->vb->pdev->dev, "Passed\n");
+		dev_info(dev, "Passed\n");
+
+	return 0;
+}
+EXPORT_SYMBOL(vpmadt032_test);
+
+/**
+ * vpmadt032_init - Initialize and load VPMADT032 firmware.
+ * @vpm:	Allocated with vpmadt032_alloc previously.
+ *
+ * Returns 0 on success.  This must be called after vpmadt032_test already
+ * checked if there appears to be a VPMADT032 installed on the board.
+ *
+ */
+int vpmadt032_init(struct vpmadt032 *vpm)
+{
+	int i;
+	u16 reg;
+	int res = -EFAULT;
+	unsigned long stoptime;
+	struct device *dev;
+	gpakPingDspStat_t pingstatus;
+
+	BUG_ON(!vpm->setchanconfig_from_state);
+	BUG_ON(!vpm->wq);
+	BUG_ON(!vpm->vb);
+
+	might_sleep();
+
+	dev = &vpm->vb->pdev->dev;
 
 	stoptime = jiffies + 3*HZ;
 	set_bit(VPM150M_HPIRESET, &vpm->control);
@@ -712,14 +722,14 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 		goto failed_exit;
 	}
 
-	res = vpmadtreg_loadfirmware(vb);
+	res = vpmadtreg_loadfirmware(vpm->vb);
 	if (res) {
-		dev_info(&vb->pdev->dev, "Failed to load the firmware.\n");
+		dev_info(&vpm->vb->pdev->dev, "Failed to load the firmware.\n");
 		return res;
 	}
 	vpm->curpage = -1;
 
-	dev_info(&vb->pdev->dev, "Booting VPMADT032\n");
+	dev_info(&vpm->vb->pdev->dev, "Booting VPMADT032\n");
 
 	stoptime = jiffies + 3*HZ;
 	set_bit(VPM150M_SWRESET, &vpm->control);
@@ -736,7 +746,7 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 	pingstatus = gpakPingDsp(vpm->dspid, &vpm->version);
 
 	if (!pingstatus) {
-		dev_info(&vb->pdev->dev, "VPM present and operational "
+		dev_info(&vpm->vb->pdev->dev, "VPM present and operational "
 			"(Firmware version %x)\n", vpm->version);
 	} else {
 		dev_notice(&vpm->vb->pdev->dev, "VPMADT032 Failed! Unable to ping the DSP (%d)!\n", pingstatus);
@@ -822,20 +832,13 @@ void vpmadt032_free(struct vpmadt032 *vpm)
 	}
 
 	BUG_ON(ifaces[vpm->dspid] != vpm);
-	write_lock(&ifacelock);
+	spin_lock(&ifacelock);
 	ifaces[vpm->dspid] = NULL;
-	write_unlock(&ifacelock);
+	spin_unlock(&ifacelock);
 	kfree(vpm);
 }
 EXPORT_SYMBOL(vpmadt032_free);
 
-int vpmadt032_module_init(void)
-{
-	rwlock_init(&ifacelock);
-	memset(ifaces, 0, sizeof(ifaces));
-	return 0;
-}
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  * gpakReadDspMemory - Read DSP memory.
  *
