@@ -92,7 +92,7 @@
 #define 	CSR9_MMC		0x00040000
 #define 	CSR9_MDI		0x00080000
 
-#define OWN_BIT (1 << 31)
+#define OWN_BIT cpu_to_le32(1 << 31)
 
 #ifdef CONFIG_VOICEBUS_ECREFERENCE
 
@@ -199,13 +199,6 @@ EXPORT_SYMBOL(dahdi_fifo_alloc);
 #endif /* CONFIG_VOICEBUS_ECREFERENCE */
 
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
-kmem_cache_t *voicebus_vbb_cache;
-#else
-struct kmem_cache *voicebus_vbb_cache;
-#endif
-EXPORT_SYMBOL(voicebus_vbb_cache);
-
 /* In memory structure shared by the host and the adapter. */
 struct voicebus_descriptor {
 	volatile __le32 des0;
@@ -243,7 +236,7 @@ vb_initialize_descriptors(struct voicebus *vb, struct voicebus_descriptor_list *
 	int i;
 	struct voicebus_descriptor *d;
 	const u32 END_OF_RING = 0x02000000;
-	u8 cache_line_size;
+	u8 cacheline_size;
 
 	BUG_ON(!dl);
 
@@ -253,15 +246,16 @@ vb_initialize_descriptors(struct voicebus *vb, struct voicebus_descriptor_list *
 	 * cache-line sizes that we support.
 	 *
 	 */
-	if (pci_read_config_byte(vb->pdev, 0x0c, &cache_line_size)) {
+	if (pci_read_config_byte(vb->pdev, PCI_CACHE_LINE_SIZE,
+				 &cacheline_size)) {
 		dev_err(&vb->pdev->dev, "Failed read of cache line "
 			"size from PCI configuration space.\n");
 		return -EIO;
 	}
 
-	if ((0x08 == cache_line_size) || (0x10 == cache_line_size) ||
-	    (0x20 == cache_line_size)) {
-		dl->padding = (cache_line_size*sizeof(u32)) - sizeof(*d);
+	if ((0x08 == cacheline_size) || (0x10 == cacheline_size) ||
+	    (0x20 == cacheline_size)) {
+		dl->padding = (cacheline_size*sizeof(u32)) - sizeof(*d);
 	} else {
 		dl->padding = 0;
 	}
@@ -274,10 +268,10 @@ vb_initialize_descriptors(struct voicebus *vb, struct voicebus_descriptor_list *
 	memset(dl->desc, 0, (sizeof(*d) + dl->padding) * DRING_SIZE);
 	for (i = 0; i < DRING_SIZE; ++i) {
 		d = vb_descriptor(dl, i);
-		d->des1 = des1;
+		d->des1 = cpu_to_le32(des1);
 	}
 	d->des1 |= cpu_to_le32(END_OF_RING);
-	atomic_set(&dl->count, 0);
+	dl->count = 0;
 	return 0;
 }
 
@@ -292,7 +286,7 @@ vb_initialize_tx_descriptors(struct voicebus *vb)
 	struct voicebus_descriptor *d;
 	struct voicebus_descriptor_list *dl = &vb->txd;
 	const u32 END_OF_RING = 0x02000000;
-	u8 cache_line_size;
+	u8 cacheline_size;
 
 	WARN_ON(!dl);
 	WARN_ON((NULL == vb->idle_vbb) || (0 == vb->idle_vbb_dma_addr));
@@ -303,15 +297,16 @@ vb_initialize_tx_descriptors(struct voicebus *vb)
 	 * cache-line sizes that we support.
 	 *
 	 */
-	if (pci_read_config_byte(vb->pdev, 0x0c, &cache_line_size)) {
+	if (pci_read_config_byte(vb->pdev, PCI_CACHE_LINE_SIZE,
+				 &cacheline_size)) {
 		dev_err(&vb->pdev->dev, "Failed read of cache line "
 			"size from PCI configuration space.\n");
 		return -EIO;
 	}
 
-	if ((0x08 == cache_line_size) || (0x10 == cache_line_size) ||
-	    (0x20 == cache_line_size)) {
-		dl->padding = (cache_line_size*sizeof(u32)) - sizeof(*d);
+	if ((0x08 == cacheline_size) || (0x10 == cacheline_size) ||
+	    (0x20 == cacheline_size)) {
+		dl->padding = (cacheline_size*sizeof(u32)) - sizeof(*d);
 	} else {
 		dl->padding = 0;
 	}
@@ -326,12 +321,12 @@ vb_initialize_tx_descriptors(struct voicebus *vb)
 
 	for (i = 0; i < DRING_SIZE; ++i) {
 		d = vb_descriptor(dl, i);
-		d->des1 = des1;
+		d->des1 = cpu_to_le32(des1);
 		dl->pending[i] = NULL;
 		d->buffer1 = 0;
 	}
 	d->des1 |= cpu_to_le32(END_OF_RING);
-	atomic_set(&dl->count, 0);
+	dl->count = 0;
 	return 0;
 }
 
@@ -478,20 +473,20 @@ vb_cleanup_tx_descriptors(struct voicebus *vb)
 	while (!list_empty(&vb->tx_complete)) {
 		vbb = list_entry(vb->tx_complete.next, struct vbb, entry);
 		list_del(&vbb->entry);
-		kmem_cache_free(voicebus_vbb_cache, vbb);
+		dma_pool_free(vb->pool, vbb, vbb->dma_addr);
 	}
 
 	for (i = 0; i < DRING_SIZE; ++i) {
 		d = vb_descriptor(dl, i);
-		if (d->buffer1 && (d->buffer1 != vb->idle_vbb_dma_addr)) {
+		if (d->buffer1 &&
+		    (d->buffer1 != le32_to_cpu(vb->idle_vbb_dma_addr))) {
 			WARN_ON(!dl->pending[i]);
-			dma_unmap_single(&vb->pdev->dev, d->buffer1,
-					 VOICEBUS_SFRAME_SIZE, DMA_TO_DEVICE);
-			kmem_cache_free(voicebus_vbb_cache, dl->pending[i]);
+			vbb = dl->pending[i];
+			dma_pool_free(vb->pool, vbb, vbb->dma_addr);
 		}
 		if (NORMAL == vb->mode) {
-			d->des1 |= 0x80000000;
-			d->buffer1 = vb->idle_vbb_dma_addr;
+			d->des1 |= cpu_to_le32(0x80000000);
+			d->buffer1 = cpu_to_le32(vb->idle_vbb_dma_addr);
 			dl->pending[i] = vb->idle_vbb;
 			SET_OWNED(d);
 		} else {
@@ -502,7 +497,7 @@ vb_cleanup_tx_descriptors(struct voicebus *vb)
 	}
 
 	dl->head = dl->tail = 0;
-	atomic_set(&dl->count, 0);
+	dl->count = 0;
 	vb_enable_deferred(vb);
 }
 
@@ -517,8 +512,6 @@ static void vb_cleanup_rx_descriptors(struct voicebus *vb)
 	for (i = 0; i < DRING_SIZE; ++i) {
 		d = vb_descriptor(dl, i);
 		if (d->buffer1) {
-			dma_unmap_single(&vb->pdev->dev, d->buffer1,
-					 VOICEBUS_SFRAME_SIZE, DMA_FROM_DEVICE);
 			d->buffer1 = 0;
 			BUG_ON(!dl->pending[i]);
 			vbb = dl->pending[i];
@@ -529,7 +522,7 @@ static void vb_cleanup_rx_descriptors(struct voicebus *vb)
 	}
 	dl->head = 0;
 	dl->tail = 0;
-	atomic_set(&dl->count, 0);
+	dl->count = 0;
 	vb_enable_deferred(vb);
 }
 
@@ -558,7 +551,7 @@ vb_free_descriptors(struct voicebus *vb, struct voicebus_descriptor_list *dl)
 	while (!list_empty(&vb->free_rx)) {
 		vbb = list_entry(vb->free_rx.next, struct vbb, entry);
 		list_del(&vbb->entry);
-		kmem_cache_free(voicebus_vbb_cache, vbb);
+		dma_pool_free(vb->pool, vbb, vbb->dma_addr);
 	}
 }
 
@@ -636,48 +629,65 @@ vb_reset_interface(struct voicebus *vb)
 	unsigned long timeout;
 	u32 reg;
 	u32 pci_access;
-	const u32 DEFAULT_PCI_ACCESS = 0xffdc0002;
-	u8 cache_line_size;
+	enum {
+		/* Software Reset */
+		SWR		= (1 << 0),
+		/* Bus Arbitration (1 for priority transmit) */
+		BAR		= (1 << 1),
+		/* Memory Write Invalidate */
+		MWI		= (1 << 24),
+		/* Memory Read Line */
+		MRL		= (1 << 23),
+		/* Descriptor Skip Length */
+		DSLShift	= 2,
+		/* Cache Alignment */
+		CALShift	= 14,
+		/* Transmit Auto Pollling */
+		TAPShift	= 17,
+	};
+	const u32 DEFAULT_PCI_ACCESS = MWI | MRL | (0x2 << TAPShift) | BAR;
+	u8 cacheline_size;
 	BUG_ON(in_interrupt());
 
-	if (pci_read_config_byte(vb->pdev, 0x0c, &cache_line_size)) {
+	if (pci_read_config_byte(vb->pdev, PCI_CACHE_LINE_SIZE,
+				 &cacheline_size)) {
 		dev_err(&vb->pdev->dev, "Failed read of cache line "
 			"size from PCI configuration space.\n");
 		return -EIO;
 	}
 
-	switch (cache_line_size) {
+	switch (cacheline_size) {
 	case 0x08:
-		pci_access = DEFAULT_PCI_ACCESS | (0x1 << 14);
+		pci_access = DEFAULT_PCI_ACCESS | (0x1 << CALShift);
 		break;
 	case 0x10:
-		pci_access = DEFAULT_PCI_ACCESS | (0x2 << 14);
+		pci_access = DEFAULT_PCI_ACCESS | (0x2 << CALShift);
 		break;
 	case 0x20:
-		pci_access = DEFAULT_PCI_ACCESS | (0x3 << 14);
+		pci_access = DEFAULT_PCI_ACCESS | (0x3 << CALShift);
 		break;
 	default:
 		if (*vb->debug) {
 			dev_warn(&vb->pdev->dev, "Host system set a cache "
 				 "size of %d which is not supported. "
 				 "Disabling memory write line and memory "
-				 "read line.\n", cache_line_size);
+				 "read line.\n", cacheline_size);
 		}
 		pci_access = 0xfe584202;
 		break;
 	}
 
 	/* The transmit and receive descriptors will have the same padding. */
-	pci_access |= ((vb->txd.padding / sizeof(u32)) << 2) & 0x7c;
+	pci_access |= ((vb->txd.padding / sizeof(u32)) << DSLShift) & 0x7c;
 
-	vb_setctl(vb, 0x0000, pci_access | 1);
+	vb_setctl(vb, 0x0000, pci_access | SWR);
 
 	timeout = jiffies + HZ/10; /* 100ms interval */
 	do {
 		reg = vb_getctl(vb, 0x0000);
-	} while ((reg & 0x00000001) && time_before(jiffies, timeout));
+	} while ((reg & SWR) && time_before(jiffies, timeout));
 
-	if (reg & 0x00000001) {
+	if (reg & SWR) {
 		if (-1 == reg) {
 			dev_err(&vb->pdev->dev,
 				"Unable to read I/O registers.\n");
@@ -715,10 +725,9 @@ vb_submit_rxb(struct voicebus *vb, struct vbb *vbb)
 
 	dl->pending[tail] = vbb;
 	dl->tail = (++tail) & DRING_MASK;
-	d->buffer1 = dma_map_single(&vb->pdev->dev, vbb->data,
-				    sizeof(vbb->data), DMA_FROM_DEVICE);
+	d->buffer1 = cpu_to_le32(vbb->dma_addr);
 	SET_OWNED(d); /* That's it until the hardware is done with it. */
-	atomic_inc(&dl->count);
+	++dl->count;
 	return 0;
 }
 
@@ -729,10 +738,11 @@ static int __voicebus_transmit(struct voicebus *vb, struct vbb *vbb)
 
 	d = vb_descriptor(dl, dl->tail);
 
-	if (unlikely((d->buffer1 != vb->idle_vbb_dma_addr) && d->buffer1)) {
+	if (unlikely((le32_to_cpu(d->buffer1) != vb->idle_vbb_dma_addr) &&
+		      d->buffer1)) {
 		if (printk_ratelimit())
 			dev_warn(&vb->pdev->dev, "Dropping tx buffer buffer\n");
-		kmem_cache_free(voicebus_vbb_cache, vbb);
+		dma_pool_free(vb->pool, vbb, vbb->dma_addr);
 		/* Schedule the underrun handler to run here, since we'll need
 		 * to cleanup as best we can. */
 		schedule_work(&vb->underrun_work);
@@ -740,11 +750,10 @@ static int __voicebus_transmit(struct voicebus *vb, struct vbb *vbb)
 	}
 
 	dl->pending[dl->tail] = vbb;
-	d->buffer1 = dma_map_single(&vb->pdev->dev, vbb->data,
-				    sizeof(vbb->data), DMA_TO_DEVICE);
+	d->buffer1 = cpu_to_le32(vbb->dma_addr);
 	dl->tail = (dl->tail + 1) & DRING_MASK;
 	SET_OWNED(d); /* That's it until the hardware is done with it. */
-	atomic_inc(&dl->count);
+	++dl->count;
 	return 0;
 }
 
@@ -776,7 +785,9 @@ static void setup_descriptors(struct voicebus *vb)
 {
 	int i;
 	struct vbb *vbb;
+	dma_addr_t dma_addr;
 	LIST_HEAD(buffers);
+	unsigned long flags;
 
 	might_sleep();
 
@@ -789,7 +800,10 @@ static void setup_descriptors(struct voicebus *vb)
 
 	for (i = 0; i < DRING_SIZE; ++i) {
 		if (list_empty(&vb->free_rx)) {
-			vbb = kmem_cache_alloc(voicebus_vbb_cache, GFP_KERNEL);
+			vbb = dma_pool_alloc(vb->pool, GFP_KERNEL,
+					     &dma_addr);
+			if (vbb)
+				vbb->dma_addr = dma_addr;
 		} else {
 			vbb = list_entry(vb->free_rx.next, struct vbb, entry);
 			list_del(&vbb->entry);
@@ -809,14 +823,18 @@ static void setup_descriptors(struct voicebus *vb)
 
 	if (BOOT != vb->mode) {
 		for (i = 0; i < vb->min_tx_buffer_count; ++i) {
-			vbb = kmem_cache_alloc(voicebus_vbb_cache, GFP_KERNEL);
+			vbb = dma_pool_alloc(vb->pool, GFP_KERNEL,
+					     &dma_addr);
 			if (unlikely(NULL == vbb))
 				BUG_ON(1);
-			else
-				list_add_tail(&vbb->entry, &buffers);
+
+			vbb->dma_addr = dma_addr;
+			list_add_tail(&vbb->entry, &buffers);
 		}
 
+		local_irq_save(flags);
 		handle_transmit(vb, &buffers);
+		local_irq_restore(flags);
 
 		vb_disable_deferred(vb);
 		while (!list_empty(&buffers)) {
@@ -882,10 +900,10 @@ static void
 dump_descriptor(struct voicebus *vb, struct voicebus_descriptor *d)
 {
 	VB_PRINTK(vb, DEBUG, "Displaying descriptor at address %08x\n", (unsigned int)d);
-	VB_PRINTK(vb, DEBUG, "   des0:      %08x\n", d->des0);
-	VB_PRINTK(vb, DEBUG, "   des1:      %08x\n", d->des1);
-	VB_PRINTK(vb, DEBUG, "   buffer1:   %08x\n", d->buffer1);
-	VB_PRINTK(vb, DEBUG, "   container: %08x\n", d->container);
+	VB_PRINTK(vb, DEBUG, "   des0:      %08x\n", le32_to_cpu(d->des0));
+	VB_PRINTK(vb, DEBUG, "   des1:      %08x\n", le32_to_cpu(d->des1));
+	VB_PRINTK(vb, DEBUG, "   buffer1:   %08x\n", le32_to_cpu(d->buffer1));
+	VB_PRINTK(vb, DEBUG, "   container: %08x\n", le32_to_cpu(d->container));
 }
 
 static void
@@ -928,15 +946,13 @@ vb_get_completed_txb(struct voicebus *vb)
 
 	d = vb_descriptor(dl, head);
 
-	if (OWNED(d) || !d->buffer1 || (d->buffer1 == vb->idle_vbb_dma_addr))
+	if (OWNED(d) || !d->buffer1 ||
+	    (le32_to_cpu(d->buffer1) == vb->idle_vbb_dma_addr))
 		return NULL;
 
-	dma_unmap_single(&vb->pdev->dev, d->buffer1,
-			 sizeof(vbb->data), DMA_TO_DEVICE);
-
 	vbb = dl->pending[head];
 	if (NORMAL == vb->mode) {
-		d->buffer1 = vb->idle_vbb_dma_addr;
+		d->buffer1 = cpu_to_le32(vb->idle_vbb_dma_addr);
 		dl->pending[head] = vb->idle_vbb;
 		SET_OWNED(d);
 	} else {
@@ -944,7 +960,7 @@ vb_get_completed_txb(struct voicebus *vb)
 		dl->pending[head] = NULL;
 	}
 	dl->head = (++head) & DRING_MASK;
-	atomic_dec(&dl->count);
+	--dl->count;
 	vb_net_capture_vbb(vb, vbb, 1, d->des0, d->container);
 	return vbb;
 }
@@ -962,12 +978,10 @@ vb_get_completed_rxb(struct voicebus *vb, u32 *des0)
 	if ((0 == d->buffer1) || OWNED(d))
 		return NULL;
 
-	dma_unmap_single(&vb->pdev->dev, d->buffer1,
-			 sizeof(vbb->data), DMA_FROM_DEVICE);
 	vbb = dl->pending[head];
 	dl->head = (++head) & DRING_MASK;
 	d->buffer1 = 0;
-	atomic_dec(&dl->count);
+	--dl->count;
 #	ifdef VOICEBUS_NET_DEBUG
 	vb_net_capture_vbb(vb, vbb, 0, d->des0, d->container);
 #	endif
@@ -1143,6 +1157,26 @@ void voicebus_stop(struct voicebus *vb)
 }
 EXPORT_SYMBOL(voicebus_stop);
 
+/**
+ * voicebus_quiesce - Halt the voicebus interface.
+ * @vb:	The voicebus structure to quiet
+ *
+ * This ensures that the device is not engaged in any DMA transactions or
+ * interrupting. It does not grab any locks since it may be called by a dying
+ * kernel.
+ */
+void voicebus_quiesce(struct voicebus *vb)
+{
+	if (!vb)
+		return;
+
+	/* Reset the device */
+	__vb_disable_interrupts(vb);
+	__vb_setctl(vb, 0x0000, 0x1);
+	__vb_getctl(vb, 0x0000);
+}
+EXPORT_SYMBOL(voicebus_quiesce);
+
 /*!
  * \brief Prepare the interface for module unload.
  *
@@ -1192,6 +1226,7 @@ voicebus_release(struct voicebus *vb)
 	pci_iounmap(vb->pdev, vb->iobase);
 	pci_clear_mwi(vb->pdev);
 	pci_disable_device(vb->pdev);
+	dma_pool_destroy(vb->pool);
 }
 EXPORT_SYMBOL(voicebus_release);
 
@@ -1228,10 +1263,13 @@ vb_increase_latency(struct voicebus *vb, unsigned int increase,
 	/* Set the minimum latency in case we're restarted...we don't want to
 	 * wait for the buffer to grow to this depth again in that case. */
 	for (i = 0; i < increase; ++i) {
-		vbb = kmem_cache_alloc(voicebus_vbb_cache, GFP_ATOMIC);
+		dma_addr_t dma_addr;
+		vbb = dma_pool_alloc(vb->pool, GFP_ATOMIC, &dma_addr);
 		WARN_ON(NULL == vbb);
-		if (likely(NULL != vbb))
+		if (likely(NULL != vbb)) {
+			vbb->dma_addr = dma_addr;
 			list_add_tail(&vbb->entry, &local);
+		}
 	}
 
 	handle_transmit(vb, &local);
@@ -1457,15 +1495,15 @@ static void vb_tasklet_normal(unsigned long data)
 	while ((vbb = vb_get_completed_txb(vb)))
 		list_add_tail(&vbb->entry, &vb->tx_complete);
 
-	if (unlikely(atomic_read(&dl->count) < 2)) {
+	if (unlikely(dl->count < 2)) {
 		softunderrun = 1;
 		d = vb_descriptor(dl, dl->head);
-		if (1 == atomic_read(&dl->count))
+		if (1 == dl->count)
 			return;
 
 		behind = 2;
 		while (!OWNED(d)) {
-			if (d->buffer1 != vb->idle_vbb_dma_addr)
+			if (le32_to_cpu(d->buffer1) != vb->idle_vbb_dma_addr)
 				goto tx_error_exit;
 			SET_OWNED(d);
 			dl->head = (dl->head + 1) & DRING_MASK;
@@ -1495,7 +1533,7 @@ static void vb_tasklet_normal(unsigned long data)
 		vb_increase_latency(vb, behind, &buffers);
 		d = vb_descriptor(dl, dl->head);
 		while (!OWNED(d)) {
-			if (d->buffer1 != vb->idle_vbb_dma_addr) {
+			if (le32_to_cpu(d->buffer1) != vb->idle_vbb_dma_addr) {
 				local_irq_restore(flags);
 				goto tx_error_exit;
 			}
@@ -1507,7 +1545,7 @@ static void vb_tasklet_normal(unsigned long data)
 		/* Now we'll get a little further ahead of the hardware. */
 		for (i = 0; i < 5; ++i) {
 			d = vb_descriptor(dl, dl->head);
-			d->buffer1 = vb->idle_vbb_dma_addr;
+			d->buffer1 = cpu_to_le32(vb->idle_vbb_dma_addr);
 			dl->pending[dl->head] = vb->idle_vbb;
 			d->des0 |= OWN_BIT;
 			dl->head = (dl->head + 1) & DRING_MASK;
@@ -1517,7 +1555,7 @@ static void vb_tasklet_normal(unsigned long data)
 	}
 
 	d = vb_descriptor(dl, dl->tail);
-	if (d->buffer1 != vb->idle_vbb_dma_addr)
+	if (le32_to_cpu(d->buffer1) != vb->idle_vbb_dma_addr)
 		goto tx_error_exit;
 
 	/* Now we can send all our buffers together in a group. */
@@ -1526,7 +1564,8 @@ static void vb_tasklet_normal(unsigned long data)
 		list_del(&vbb->entry);
 		__voicebus_transmit(vb, vbb);
 	}
-	__vb_setctl(vb, 0x0008, 0x00000000);
+
+	writel(0, vb->iobase + 0x8);
 
 	/* Print any messages about soft latency bumps after we fix the transmit
 	 * descriptor ring. Otherwise it's possible to take so much time
@@ -1583,7 +1622,7 @@ tx_error_exit:
 	while (!list_empty(&buffers)) {
 		vbb = list_entry(buffers.next, struct vbb, entry);
 		list_del(&vbb->entry);
-		kmem_cache_free(voicebus_vbb_cache, vbb);
+		dma_pool_free(vb->pool, vbb, vbb->dma_addr);
 	}
 	return;
 }
@@ -1644,6 +1683,7 @@ vb_isr(int irq, void *dev_id)
 #endif
 {
 	struct voicebus *vb = dev_id;
+	unsigned long flags;
 	u32 int_status;
 
 	int_status = __vb_getctl(vb, SR_CSR5);
@@ -1654,6 +1694,8 @@ vb_isr(int irq, void *dev_id)
 	if (!int_status)
 		return IRQ_NONE;
 
+	local_irq_save(flags);
+
 	if (unlikely((int_status &
 	    (TX_UNAVAILABLE_INTERRUPT|RX_UNAVAILABLE_INTERRUPT)) &&
 	    !test_bit(VOICEBUS_STOP, &vb->flags) &&
@@ -1694,6 +1736,7 @@ vb_isr(int irq, void *dev_id)
 		/* Clear the interrupt(s) */
 		__vb_setctl(vb, SR_CSR5, int_status);
 	}
+	local_irq_restore(flags);
 
 	return IRQ_HANDLED;
 }
@@ -1782,8 +1825,6 @@ __voicebus_init(struct voicebus *vb, const char *board_name,
 		goto cleanup;
 	}
 
-	/* \todo This driver should be modified to use the memory mapped I/O
-	   as opposed to IO space for portability and performance. */
 	if (0 == (pci_resource_flags(vb->pdev, 0)&IORESOURCE_IO)) {
 		dev_err(&vb->pdev->dev, "BAR0 is not IO Memory.\n");
 		retval = -EIO;
@@ -1803,6 +1844,13 @@ __voicebus_init(struct voicebus *vb, const char *board_name,
 		}
 	}
 
+	vb->pool = dma_pool_create(board_name, &vb->pdev->dev,
+				   sizeof(struct vbb), 64, 0);
+	if (!vb->pool) {
+		retval = -ENOMEM;
+		goto cleanup;
+	}
+
 	vb->idle_vbb = dma_alloc_coherent(&vb->pdev->dev, VOICEBUS_SFRAME_SIZE,
 					  &vb->idle_vbb_dma_addr, GFP_KERNEL);
 
@@ -1853,6 +1901,8 @@ cleanup:
 
 	tasklet_kill(&vb->tasklet);
 
+	if (vb->pool)
+		dma_pool_destroy(vb->pool);
 	/* Cleanup memory and software resources. */
 	if (vb->txd.desc)
 		vb_free_descriptors(vb, &vb->txd);
@@ -1976,54 +2026,17 @@ EXPORT_SYMBOL(vpmadtreg_unregister);
 
 static int __init voicebus_module_init(void)
 {
-	int res;
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
-	voicebus_vbb_cache = kmem_cache_create(THIS_MODULE->name,
-					 sizeof(struct vbb), 0,
-#if defined(CONFIG_SLUB) && (LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 22))
-					 SLAB_HWCACHE_ALIGN |
-					 SLAB_STORE_USER, NULL,
-					 NULL);
-#else
-					 SLAB_HWCACHE_ALIGN, NULL,
-					 NULL);
-#endif
-#else
-#if (defined(CONFIG_SLAB) && defined(CONFIG_SLAB_DEBUG)) || \
-    (defined(CONFIG_SLUB) && defined(CONFIG_SLUB_DEBUG))
-	voicebus_vbb_cache = kmem_cache_create(THIS_MODULE->name,
-					 sizeof(struct vbb), 0,
-					 SLAB_HWCACHE_ALIGN | SLAB_STORE_USER |
-					 SLAB_POISON | SLAB_DEBUG_FREE, NULL);
-#else
-	voicebus_vbb_cache = kmem_cache_create(THIS_MODULE->name,
-					 sizeof(struct vbb), 0,
-					 SLAB_HWCACHE_ALIGN, NULL);
-#endif
-#endif
-
-	if (NULL == voicebus_vbb_cache) {
-		printk(KERN_ERR "%s: Failed to allocate buffer cache.\n",
-		       THIS_MODULE->name);
-		return -ENOMEM;
-	}
-
 	/* This registration with dahdi.ko will fail since the span is not
 	 * defined, but it will make sure that this module is a dependency of
 	 * dahdi.ko, so that when it is being unloded, this module will be
 	 * unloaded as well. */
 	dahdi_register(NULL, 0);
 	spin_lock_init(&loader_list_lock);
-	res = vpmadt032_module_init();
-	if (res)
-		return res;
 	return 0;
 }
 
 static void __exit voicebus_module_cleanup(void)
 {
-	kmem_cache_destroy(voicebus_vbb_cache);
 	WARN_ON(!list_empty(&binary_loader_list));
 }
 
