@@ -32,11 +32,6 @@
 #include <linux/interrupt.h>
 
 
-#ifdef VOICEBUS_NET_DEBUG
-#include <linux/netdevice.h>
-#include <linux/etherdevice.h>
-#endif
-
 #define VOICEBUS_DEFAULT_LATENCY	3U
 #define VOICEBUS_DEFAULT_MAXLATENCY	25U
 #define VOICEBUS_MAXLATENCY_BUMP	6U
@@ -58,11 +53,36 @@
 /* Define this in order to create a debugging network interface. */
 #undef VOICEBUS_NET_DEBUG
 
+/* Define this to only run the processing in an interrupt handler
+ * (and not tasklet). */
+#define CONFIG_VOICEBUS_INTERRUPT
+
+/* Define this to use a FIFO for the software echocan reference.
+ * (experimental) */
+#undef CONFIG_VOICEBUS_ECREFERENCE
+
+#ifdef CONFIG_VOICEBUS_ECREFERENCE
+
+struct dahdi_fifo;
+unsigned int __dahdi_fifo_put(struct dahdi_fifo *fifo, u8 *data, size_t size);
+unsigned int __dahdi_fifo_get(struct dahdi_fifo *fifo, u8 *data, size_t size);
+void dahdi_fifo_free(struct dahdi_fifo *fifo);
+struct dahdi_fifo *dahdi_fifo_alloc(size_t maxsize, gfp_t alloc_flags);
+
+#endif
+
+#ifdef VOICEBUS_NET_DEBUG
+#include <linux/skbuff.h>
+#include <linux/netdevice.h>
+#include <linux/etherdevice.h>
+#endif
+
 struct voicebus;
 
 struct vbb {
 	u8 data[VOICEBUS_SFRAME_SIZE];
 	struct list_head entry;
+	dma_addr_t dma_addr;
 };
 
 struct voicebus_operations {
@@ -80,7 +100,7 @@ struct voicebus_descriptor_list {
 	unsigned int 	tail;
 	void  		*pending[DRING_SIZE];
 	dma_addr_t	desc_dma;
-	atomic_t 	count;
+	unsigned long	count;
 	unsigned int	padding;
 };
 
@@ -96,7 +116,7 @@ struct voicebus_descriptor_list {
  *
  * NORMAL:	For non-hx8 boards.  Uses idle_buffers.
  * BOOT:	For hx8 boards.  For sending single packets at a time.
- * RELAXED:	Normal operating mode for Hx8 Boards.  Does not use
+ * HX8:		Normal operating mode for Hx8 Boards.  Does not use
  *		idle_buffers.
  */
 enum voicebus_mode {
@@ -123,6 +143,10 @@ struct voicebus {
 	struct tasklet_struct 	tasklet;
 	enum voicebus_mode	mode;
 
+#if defined(CONFIG_VOICEBUS_INTERRUPT)
+	atomic_t		deferred_disabled_count;
+#endif
+
 #if defined(CONFIG_VOICEBUS_TIMER)
 	struct timer_list	timer;
 #endif
@@ -134,6 +158,7 @@ struct voicebus {
 	unsigned int		max_latency;
 	struct list_head	tx_complete;
 	struct list_head	free_rx;
+	struct dma_pool		*pool;
 
 #ifdef VOICEBUS_NET_DEBUG
 	struct sk_buff_head captured_packets;
@@ -147,17 +172,12 @@ struct voicebus {
 #endif
 };
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
-extern kmem_cache_t *voicebus_vbb_cache;
-#else
-extern struct kmem_cache *voicebus_vbb_cache;
-#endif
-
 int __voicebus_init(struct voicebus *vb, const char *board_name,
 		    enum voicebus_mode mode);
 void voicebus_release(struct voicebus *vb);
 int voicebus_start(struct voicebus *vb);
 void voicebus_stop(struct voicebus *vb);
+void voicebus_quiesce(struct voicebus *vb);
 int voicebus_transmit(struct voicebus *vb, struct vbb *vbb);
 int voicebus_set_minlatency(struct voicebus *vb, unsigned int milliseconds);
 int voicebus_current_latency(struct voicebus *vb);
@@ -216,10 +236,11 @@ static inline void voicebus_set_hx8_mode(struct voicebus *vb)
 static inline void
 voicebus_set_maxlatency(struct voicebus *vb, unsigned int max_latency)
 {
-	spin_lock_bh(&vb->lock);
+	unsigned long flags;
+	spin_lock_irqsave(&vb->lock, flags);
 	vb->max_latency = clamp(max_latency,
 				vb->min_tx_buffer_count,
 				VOICEBUS_DEFAULT_MAXLATENCY);
-	spin_unlock_bh(&vb->lock);
+	spin_unlock_irqrestore(&vb->lock, flags);
 }
 #endif /* __VOICEBUS_H__ */
