@@ -2,7 +2,7 @@
  * WCB410P  Quad-BRI PCI Driver
  * Written by Andrew Kohlsmith <akohlsmith@mixdown.ca>
  *
- * Copyright (C) 2009 Digium, Inc.
+ * Copyright (C) 2009-2011 Digium, Inc.
  * All rights reserved.
  *
  */
@@ -152,12 +152,11 @@ static const struct dahdi_echocan_features my_ec_features = {
 };
 
 static const struct dahdi_echocan_ops my_ec_ops = {
-	.name = "HWEC",
 	.echocan_free = echocan_free,
 };
 
 #if 0
-static const char *wcb4xxp_rcsdata = "$RCSfile: base.c,v $ $Revision: 9702 $";
+static const char *wcb4xxp_rcsdata = "$RCSfile: base.c,v $ $Revision: 10079 $";
 static const char *build_stamp = "" __DATE__ " " __TIME__ "";
 #endif
 
@@ -176,8 +175,10 @@ static inline unsigned char __pci_in8(struct b4xxp *b4, const unsigned int reg)
 	unsigned char ret = ioread8(b4->addr + reg);
 
 #ifdef DEBUG_LOWLEVEL_REGS
-	if (unlikely(DBG_REGS))
-		drv_dbg(b4->dev, "read 0x%02x from 0x%p\n", ret, b4->addr + reg);
+	if (unlikely(DBG_REGS)) {
+		drv_dbg(&b4->pdev->dev,
+			"read 0x%02x from 0x%p\n", ret, b4->addr + reg);
+	}
 #endif
 	if (unlikely(pedanticpci)) {
 		udelay(3);
@@ -191,8 +192,10 @@ static inline unsigned short __pci_in16(struct b4xxp *b4, const unsigned int reg
 	unsigned short ret = ioread16(b4->addr + reg);
 
 #ifdef DEBUG_LOWLEVEL_REGS
-	if (unlikely(DBG_REGS))
-		drv_dbg(b4->dev, "read 0x%04x from 0x%p\n", ret, b4->addr + reg);
+	if (unlikely(DBG_REGS)) {
+		drv_dbg(&b4->pdev->dev,
+			"read 0x%04x from 0x%p\n", ret, b4->addr + reg);
+	}
 #endif
 	if (unlikely(pedanticpci)) {
 		udelay(3);
@@ -206,8 +209,10 @@ static inline unsigned int __pci_in32(struct b4xxp *b4, const unsigned int reg)
 	unsigned int ret = ioread32(b4->addr + reg);
 
 #ifdef DEBUG_LOWLEVEL_REGS
-	if (unlikely(DBG_REGS))
-		drv_dbg(b4->dev, "read 0x%04x from 0x%p\n", ret, b4->addr + reg);
+	if (unlikely(DBG_REGS)) {
+		drv_dbg(&b4->pdev->dev,
+			"read 0x%04x from 0x%p\n", ret, b4->addr + reg);
+	}
 #endif
 	if (unlikely(pedanticpci)) {
 		udelay(3);
@@ -219,8 +224,10 @@ static inline unsigned int __pci_in32(struct b4xxp *b4, const unsigned int reg)
 static inline void __pci_out32(struct b4xxp *b4, const unsigned int reg, const unsigned int val)
 {
 #ifdef DEBUG_LOWLEVEL_REGS
-	if (unlikely(DBG_REGS))
-		drv_dbg(b4->dev, "writing 0x%02x to 0x%p\n", val, b4->addr + reg);
+	if (unlikely(DBG_REGS)) {
+		drv_dbg(&b4->pdev->dev,
+			"writing 0x%02x to 0x%p\n", val, b4->addr + reg);
+	}
 #endif
 	iowrite32(val, b4->addr + reg);
 
@@ -233,8 +240,10 @@ static inline void __pci_out32(struct b4xxp *b4, const unsigned int reg, const u
 static inline void __pci_out8(struct b4xxp *b4, const unsigned int reg, const unsigned char val)
 {
 #ifdef DEBUG_LOWLEVEL_REGS
-	if (unlikely(DBG_REGS)) 
-		drv_dbg(b4->dev, "writing 0x%02x to 0x%p\n", val, b4->addr + reg);
+	if (unlikely(DBG_REGS)) {
+		drv_dbg(&b4->pdev->dev,
+			"writing 0x%02x to 0x%p\n", val, b4->addr + reg);
+	}
 #endif
 	iowrite8(val, b4->addr + reg);
 
@@ -292,7 +301,8 @@ retry:
 
 #ifndef DEBUG_LOWLEVEL_REGS
 	if (unlikely(DBG_REGS)) {
-		dev_dbg(b4->dev, "read 0x%02x from 0x%p\n", ret, b4->addr + reg);
+		dev_dbg(&b4->pdev->dev,
+			"read 0x%02x from 0x%p\n", ret, b4->addr + reg);
 	}
 #endif
 	return ret;
@@ -309,7 +319,8 @@ static inline unsigned int b4xxp_getreg32(struct b4xxp *b4, const unsigned int r
 
 #ifndef DEBUG_LOWLEVEL_REGS
 	if (unlikely(DBG_REGS)) {
-		dev_dbg(b4->dev, "read 0x%04x from 0x%p\n", ret, b4->addr + reg);
+		dev_dbg(&b4->pdev->dev,
+			 "read 0x%04x from 0x%p\n", ret, b4->addr + reg);
 	}
 #endif
 	return ret;
@@ -326,7 +337,8 @@ static inline unsigned short b4xxp_getreg16(struct b4xxp *b4, const unsigned int
 
 #ifndef DEBUG_LOWLEVEL_REGS
 	if (unlikely(DBG_REGS)) {
-		dev_dbg(b4->dev, "read 0x%04x from 0x%p\n", ret, b4->addr + reg);
+		dev_dbg(&b4->pdev->dev,
+			"read 0x%04x from 0x%p\n", ret, b4->addr + reg);
 	}
 #endif
 	return ret;
@@ -338,7 +350,8 @@ static inline void b4xxp_setreg32(struct b4xxp *b4, const unsigned int reg, cons
 
 #ifndef DEBUG_LOWLEVEL_REGS
 	if (unlikely(DBG_REGS)) {
-		dev_dbg(b4->dev, "writing 0x%02x to 0x%p\n", val, b4->addr + reg);
+		dev_dbg(&b4->pdev->dev,
+			"writing 0x%02x to 0x%p\n", val, b4->addr + reg);
 	}
 #endif
 	spin_lock_irqsave(&b4->reglock, irq_flags);
@@ -352,7 +365,8 @@ static inline void b4xxp_setreg8(struct b4xxp *b4, const unsigned int reg, const
 
 #ifndef DEBUG_LOWLEVEL_REGS
 	if (unlikely(DBG_REGS)) {
-		dev_dbg(b4->dev, "writing 0x%02x to 0x%p\n", val, b4->addr + reg);
+		dev_dbg(&b4->pdev->dev,
+			"writing 0x%02x to 0x%p\n", val, b4->addr + reg);
 	}
 #endif
 	spin_lock_irqsave(&b4->reglock, irq_flags);
@@ -622,11 +636,11 @@ static inline void ec_write(struct b4xxp *b4, int which, unsigned short addr, un
 
 	in = ec_read(b4, which, addr);
 
-	if (in != data) {
-		if (printk_ratelimit()) {
-			dev_warn(b4->dev, "ec_write: Wrote 0x%02x to register 0x%02x "
-			         "of VPM %d but got back 0x%02x\n", data, addr, which, in);
-		}
+	if ((in != data) && printk_ratelimit()) {
+		dev_warn(&b4->pdev->dev,
+			 "ec_write: Wrote 0x%02x to register 0x%02x "
+			 "of VPM %d but got back 0x%02x\n",
+			 data, addr, which, in);
 	}
 }
 
@@ -659,7 +673,8 @@ static void ec_init(struct b4xxp *b4)
 	for (i=0; i < NUM_EC; i++) {
 		b = ec_read(b4, i, 0x1a0);
 
-		dev_info(b4->dev, "VPM %d/%d init: chip ver %02x\n", i, NUM_EC - 1, b);
+		dev_info(&b4->pdev->dev,
+			 "VPM %d/%d init: chip ver %02x\n", i, NUM_EC - 1, b);
 
 		for (j=0; j < b4->numspans; j++) {
 			ec_write(b4, i, 0x1a8 + j, 0x00);	/* GPIO out */
@@ -689,16 +704,16 @@ static void ec_init(struct b4xxp *b4)
 		b |= 0x12;
 		if (!strcasecmp(companding, "alaw")) {
 			if (DBG)
-				dev_info(b4->dev, "Setting alaw mode\n");
+				dev_info(&b4->pdev->dev, "Setting alaw mode\n");
 			b |= 0x01;
 		} else {
 			if (DBG)
-				dev_info(b4->dev, "Setting ulaw mode");
+				dev_info(&b4->pdev->dev, "Setting ulaw mode");
 		}
 
 		ec_write(b4, i, 0x20, b);
 		if (DBG)
-			dev_info(b4->dev, "reg 0x20 is 0x%02x\n", b);
+			dev_info(&b4->pdev->dev, "reg 0x20 is 0x%02x\n", b);
 
 //		ec_write(b4, i, 0x20, 0x38);
 
@@ -706,8 +721,10 @@ static void ec_init(struct b4xxp *b4)
 		ec_write(b4, i, 0x24, 0x02);
 		b = ec_read(b4, i, 0x24);
 #endif
-		if (DBG)
-			dev_info(b4->dev, "NLP threshold is set to %d (0x%02x)\n", b, b);
+		if (DBG) {
+			dev_info(&b4->pdev->dev,
+				 "NLP threshold is set to %d (0x%02x)\n", b, b);
+		}
 
 /* Initialize echo cans */
 		for (j=0; j < MAX_TDM_CHAN; j++) {
@@ -763,9 +780,10 @@ static void hfc_setreg_waitbusy(struct b4xxp *b4, const unsigned int reg, const
 		}
 	};
 
-	if (timeout) {
-		if (printk_ratelimit())
-			dev_warn(b4->dev, "hfc_setreg_waitbusy(write 0x%02x to 0x%02x) timed out waiting for busy flag to clear!\n", val, reg);
+	if (timeout && printk_ratelimit()) {
+		dev_warn(&b4->pdev->dev,
+			 "hfc_setreg_waitbusy(write 0x%02x to 0x%02x) timed "
+			 "out waiting for busy flag to clear!\n", val, reg);
 	}
 }
 
@@ -782,9 +800,10 @@ static inline unsigned char hfc_readcounter8(struct b4xxp *b4, const unsigned in
 		r2 = b4xxp_getreg8(b4, reg);
 	} while ((r1 != r2) && maxwait--);
 
-	if (!maxwait) {
-		if (printk_ratelimit())
-			dev_warn(b4->dev, "hfc_readcounter8(reg 0x%02x) timed out waiting for data to settle!\n", reg);
+	if (!maxwait && printk_ratelimit()) {
+		dev_warn(&b4->pdev->dev,
+			 "hfc_readcounter8(reg 0x%02x) timed out waiting "
+			 "for data to settle!\n", reg);
 	}
 
 	return r1;
@@ -803,9 +822,10 @@ static inline unsigned short hfc_readcounter16(struct b4xxp *b4, const unsigned
 		r2 = b4xxp_getreg16(b4, reg);
 	} while ((r1 != r2) && maxwait--);
 
-	if (!maxwait) {
-		if (printk_ratelimit())
-			dev_warn(b4->dev, "hfc_readcounter16(reg 0x%02x) timed out waiting for data to settle!\n", reg);
+	if (!maxwait && printk_ratelimit()) {
+		dev_warn(&b4->pdev->dev,
+			 "hfc_readcounter16(reg 0x%02x) timed out waiting "
+			 "for data to settle!\n", reg);
 	}
 
 	return r1;
@@ -821,9 +841,10 @@ static inline unsigned int hfc_readcounter32(struct b4xxp *b4, const unsigned in
 		r2 = b4xxp_getreg32(b4, reg);
 	} while ((r1 != r2) && maxwait--);
 
-	if (!maxwait) {
-		if (printk_ratelimit())
-			dev_warn(b4->dev, "hfc_readcounter32(reg 0x%02x) timed out waiting for data to settle!\n", reg);
+	if (!maxwait && printk_ratelimit()) {
+		dev_warn(&b4->pdev->dev,
+			 "hfc_readcounter32(reg 0x%02x) timed out waiting "
+			 "for data to settle!\n", reg);
 	}
 
 	return r1;
@@ -866,7 +887,7 @@ static void hfc_reset(struct b4xxp *b4)
 	while ((b = b4xxp_getreg8(b4, R_F0_CNTL)) < 2 && c) { udelay(100); c--; }
 
 	if (!c && b < 2) {
-		dev_warn(b4->dev, "hfc_reset() did not get the green light from the PCM system!\n");
+		dev_warn(&b4->pdev->dev, "hfc_reset() did not get the green light from the PCM system!\n");
 	}
 }
 
@@ -924,7 +945,7 @@ static void hfc_assign_bchan_fifo_ec(struct b4xxp *b4, int port, int bchan)
 
 	if (first) {
 		first = 0;
-		dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
+		dev_info(&b4->pdev->dev, "Hardware echo cancellation enabled.\n");
 	}
 
 	fifo = port * 2;
@@ -941,8 +962,12 @@ static void hfc_assign_bchan_fifo_ec(struct b4xxp *b4, int port, int bchan)
 	b4->spans[port].fifos[bchan] = fifo;
 	spin_lock_irqsave(&b4->fifolock, irq_flags);
 
-	if (DBG)
-		dev_info(b4->dev, "port %d, B channel %d\n\tS/T -> PCM ts %d uses HFC chan %d via FIFO %d\n", port, bchan, ts + 1, hfc_chan, 16 + fifo);
+	if (DBG) {
+		dev_info(&b4->pdev->dev,
+			 "port %d, B channel %d\n\tS/T -> PCM ts %d uses HFC "
+			 "chan %d via FIFO %d\n",
+			 port, bchan, ts + 1, hfc_chan, 16 + fifo);
+	}
 
 /* S/T RX -> PCM TX FIFO, transparent mode, no IRQ. */
 	hfc_setreg_waitbusy(b4, R_FIFO, ((16 + fifo) << V_FIFO_NUM_SHIFT));
@@ -1001,7 +1026,7 @@ static void hfc_assign_bchan_fifo_noec(struct b4xxp *b4, int port, int bchan)
 
 	if (first) {
 		first = 0;
-		dev_info(b4->dev, "NOTE: hardware echo cancellation has been disabled\n");
+		dev_info(&b4->pdev->dev, "NOTE: hardware echo cancellation has been disabled\n");
 	}
 
 	fifo = port * 2;
@@ -1018,8 +1043,12 @@ static void hfc_assign_bchan_fifo_noec(struct b4xxp *b4, int port, int bchan)
 	b4->spans[port].fifos[bchan] = fifo;
 	spin_lock_irqsave(&b4->fifolock, irq_flags);
 
-	if (DBG)
-		dev_info(b4->dev, "port %d, B channel %d\n\thost -> S/T uses HFC chan %d via FIFO %d\n", port, bchan, hfc_chan, fifo);
+	if (DBG) {
+		dev_info(&b4->pdev->dev,
+			 "port %d, B channel %d\n\thost -> S/T "
+			 "uses HFC chan %d via FIFO %d\n",
+			 port, bchan, hfc_chan, fifo);
+	}
 
 	hfc_setreg_waitbusy(b4, R_FIFO, (fifo << V_FIFO_NUM_SHIFT));
 	b4xxp_setreg8(b4, A_CON_HDLC, V_IFF | V_HDLC_TRP | V_DATA_FLOW_000);
@@ -1068,8 +1097,11 @@ static void hfc_assign_dchan_fifo(struct b4xxp *b4, int port)
 /* record the host's FIFO # in the span fifo array */
 	b4->spans[port].fifos[2] = fifo;
 
-	if (DBG)
-		dev_info(b4->dev, "port %d, D channel\n\thost -> S/T uses HFC chan %d via FIFO %d\n", port, hfc_chan, fifo);
+	if (DBG) {
+		dev_info(&b4->pdev->dev,
+			 "port %d, D channel\n\thost -> S/T uses HFC chan %d "
+			 "via FIFO %d\n", port, hfc_chan, fifo);
+	}
 
 	spin_lock_irqsave(&b4->fifolock, irq_flags);
 
@@ -1150,11 +1182,11 @@ static int b4xxp_find_sync(struct b4xxp *b4)
 
 	for (i=0; i < b4->numspans; i++) {
 		if (DBG)
-			dev_info(b4->dev, "Checking sync pos %d, have span %d\n", i, b4->spans[i].sync);
+			dev_info(&b4->pdev->dev, "Checking sync pos %d, have span %d\n", i, b4->spans[i].sync);
 		psrc = b4->spans[i].sync;
 		if (psrc > 0 && !b4->spans[psrc - 1].span.alarms) {
 			if (DBG)
-				dev_info(b4->dev, "chosen\n");
+				dev_info(&b4->pdev->dev, "chosen\n");
 			src = psrc;
 			break;
 		}
@@ -1180,17 +1212,17 @@ static DEVICE_ATTR(timing_master, 0400, b4_timing_master_show, NULL);
 static void create_sysfs_files(struct b4xxp *b4)
 {
 	int ret;
-	ret = device_create_file(b4->dev,
+	ret = device_create_file(&b4->pdev->dev,
 				 &dev_attr_timing_master);
 	if (ret) {
-		dev_info(b4->dev,
+		dev_info(&b4->pdev->dev,
 			"Failed to create device attributes.\n");
 	}
 }
 
 static void remove_sysfs_files(struct b4xxp *b4)
 {
-	device_remove_file(b4->dev,
+	device_remove_file(&b4->pdev->dev,
 			   &dev_attr_timing_master);
 }
 
@@ -1219,7 +1251,7 @@ static char *hfc_decode_st_state(struct b4xxp *b4, int port, unsigned char state
 	};
 
 	if (!(str = kmalloc(256, GFP_KERNEL))) {
-		dev_warn(b4->dev, "could not allocate mem for ST state decode string!\n");
+		dev_warn(&b4->pdev->dev, "could not allocate mem for ST state decode string!\n");
 		return NULL;
 	}
 
@@ -1261,7 +1293,9 @@ static void hfc_force_st_state(struct b4xxp *b4, int port, int state, int resume
 		char *x;
 
 		x = hfc_decode_st_state(b4, port, state, 1);
-		dev_info(b4->dev, "forced port %d to state %d (auto: %d), new decode: %s\n", port + 1, state, resume_auto, x);
+		dev_info(&b4->pdev->dev,
+			 "forced port %d to state %d (auto: %d), "
+			 "new decode: %s\n", port + 1, state, resume_auto, x);
 		kfree(x);
 	}
 
@@ -1274,8 +1308,13 @@ static void hfc_timer_expire(struct b4xxp_span *s, int t_no)
 {
 	struct b4xxp *b4 = s->parent;
 
-	if (DBG_ST)
-		dev_info(b4->dev, "%lu: hfc_timer_expire, Port %d T%d expired (value=%lu ena=%d)\n", b4->ticks, s->port + 1, t_no + 1, s->hfc_timers[t_no], s->hfc_timer_on[t_no]);
+	if (DBG_ST) {
+		dev_info(&b4->pdev->dev,
+			 "%lu: hfc_timer_expire, Port %d T%d expired "
+			 "(value=%lu ena=%d)\n",
+			 b4->ticks, s->port + 1, t_no + 1, s->hfc_timers[t_no],
+			 s->hfc_timer_on[t_no]);
+	}
 /*
  * there are three timers associated with every HFC S/T port.
  * T1 is used by the NT state machine, and is the maximum time the NT side should wait for G3 (active) state.
@@ -1297,8 +1336,11 @@ static void hfc_timer_expire(struct b4xxp_span *s, int t_no)
 		hfc_force_st_state(b4, s->port, 3, 1);
 		break;
 	default:
-		if (printk_ratelimit())
-			dev_warn(b4->dev, "hfc_timer_expire found an unknown expired timer (%d)??\n", t_no);
+		if (printk_ratelimit()) {
+			dev_warn(&b4->pdev->dev,
+				 "hfc_timer_expire found an unknown expired "
+				 "timer (%d)??\n", t_no);
+		}
 	}
 }
 
@@ -1332,7 +1374,7 @@ static void hfc_update_st_timers(struct b4xxp *b4)
 			}
 			b4xxp_set_sync_src(b4, b4xxp_find_sync(b4));
 			if (DBG_ALARM) {
-				dev_info(b4->dev,
+				dev_info(&b4->pdev->dev,
 					 "span %d: alarm %d "
 					 "debounced\n",
 					 i + 1, s->newalarm);
@@ -1359,7 +1401,9 @@ static void hfc_handle_state(struct b4xxp_span *s)
 		char *x;
 
 		x = hfc_decode_st_state(b4, s->port, state, 1);
-		dev_info(b4->dev, "port %d A_ST_RD_STA old=0x%02x now=0x%02x, decoded: %s\n", s->port + 1, s->oldstate, state, x);
+		dev_info(&b4->pdev->dev,
+			 "port %d A_ST_RD_STA old=0x%02x now=0x%02x, "
+			 "decoded: %s\n", s->port + 1, s->oldstate, state, x);
 		kfree(x);
 	}
 
@@ -1407,7 +1451,7 @@ static void hfc_handle_state(struct b4xxp_span *s)
 	s->oldstate = state;
 
 	if (DBG_ALARM) {
-		dev_info(b4->dev, "span %d: old alarm %d expires %ld, new alarm %d expires %ld\n",
+		dev_info(&b4->pdev->dev, "span %d: old alarm %d expires %ld, new alarm %d expires %ld\n",
 			s->port + 1, oldalarm, oldtimer, s->newalarm, s->alarmtimer);
 	}
 
@@ -1422,7 +1466,9 @@ static void hfc_handle_state(struct b4xxp_span *s)
 		s->hfc_timers[HFC_T3] = b4->ticks + timer_3_ms;
 		s->hfc_timer_on[HFC_T3] = 1;
 		if (DBG_ST) {
-			dev_info(b4->dev, "port %d: receiving INFO0 in state 3, setting T3 and jumping to F4\n", s->port + 1);
+			dev_info(&b4->pdev->dev,
+				 "port %d: receiving INFO0 in state 3, "
+				 "setting T3 and jumping to F4\n", s->port + 1);
 		}
 		hfc_force_st_state(b4, s->port, 4, 1);
 	}
@@ -1430,8 +1476,11 @@ static void hfc_handle_state(struct b4xxp_span *s)
 /* read in R_BERT_STA to determine where our current sync source is */
 	newsync = b4xxp_getreg8(b4, R_BERT_STA) & 0x07;
 	if (newsync != b4->syncspan) {
-		if (printk_ratelimit() || DBG)
-			dev_info(b4->dev, "new card sync source: port %d\n", newsync + 1);
+		if (printk_ratelimit() || DBG) {
+			dev_info(&b4->pdev->dev,
+				 "new card sync source: port %d\n",
+				 newsync + 1);
+		}
 		b4->syncspan = newsync;
 	}
 }
@@ -1483,14 +1532,20 @@ static void hfc_start_st(struct b4xxp_span *s)
 		s->hfc_timers[HFC_T3] = b4->ticks + 500;	/* 500ms wait first time, timer_t3_ms afterward. */
 		s->hfc_timer_on[HFC_T3] = 1;
 		s->hfc_timer_on[HFC_T1] = 0;
-		if (DBG_ST)
-			dev_info(b4->dev, "setting port %d t3 timer to %lu\n", s->port + 1, s->hfc_timers[HFC_T3]);
+		if (DBG_ST) {
+			dev_info(&b4->pdev->dev,
+				 "setting port %d t3 timer to %lu\n",
+				 s->port + 1, s->hfc_timers[HFC_T3]);
+		}
 	} else {
 		s->hfc_timers[HFC_T1] = b4->ticks + timer_1_ms;
 		s->hfc_timer_on[HFC_T1] = 1;
 		s->hfc_timer_on[HFC_T3] = 0;
-		if (DBG_ST)
-			dev_info(b4->dev, "setting port %d t1 timer to %lu\n", s->port + 1, s->hfc_timers[HFC_T1]);
+		if (DBG_ST) {
+			dev_info(&b4->pdev->dev,
+				 "setting port %d t1 timer to %lu\n",
+				 s->port + 1, s->hfc_timers[HFC_T1]);
+		}
 	}
 }
 
@@ -1534,7 +1589,8 @@ static void hfc_init_all_st(struct b4xxp *b4)
 
 		s->te_mode = !nt;
 
-		dev_info(b4->dev, "Port %d: %s mode\n", i + 1, (nt ? "NT" : "TE"));
+		dev_info(&b4->pdev->dev,
+			 "Port %d: %s mode\n", i + 1, (nt ? "NT" : "TE"));
 
 		hfc_reset_st(s);
 		hfc_start_st(s);
@@ -1690,7 +1746,7 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 /* first check to make sure we really do have HDLC frames available to retrieve */
 	if (flen == 0) {
 		if (DBG_HDLC && DBG_SPANFILTER) {
-			dev_info(b4->dev, "hdlc_rx_frame(span %d): no frames available?\n",
+			dev_info(&b4->pdev->dev, "hdlc_rx_frame(span %d): no frames available?\n",
 				bspan->port + 1);
 		}
 
@@ -1721,7 +1777,7 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 
 		zleft -= j;
 		if (DBG_HDLC && DBG_SPANFILTER) {
-			dev_info(b4->dev, "hdlc_rx_frame(span %d): z1/z2/zlen=%d/%d/%d, zleft=%d\n",
+			dev_info(&b4->pdev->dev, "hdlc_rx_frame(span %d): z1/z2/zlen=%d/%d/%d, zleft=%d\n",
 				bspan->port + 1, z1, z2, zlen, zleft);
 			for (i=0; i < j; i++) printk("%02x%c", buf[i], (i < ( j - 1)) ? ' ':'\n');
 		}
@@ -1742,14 +1798,18 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 	++bspan->frames_in;
 	if (zlen < 3) {
 		if (DBG_HDLC && DBG_SPANFILTER)
-			dev_notice(b4->dev, "odd, zlen less then 3?\n");
+			dev_notice(&b4->pdev->dev, "odd, zlen less then 3?\n");
 		dahdi_hdlc_abort(bspan->sigchan, DAHDI_EVENT_ABORT);
 	} else {
 
 /* if STAT != 0, indicates bad frame */
 		if (stat != 0x00) {
-			if (DBG_HDLC && DBG_SPANFILTER)
-				dev_info(b4->dev, "(span %d) STAT=0x%02x indicates frame problem: ", bspan->port + 1, stat);
+			if (DBG_HDLC && DBG_SPANFILTER) {
+				dev_info(&b4->pdev->dev,
+					 "(span %d) STAT=0x%02x indicates "
+					 "frame problem: ",
+					 bspan->port + 1, stat);
+			}
 			if (stat == 0xff) {
 				if (DBG_HDLC && DBG_SPANFILTER)
 					printk("HDLC Abort\n");
@@ -1761,8 +1821,11 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 			}
 /* STAT == 0, means frame was OK */
 		} else {
-			if (DBG_HDLC && DBG_SPANFILTER)
-				dev_info(b4->dev, "(span %d) Frame %d is good!\n", bspan->port + 1, bspan->frames_in);
+			if (DBG_HDLC && DBG_SPANFILTER) {
+				dev_info(&b4->pdev->dev,
+					 "(span %d) Frame %d is good!\n",
+					 bspan->port + 1, bspan->frames_in);
+			}
 			dahdi_hdlc_finish(bspan->sigchan);
 		}
 	}
@@ -1833,8 +1896,8 @@ static int hdlc_tx_frame(struct b4xxp_span *bspan)
 	spin_unlock_irqrestore(&b4->fifolock, irq_flags);
 
 	if (DBG_HDLC && DBG_SPANFILTER) {
-		dev_info(b4->dev, "%s", debugbuf);
-		dev_info(b4->dev, "hdlc_tx_frame(span %d): DAHDI gave %d bytes for FIFO %d (res=%d)\n",
+		dev_info(&b4->pdev->dev, "%s", debugbuf);
+		dev_info(&b4->pdev->dev, "hdlc_tx_frame(span %d): DAHDI gave %d bytes for FIFO %d (res=%d)\n",
 			bspan->port + 1, size, fifo, res);
 		for (i=0; i < size; i++)
 			printk("%02x%c", buf[i], (i < (size - 1)) ? ' ' : '\n');
@@ -1907,8 +1970,6 @@ static void b4xxp_init_stage1(struct b4xxp *b4)
 	flush_pci();
 
 	udelay(100);				/* wait a bit for clock to settle */
-
-	create_sysfs_files(b4);
 }
 
 /*
@@ -2140,6 +2201,15 @@ static void b4xxp_update_leds(struct b4xxp *b4)
 	}
 }
 
+static const char *b4xxp_echocan_name(const struct dahdi_chan *chan)
+{
+	struct b4xxp_span *bspan = container_of(chan->span, struct b4xxp_span,
+						span);
+	if (bspan->parent->card_type == B410P)
+		return "LASVEGAS2";
+	return NULL;
+}
+
 static int b4xxp_echocan_create(struct dahdi_chan *chan,
 				struct dahdi_echocanparams *ecp,
 				struct dahdi_echocanparam *p,
@@ -2203,7 +2273,7 @@ static int b4xxp_ioctl(struct dahdi_chan *chan, unsigned int cmd, unsigned long
 	return 0;
 }
 
-static int b4xxp_startup(struct dahdi_span *span)
+static int b4xxp_startup(struct file *file, struct dahdi_span *span)
 {
 	struct b4xxp_span *bspan = container_of(span, struct b4xxp_span, span);
 	struct b4xxp *b4 = bspan->parent;
@@ -2236,24 +2306,28 @@ static void b4xxp_reset_span(struct b4xxp_span *bspan)
 }
 
 /* spanconfig for us means to set up the HFC FIFO and channel mapping */
-static int b4xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
+static int b4xxp_spanconfig(struct file *file, struct dahdi_span *span,
+			    struct dahdi_lineconfig *lc)
 {
 	int i;
 	struct b4xxp_span *bspan = container_of(span, struct b4xxp_span, span);
 	struct b4xxp *b4 = bspan->parent;
 
 	if (DBG)
-		dev_info(b4->dev, "Configuring span %d offset %d to be sync %d\n", span->spanno, span->offset, lc->sync);
+		dev_info(&b4->pdev->dev, "Configuring span %d offset %d to be sync %d\n", span->spanno, span->offset, lc->sync);
 
 #if 0
 	if (lc->sync > 0 && !bspan->te_mode) {
-		dev_info(b4->dev, "Span %d is not in NT mode, removing from sync source list\n", span->spanno);
+		dev_info(&b4->pdev->dev, "Span %d is not in NT mode, removing "
+			 "from sync source list\n", span->spanno);
 		lc->sync = 0;
 	}
 #endif
 
 	if (lc->sync < 0 || lc->sync > 4) {
-		dev_info(b4->dev, "Span %d has invalid sync priority (%d), removing from sync source list\n", span->spanno, lc->sync);
+		dev_info(&b4->pdev->dev,
+			 "Span %d has invalid sync priority (%d), removing "
+			 "from sync source list\n", span->spanno, lc->sync);
 		lc->sync = 0;
 	}
 
@@ -2264,16 +2338,13 @@ static int b4xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc
 		}
 	}
 
-	/* if a sync src, put it in proper place */
-	b4->spans[span->offset].syncpos = lc->sync;
-	if (lc->sync) {
+	if (lc->sync)
 		b4->spans[lc->sync - 1].sync = (span->offset + 1);
-	}
 
 	b4xxp_reset_span(bspan);
 
 /* call startup() manually here, because DAHDI won't call the startup function unless it receives an IOCTL to do so, and dahdi_cfg doesn't. */
-	b4xxp_startup(&bspan->span);
+	b4xxp_startup(file, &bspan->span);
 
 	span->flags |= DAHDI_FLAG_RUNNING;
 
@@ -2281,7 +2352,8 @@ static int b4xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc
 }
 
 /* chanconfig for us means to configure the HDLC controller, if appropriate */
-static int b4xxp_chanconfig(struct dahdi_chan *chan, int sigtype)
+static int
+b4xxp_chanconfig(struct file *file, struct dahdi_chan *chan, int sigtype)
 {
 	int alreadyrunning;
 	struct b4xxp *b4 = chan->pvt;
@@ -2291,24 +2363,27 @@ static int b4xxp_chanconfig(struct dahdi_chan *chan, int sigtype)
 	alreadyrunning = bspan->span.flags & DAHDI_FLAG_RUNNING;
 
 	if (DBG_FOPS) {
-		dev_info(b4->dev, "%s channel %d (%s) sigtype %08x\n",
+		dev_info(&b4->pdev->dev, "%s channel %d (%s) sigtype %08x\n",
 			alreadyrunning ? "Reconfigured" : "Configured", chan->channo, chan->name, sigtype);
 	}
 
 	/* (re)configure signalling channel */
 	if ((sigtype == DAHDI_SIG_HARDHDLC) || (bspan->sigchan == chan)) {
 		if (DBG_FOPS)
-			dev_info(b4->dev, "%sonfiguring hardware HDLC on %s\n",
+			dev_info(&b4->pdev->dev, "%sonfiguring hardware HDLC on %s\n",
 				((sigtype == DAHDI_SIG_HARDHDLC) ? "C" : "Unc"), chan->name);
 
 		if (alreadyrunning && bspan->sigchan) {
 			hdlc_stop(b4, fifo);
+			atomic_set(&bspan->hdlc_pending, 0);
+			bspan->sigactive = 0;
+			smp_mb();
 			bspan->sigchan = NULL;
 		}
 
 		if (sigtype == DAHDI_SIG_HARDHDLC) {
 			if (hdlc_start(b4, fifo)) {
-				dev_warn(b4->dev, "Error initializing signalling controller\n");
+				dev_warn(&b4->pdev->dev, "Error initializing signalling controller\n");
 				return -1;
 			}
 		}
@@ -2328,8 +2403,10 @@ static int b4xxp_open(struct dahdi_chan *chan)
 	struct b4xxp *b4 = chan->pvt;
 	struct b4xxp_span *bspan = &b4->spans[chan->span->offset];
 
-	if (DBG_FOPS && DBG_SPANFILTER)
-		dev_info(b4->dev, "open() on chan %s (%i/%i)\n", chan->name, chan->channo, chan->chanpos);
+	if (DBG_FOPS && DBG_SPANFILTER) {
+		dev_info(&b4->pdev->dev, "open() on chan %s (%i/%i)\n",
+			 chan->name, chan->channo, chan->chanpos);
+	}
 
 	hfc_reset_fifo_pair(b4, bspan->fifos[chan->chanpos - 1], 0, 0);
 	return 0;
@@ -2341,8 +2418,10 @@ static int b4xxp_close(struct dahdi_chan *chan)
 	struct b4xxp *b4 = chan->pvt;
 	struct b4xxp_span *bspan = &b4->spans[chan->span->offset];
 
-	if (DBG_FOPS && DBG_SPANFILTER)
-		dev_info(b4->dev, "close() on chan %s (%i/%i)\n", chan->name, chan->channo, chan->chanpos);
+	if (DBG_FOPS && DBG_SPANFILTER) {
+		dev_info(&b4->pdev->dev, "close() on chan %s (%i/%i)\n",
+			 chan->name, chan->channo, chan->chanpos);
+	}
 
 	hfc_reset_fifo_pair(b4, bspan->fifos[chan->chanpos - 1], 1, 1);
 	return 0;
@@ -2356,7 +2435,7 @@ static void b4xxp_hdlc_hard_xmit(struct dahdi_chan *chan)
 	struct b4xxp_span *bspan = &b4->spans[span];
 
 	if ((DBG_FOPS || DBG_HDLC) && DBG_SPANFILTER)
-		dev_info(b4->dev, "hdlc_hard_xmit on chan %s (%i/%i), span=%i\n",
+		dev_info(&b4->pdev->dev, "hdlc_hard_xmit on chan %s (%i/%i), span=%i\n",
 			chan->name, chan->channo, chan->chanpos, span + 1);
 
 /*
@@ -2381,6 +2460,7 @@ static const struct dahdi_span_ops b4xxp_span_ops = {
 	.ioctl = b4xxp_ioctl,
 	.hdlc_hard_xmit = b4xxp_hdlc_hard_xmit,
 	.echocan_create = b4xxp_echocan_create,
+	.echocan_name = b4xxp_echocan_name,
 };
 
 /* initialize the span/chan structures. Doesn't touch hardware, although the callbacks might. */
@@ -2415,7 +2495,8 @@ static void init_spans(struct b4xxp *b4)
 		sprintf(bspan->span.name, "B4/%d/%d", b4->cardno, i+1);
 		sprintf(bspan->span.desc, "B4XXP (PCI) Card %d Span %d", b4->cardno, i+1);
 		bspan->span.manufacturer = "Digium";
-		dahdi_copy_string(bspan->span.devicetype, b4->variety, sizeof(bspan->span.devicetype));
+		strlcpy(bspan->span.devicetype, b4->variety,
+			sizeof(bspan->span.devicetype));
 		sprintf(bspan->span.location, "PCI Bus %02d Slot %02d",
 			b4->pdev->bus->number, PCI_SLOT(b4->pdev->devfn) + 1);
 
@@ -2425,7 +2506,6 @@ static void init_spans(struct b4xxp *b4)
 		bspan->sigactive = 0;
 
 		bspan->span.chans = bspan->chans;
-		init_waitqueue_head(&bspan->span.maintq);
 
 /* now initialize each channel in the span */
 		for (j=0; j < WCB4XXP_CHANNELS_PER_SPAN; j++) {
@@ -2524,13 +2604,12 @@ DAHDI_IRQ_HANDLER(b4xxp_interrupt)
 static void b4xxp_bottom_half(unsigned long data)
 {
 	struct b4xxp *b4 = (struct b4xxp *)data;
-	int i, j, k, gotrxfifo, fifo, fifo_low, fifo_high;
+	int i, j, k, fifo, fifo_low, fifo_high;
 	unsigned char b, b2;
 
 	if (b4->shutdown)
 		return;
 
-	gotrxfifo = 0;
 	/* HFC-4S d-chan fifos 8-11 *** HFC-8S d-chan fifos 16-23 */
 	if (b4->numspans == 8) {
 		fifo_low = 16;
@@ -2561,7 +2640,7 @@ static void b4xxp_bottom_half(unsigned long data)
 					}  while (k);
 				} else {
 					if (printk_ratelimit())
-						dev_warn(b4->dev, "Got FIFO TX int from non-d-chan FIFO %d??\n", fifo);
+						dev_warn(&b4->pdev->dev, "Got FIFO TX int from non-d-chan FIFO %d??\n", fifo);
 				}
 			}
 
@@ -2578,7 +2657,7 @@ static void b4xxp_bottom_half(unsigned long data)
 					} while (k);
 				} else {
 					if (printk_ratelimit())
-						dev_warn(b4->dev, "Got FIFO RX int from non-d-chan FIFO %d??\n", fifo);
+						dev_warn(&b4->pdev->dev, "Got FIFO RX int from non-d-chan FIFO %d??\n", fifo);
 				}
 			}
 
@@ -2653,7 +2732,8 @@ static int b4xxp_proc_read_one(char *buf, struct b4xxp *b4)
 	char str[80], sBuf[4096];
 
 	*sBuf=0;
-	sprintf(sBuf, "Card %d, PCI identifier %s, IRQ %d\n", b4->cardno + 1, b4->dev->bus_id, b4->irq);
+	sprintf(sBuf, "Card %d, PCI identifier %s, IRQ %d\n",
+		b4->cardno + 1, b4->pdev->dev.bus_id, b4->irq);
 
 	strcat(sBuf,"Tx:\n");
 	for (j=0; j<(b4->numspans * 2) ; j++) {			/* B Channels */
@@ -2741,7 +2821,7 @@ static int b4xxp_proc_read(char *buf, char **start, off_t offset, int count, int
 static int b4xxp_startdefaultspan(struct b4xxp *b4)
 {
 	struct dahdi_lineconfig lc = {0,};
-	return b4xxp_spanconfig(&b4->spans[0].span, &lc);
+	return b4xxp_spanconfig(NULL, &b4->spans[0].span, &lc);
 }
 
 static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
@@ -2777,7 +2857,6 @@ static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id
 	b4->variety = dt->desc;
 	b4->card_type = dt->card_type;
 	b4->pdev = pdev;
-	b4->dev = &pdev->dev;
 	pci_set_drvdata(pdev, b4);
 
 	b4->ioaddr = pci_iomap(pdev, 0, 0);
@@ -2807,11 +2886,13 @@ static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id
 	b4->numspans = dt->ports;
 	b4->syncspan = -1;		/* sync span is unknown */
 	if (b4->numspans > MAX_SPANS_PER_CARD) {
-		dev_err(b4->dev, "Driver does not know how to handle a %d span card!\n", b4->numspans);
+		dev_err(&b4->pdev->dev,
+			"Driver does not know how to handle a %d span card!\n",
+			b4->numspans);
 		goto err_out_free_mem;
 	}
 
-	dev_info(b4->dev, "Identified %s (controller rev %d) at %p, IRQ %i\n",
+	dev_info(&b4->pdev->dev, "Identified %s (controller rev %d) at %p, IRQ %i\n",
 		b4->variety, b4->chiprev, b4->ioaddr, b4->irq);
 
 /* look for the next free card structure */
@@ -2831,8 +2912,11 @@ static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id
 
 	b4xxp_init_stage1(b4);
 
+	create_sysfs_files(b4);
+
 	if (request_irq(pdev->irq, b4xxp_interrupt, DAHDI_IRQ_SHARED_DISABLED, "b4xxp", b4)) {
-		dev_err(b4->dev, "Unable to request IRQ %d\n", pdev->irq);
+		dev_err(&b4->pdev->dev, "Unable to request IRQ %d\n",
+			pdev->irq);
 		ret = -EIO;
 		goto err_out_del_from_card_array;
 	}
@@ -2849,7 +2933,9 @@ static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id
 	init_spans(b4);
 	for (x=0; x < b4->numspans; x++) {
 		if (dahdi_register(&b4->spans[x].span, 0)) {
-			dev_err(b4->dev, "Unable to register span %s\n", b4->spans[x].span.name);
+			dev_err(&b4->pdev->dev,
+				"Unable to register span %s\n",
+				b4->spans[x].span.name);
 			goto err_out_unreg_spans;
 		}
 	}
@@ -2875,7 +2961,7 @@ static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id
 			break;
 	}
 #else
-	dev_info(b4->dev, "Did not do the highestorder stuff\n");
+	dev_info(&b4->pdev->dev, "Did not do the highestorder stuff\n");
 #endif
 
 	ret = b4xxp_startdefaultspan(b4);
@@ -2970,6 +3056,7 @@ static DEFINE_PCI_DEVICE_TABLE(b4xx_ids) =
 	{ 0x1397, 0x08b4, 0x1397, 0xb761, 0, 0, (unsigned long)&hfc2s_BN },
 	{ 0x1397, 0x08b4, 0x1397, 0xb560, 0, 0, (unsigned long)&hfc4s_BN },
 	{ 0x1397, 0x08b4, 0x1397, 0xb550, 0, 0, (unsigned long)&hfc4s_BN },
+	{ 0x1397, 0x08b4, 0x1397, 0xb762, 0, 0, (unsigned long)&hfc4s_BN },
 	{ 0x1397, 0x16b8, 0x1397, 0xb562, 0, 0, (unsigned long)&hfc8s_BN },
 	{ 0x1397, 0x16b8, 0x1397, 0xb56b, 0, 0, (unsigned long)&hfc8s_BN },
 	{ 0x1397, 0x08b4, 0x1397, 0xb540, 0, 0, (unsigned long)&hfc4s_SW },
@@ -2978,11 +3065,17 @@ static DEFINE_PCI_DEVICE_TABLE(b4xx_ids) =
 
 };
 
+static int b4xx_suspend(struct pci_dev *pdev, pm_message_t state)
+{
+	return -ENOSYS;
+}
+
 static struct pci_driver b4xx_driver = {
 	.name = "wcb4xxp",
 	.probe = b4xx_probe,
 	.remove = __devexit_p(b4xxp_remove),
 	.id_table = b4xx_ids,
+	.suspend = b4xx_suspend,
 };
 
 static int __init b4xx_init(void)
@@ -3010,7 +3103,7 @@ static void __exit b4xx_exit(void)
 
 module_param(debug, int, S_IRUGO | S_IWUSR);
 module_param(spanfilter, int, S_IRUGO | S_IWUSR);
-#ifdef LOOKBACK_SUPPORTED
+#ifdef LOOPBACK_SUPPORTED
 module_param(loopback, int, S_IRUGO | S_IWUSR);
 #endif
 module_param(milliwatt, int, S_IRUGO | S_IWUSR);
@@ -3024,7 +3117,7 @@ module_param(companding, charp, S_IRUGO);
 
 MODULE_PARM_DESC(debug, "bitmap: 1=general 2=dtmf 4=regops 8=fops 16=ec 32=st state 64=hdlc 128=alarm");
 MODULE_PARM_DESC(spanfilter, "debug filter for spans. bitmap: 1=port 1, 2=port 2, 4=port 3, 8=port 4");
-#ifdef LOOKBACK_SUPPORTED
+#ifdef LOOPBACK_SUPPORTED
 MODULE_PARM_DESC(loopback, "TODO: bitmap: 1=loop back S/T port 2=loop back DAHDI");
 #endif
 MODULE_PARM_DESC(milliwatt, "1=replace outgoing S/T data with mu-law milliwatt");
