@@ -390,7 +390,6 @@ struct b4xxp_span {
 	unsigned char readchunk[WCB4XXP_CHANNELS_PER_SPAN * DAHDI_CHUNKSIZE];
 
 	int sync;				/* sync priority */
-	int syncpos;				/* sync priority */
 
 	int oldstate;				/* old state machine state */
 	int newalarm;				/* alarm to send to zaptel once alarm timer expires */
@@ -435,7 +434,6 @@ struct b4xxp {
 	int chiprev;				/* revision of HFC-4S */
 
 	struct pci_dev *pdev;			/* Pointer to PCI device */
-	struct device *dev;			/* kernel dev struct (from pdev->dev) */
 	void __iomem *addr;			/* I/O address (memory mapped) */
 	void __iomem *ioaddr;			/* I/O address (index based) */
 	int irq;				/* IRQ used by device */
