@@ -68,14 +68,9 @@
 
 #define WARN_ALWAYS() WARN_ON(1)
 
-#define DTE_PRINTK(_lvl, _fmt, _args...) \
-	printk(KERN_##_lvl "%s: %s: " _fmt, THIS_MODULE->name, \
-		(wc)->board_name, ## _args)
-
 #define DTE_DEBUG(_dbgmask, _fmt, _args...)				\
 	if ((debug & _dbgmask) == (_dbgmask)) {				\
-		printk(KERN_DEBUG "%s: %s: " _fmt, THIS_MODULE->name,	\
-			(wc)->board_name, ## _args);			\
+		dev_info(&(wc)->pdev->dev, _fmt, ## _args);		\
 	}								\
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
@@ -92,9 +87,9 @@
 #endif
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 14)
 /* also added in RHEL kernels with the OpenInfiniband backport: */
-#if LINUX_VERSION_CODE != KERNEL_VERSION(2,6,9) || !defined(DEFINE_SPINLOCK)
+#if LINUX_VERSION_CODE != KERNEL_VERSION(2, 6, 9) || !defined(DEFINE_SPINLOCK)
 typedef	unsigned gfp_t;		/* Added in 2.6.14 */
 #endif
 #endif
@@ -399,22 +394,16 @@ struct wcdte {
 #endif
 };
 
-#ifdef HAVE_NETDEV_PRIV
 struct wcdte_netdev_priv {
 	struct wcdte *wc;
 };
-#endif
 
 static inline struct wcdte *
 wcdte_from_netdev(struct net_device *netdev)
 {
-#ifdef HAVE_NETDEV_PRIV
 	struct wcdte_netdev_priv *priv;
 	priv = netdev_priv(netdev);
 	return priv->wc;
-#else
-	return netdev->priv;
-#endif
 }
 
 
@@ -490,7 +479,7 @@ wctc4xxp_skb_to_cmd(struct wcdte *wc, const struct sk_buff *skb)
 		cmd->data_len = skb->len;
 		res = skb_copy_bits(skb, 0, cmd->data, cmd->data_len);
 		if (res) {
-			DTE_PRINTK(WARNING,
+			dev_warn(&wc->pdev->dev,
 			   "Failed call to skb_copy_bits.\n");
 			free_cmd(cmd);
 			cmd = NULL;
@@ -626,19 +615,19 @@ wctc4xxp_net_waitfor_promiscuous(struct wcdte *wc)
 	unsigned long start = jiffies;
 	struct net_device *netdev = wc->netdev;
 
-	DTE_PRINTK(INFO,
+	dev_info(&wc->pdev->dev,
 	   "Waiting %d seconds for adapter to be placed in " \
 	   "promiscuous mode for early trace.\n", seconds);
 
 	while (!netdev->promiscuity) {
 		if (signal_pending(current)) {
-			DTE_PRINTK(INFO,
+			dev_info(&wc->pdev->dev,
 			   "Aborting wait due to signal.\n");
 			break;
 		}
 		msleep(100);
 		if (time_after(jiffies, start + (seconds * HZ))) {
-			DTE_PRINTK(INFO,
+			dev_info(&wc->pdev->dev,
 			   "Aborting wait due to timeout.\n");
 			break;
 		}
@@ -691,23 +680,14 @@ wctc4xxp_net_register(struct wcdte *wc)
 {
 	int res;
 	struct net_device *netdev;
-#	ifdef HAVE_NETDEV_PRIV
 	struct wcdte_netdev_priv *priv;
-#	endif
 	const char our_mac[] = { 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff};
 
-#	ifdef HAVE_NETDEV_PRIV
 	netdev = alloc_netdev(sizeof(*priv), wc->board_name, ether_setup);
 	if (!netdev)
 		return -ENOMEM;
 	priv = netdev_priv(netdev);
 	priv->wc = wc;
-#	else
-	netdev = alloc_netdev(0, wc->board_name, ether_setup);
-	if (!netdev)
-		return -ENOMEM;
-	netdev->priv = wc;
-#	endif
 	memcpy(netdev->dev_addr, our_mac, sizeof(our_mac));
 
 #	ifdef HAVE_NET_DEVICE_OPS
@@ -733,7 +713,7 @@ wctc4xxp_net_register(struct wcdte *wc)
 
 	res = register_netdev(netdev);
 	if (res) {
-		DTE_PRINTK(WARNING,
+		dev_warn(&wc->pdev->dev,
 		   "Failed to register network device %s.\n",
 		   wc->board_name);
 		goto error_sw;
@@ -745,7 +725,7 @@ wctc4xxp_net_register(struct wcdte *wc)
 	if (debug & DTE_DEBUG_NETWORK_EARLY)
 		wctc4xxp_net_waitfor_promiscuous(wc);
 
-	DTE_PRINTK(DEBUG,
+	dev_info(&wc->pdev->dev,
 	   "Created network device %s for debug.\n", wc->board_name);
 	return 0;
 
@@ -937,8 +917,8 @@ wctc4xxp_submit(struct wctc4xxp_descriptor_ring *dr, struct tcb *c)
 	}
 	d->des1 &= cpu_to_le32(~(BUFFER1_SIZE_MASK));
 	d->des1 |= cpu_to_le32(len & BUFFER1_SIZE_MASK);
-	d->buffer1 = pci_map_single(dr->pdev, c->data,
-			SFRAME_SIZE, dr->direction);
+	d->buffer1 = cpu_to_le32(pci_map_single(dr->pdev, c->data,
+			SFRAME_SIZE, dr->direction));
 
 	SET_OWNED(d); /* That's it until the hardware is done with it. */
 	dr->pending[dr->tail] = c;
@@ -958,7 +938,7 @@ wctc4xxp_retrieve(struct wctc4xxp_descriptor_ring *dr)
 	spin_lock_irqsave(&dr->lock, flags);
 	d = wctc4xxp_descriptor(dr, head);
 	if (d->buffer1 && !OWNED(d)) {
-		pci_unmap_single(dr->pdev, d->buffer1,
+		pci_unmap_single(dr->pdev, le32_to_cpu(d->buffer1),
 			SFRAME_SIZE, dr->direction);
 		c = dr->pending[head];
 		WARN_ON(!c);
@@ -966,7 +946,7 @@ wctc4xxp_retrieve(struct wctc4xxp_descriptor_ring *dr)
 		d->buffer1 = 0;
 		--dr->count;
 		WARN_ON(!c);
-		c->data_len = (d->des0 >> 16) & BUFFER1_SIZE_MASK;
+		c->data_len = (le32_to_cpu(d->des0) >> 16) & BUFFER1_SIZE_MASK;
 		WARN_ON(c->data_len > SFRAME_SIZE);
 	} else {
 		c = NULL;
@@ -1144,9 +1124,12 @@ send_create_channel_cmd(struct wcdte *wc, struct tcb *cmd, u16 timeslot,
 		return res;
 
 	if (0x0000 != response_header(cmd)->params[0]) {
-		/* The DTE failed to create the channel. */
-		/* TODO put some debug information here. */
-		WARN_ON(1);
+		if (printk_ratelimit()) {
+			dev_warn(&wc->pdev->dev,
+				 "Failed to create channel in timeslot " \
+				 "%d.  Response from DTE was (%04x).\n",
+				 timeslot, response_header(cmd)->params[0]);
+		}
 		free_cmd(cmd->response);
 		cmd->response = NULL;
 		return -EIO;
@@ -1298,7 +1281,8 @@ send_destroy_channel_cmd(struct wcdte *wc, struct tcb *cmd, u16 channel)
 	/* Let's check the response for any error codes.... */
 	result = le16_to_cpu(response_header(cmd)->params[0]);
 	if (0x0000 != result) {
-		DTE_PRINTK(ERR, "Failed to destroy channel %04d (%04x)\n",
+		dev_err(&wc->pdev->dev,
+			"Failed to destroy channel %04d (%04x)\n",
 			channel, result);
 		return -EIO;
 	}
@@ -1313,8 +1297,8 @@ send_set_ip_hdr_channel_cmd(struct channel_pvt *pvt, struct tcb *cmd)
 	struct wcdte *wc = pvt->wc;
 	const u16 parameters[] = {0, 0x0045, 0, 0, 0x0040, 0x1180, 0,
 		0xa8c0, 0x0309, 0xa8c0, 0x0309,
-		cpu_to_be16(pvt->timeslot_out_num + 0x5000),
-		cpu_to_be16(pvt->timeslot_in_num + 0x5000),
+		swab16(pvt->timeslot_out_num + 0x5000),
+		swab16(pvt->timeslot_in_num + 0x5000),
 		0, 0};
 	create_channel_cmd(pvt, cmd, CONFIG_CHANGE_TYPE, CONFIG_CHANNEL_CLASS,
 		0x9000, parameters, ARRAY_SIZE(parameters));
@@ -1324,7 +1308,7 @@ send_set_ip_hdr_channel_cmd(struct channel_pvt *pvt, struct tcb *cmd)
 	/* Let's check the response for any error codes.... */
 	result = le16_to_cpu(response_header(cmd)->params[0]);
 	if (0x0000 != result) {
-		DTE_PRINTK(ERR, "Failure in %s (%04x)\n",
+		dev_err(&wc->pdev->dev, "Failure in %s (%04x)\n",
 			__func__, result);
 		return -EIO;
 	}
@@ -1346,7 +1330,7 @@ send_voip_vceopt_cmd(struct channel_pvt *pvt, struct tcb *cmd, u16 length)
 	/* Let's check the response for any error codes.... */
 	result = le16_to_cpu(response_header(cmd)->params[0]);
 	if (0x0000 != result) {
-		DTE_PRINTK(ERR, "Failure in %s (%04x)\n",
+		dev_err(&wc->pdev->dev, "Failure in %s (%04x)\n",
 			__func__, result);
 		return -EIO;
 	}
@@ -1368,7 +1352,7 @@ send_voip_tonectl_cmd(struct channel_pvt *pvt, struct tcb *cmd)
 	/* Let's check the response for any error codes.... */
 	result = le16_to_cpu(response_header(cmd)->params[0]);
 	if (0x0000 != result) {
-		DTE_PRINTK(ERR, "Failure in %s (%04x)\n",
+		dev_err(&wc->pdev->dev, "Failure in %s (%04x)\n",
 			__func__, result);
 		return -EIO;
 	}
@@ -1615,6 +1599,20 @@ wctc4xxp_transmit_cmd(struct wcdte *wc, struct tcb *cmd)
 {
 	int res;
 
+	/* If we're shutdown all commands will timeout. Just complete the
+	 * command here with the timeout flag */
+	if (unlikely(test_bit(DTE_SHUTDOWN, &wc->flags))) {
+		if (cmd->flags & DO_NOT_AUTO_FREE) {
+			cmd->flags |= DTE_CMD_TIMEOUT;
+			list_del_init(&cmd->node);
+			complete(&cmd->complete);
+		} else {
+			list_del(&cmd->node);
+			free_cmd(cmd);
+		}
+		return;
+	}
+
 	if (cmd->data_len < MIN_PACKET_LEN) {
 		memset((u8 *)(cmd->data) + cmd->data_len, 0,
 		       MIN_PACKET_LEN-cmd->data_len);
@@ -1913,10 +1911,10 @@ wctc4xxp_operation_release(struct dahdi_transcoder_channel *dtc)
 
 	packets_received = atomic_read(&cpvt->stats.packets_received);
 	packets_sent = atomic_read(&cpvt->stats.packets_sent);
-		
+
 	if ((packets_sent - packets_received) > 5) {
-		DTE_DEBUG(DTE_DEBUG_GENERAL, "%s channel %d sent %d packets " 
-			"and received %d packets.\n", (cpvt->encoder) ? 
+		DTE_DEBUG(DTE_DEBUG_GENERAL, "%s channel %d sent %d packets "
+			"and received %d packets.\n", (cpvt->encoder) ?
 			"encoder" : "decoder", cpvt->chan_out_num,
 			packets_sent, packets_received);
 	}
@@ -2009,10 +2007,11 @@ wctc4xxp_handle_receive_ring(struct wcdte *wc)
 		spin_unlock(&wc->rx_list_lock);
 		cmd = __alloc_cmd(SFRAME_SIZE, GFP_ATOMIC, 0);
 		if (!cmd) {
-			DTE_PRINTK(ERR, "Out of memory in %s.\n", __func__);
+			dev_err(&wc->pdev->dev,
+				"Out of memory in %s.\n", __func__);
 		} else {
 			if (wctc4xxp_submit(wc->rxd, cmd)) {
-				DTE_PRINTK(ERR, "Failed submit in %s\n",
+				dev_err(&wc->pdev->dev, "Failed submit in %s\n",
 					__func__);
 				free_cmd(cmd);
 			}
@@ -2096,7 +2095,7 @@ wctc4xxp_read(struct file *file, char __user *frame, size_t count, loff_t *ppos)
 			}
 
 			if (printk_ratelimit()) {
-				DTE_PRINTK(ERR,
+				dev_err(&wc->pdev->dev,
 				  "Cannot copy %zd bytes into %zd byte user " \
 				  "buffer.\n", payload_bytes, count);
 			}
@@ -2109,7 +2108,7 @@ wctc4xxp_read(struct file *file, char __user *frame, size_t count, loff_t *ppos)
 		ret = copy_to_user(&frame[returned_bytes],
 				   &packet->payload[0], payload_bytes);
 		if (unlikely(ret)) {
-			DTE_PRINTK(ERR, "Failed to copy data in %s\n",
+			dev_err(&wc->pdev->dev, "Failed to copy data in %s\n",
 				   __func__);
 			free_cmd(cmd);
 			return -EFAULT;
@@ -2181,7 +2180,8 @@ wctc4xxp_write(struct file *file, const char __user *frame,
 	/* Copy the data directly from user space into the command buffer. */
 	if (copy_from_user(&((struct rtp_packet *)(cmd->data))->payload[0],
 		frame, count)) {
-		DTE_PRINTK(ERR, "Failed to copy packet from userspace.\n");
+		dev_err(&wc->pdev->dev,
+			"Failed to copy packet from userspace.\n");
 		free_cmd(cmd);
 		return -EFAULT;
 	}
@@ -2343,7 +2343,7 @@ print_command(struct wcdte *wc, const struct tcb *cmd)
 
 	buffer = kzalloc(BUFFER_SIZE + 1, GFP_ATOMIC);
 	if (!buffer) {
-		DTE_PRINTK(DEBUG, "Failed print_command\n");
+		dev_info(&wc->pdev->dev, "Failed print_command\n");
 		return;
 	}
 	curlength = snprintf(buffer, BUFFER_SIZE,
@@ -2360,7 +2360,7 @@ print_command(struct wcdte *wc, const struct tcb *cmd)
 			BUFFER_SIZE - curlength, " %04x",
 			le16_to_cpu(hdr->params[i]));
 	}
-	DTE_PRINTK(DEBUG, "%s\n", buffer);
+	dev_info(&wc->pdev->dev, "%s\n", buffer);
 	kfree(buffer);
 }
 
@@ -2385,7 +2385,7 @@ receive_csm_encaps_packet(struct wcdte *wc, struct tcb *cmd)
 			}
 
 			if (0x75 == hdr->class) {
-				DTE_PRINTK(WARNING,
+				dev_warn(&wc->pdev->dev,
 				   "Received alert (0x%04x) from dsp\n",
 				   le16_to_cpu(hdr->params[0]));
 			}
@@ -2396,12 +2396,12 @@ receive_csm_encaps_packet(struct wcdte *wc, struct tcb *cmd)
 						  hdr->channel);
 			}
 			if (hdr->params[0] != le16_to_cpu(0xffff)) {
-				DTE_PRINTK(WARNING,
+				dev_warn(&wc->pdev->dev,
 				   "DTE Failed self test (%04x).\n",
 				   le16_to_cpu(hdr->params[0]));
 			} else if ((hdr->params[1] != le16_to_cpu(0x000c)) &&
 				(hdr->params[1] != le16_to_cpu(0x010c))) {
-				DTE_PRINTK(WARNING,
+				dev_warn(&wc->pdev->dev,
 				   "Unexpected ERAM status (%04x).\n",
 				   le16_to_cpu(hdr->params[1]));
 			} else {
@@ -2414,7 +2414,8 @@ receive_csm_encaps_packet(struct wcdte *wc, struct tcb *cmd)
 				wctc4xxp_send_ack(wc, hdr->seq_num,
 						  hdr->channel);
 			}
-			DTE_PRINTK(WARNING, "Received diagnostic message:\n");
+			dev_warn(&wc->pdev->dev,
+				 "Received diagnostic message:\n");
 			print_command(wc, cmd);
 			free_cmd(cmd);
 		} else {
@@ -2422,7 +2423,7 @@ receive_csm_encaps_packet(struct wcdte *wc, struct tcb *cmd)
 				wctc4xxp_send_ack(wc, hdr->seq_num,
 						  hdr->channel);
 			}
-			DTE_PRINTK(WARNING,
+			dev_warn(&wc->pdev->dev,
 			  "Unknown command type received. %02x\n", hdr->type);
 			free_cmd(cmd);
 		}
@@ -2452,7 +2453,7 @@ queue_rtp_packet(struct wcdte *wc, struct tcb *cmd)
 
 	index = (be16_to_cpu(packet->udphdr.dest) - 0x5000) / 2;
 	if (unlikely(!(index < wc->numchannels))) {
-		DTE_PRINTK(ERR,
+		dev_err(&wc->pdev->dev,
 		  "Invalid channel number in response from DTE.\n");
 		free_cmd(cmd);
 		return;
@@ -2468,7 +2469,7 @@ queue_rtp_packet(struct wcdte *wc, struct tcb *cmd)
 		dtc = &(wc->uencode->channels[index]);
 		break;
 	default:
-		DTE_PRINTK(ERR, "Unknown codec in packet (0x%02x).\n",\
+		dev_err(&wc->pdev->dev, "Unknown codec in packet (0x%02x).\n",\
 			packet->rtphdr.type);
 		free_cmd(cmd);
 		return;
@@ -2500,7 +2501,7 @@ wctc4xxp_receiveprep(struct wcdte *wc, struct tcb *cmd)
 	}
 }
 
-static inline void service_tx_ring(struct wcdte *wc)
+static void service_tx_ring(struct wcdte *wc)
 {
 	struct tcb *cmd;
 	unsigned long flags;
@@ -2541,7 +2542,7 @@ static inline void service_tx_ring(struct wcdte *wc)
 	}
 }
 
-static inline void service_rx_ring(struct wcdte *wc)
+static void service_rx_ring(struct wcdte *wc)
 {
 	struct tcb *cmd;
 	unsigned long flags;
@@ -2563,13 +2564,6 @@ static inline void service_rx_ring(struct wcdte *wc)
 	wctc4xxp_receive_demand_poll(wc);
 }
 
-static inline void service_dte(struct wcdte *wc)
-{
-	service_tx_ring(wc);
-	service_rx_ring(wc);
-}
-
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
 static void deferred_work_func(void *param)
 {
@@ -2579,7 +2573,8 @@ static void deferred_work_func(struct work_struct *work)
 {
 	struct wcdte *wc = container_of(work, struct wcdte, deferred_work);
 #endif
-	service_dte(wc);
+	service_tx_ring(wc);
+	service_rx_ring(wc);
 }
 
 DAHDI_IRQ_HANDLER(wctc4xxp_interrupt)
@@ -2621,27 +2616,30 @@ DAHDI_IRQ_HANDLER(wctc4xxp_interrupt)
 		__wctc4xxp_setctl(wc, 0x0028, reg);
 	} else {
 		if ((ints & 0x00008000) && debug)
-			DTE_PRINTK(INFO, "Abnormal Interrupt.\n");
+			dev_info(&wc->pdev->dev, "Abnormal Interrupt.\n");
 
 		if ((ints & 0x00002000) && debug)
-			DTE_PRINTK(INFO, "Fatal Bus Error INT\n");
+			dev_info(&wc->pdev->dev, "Fatal Bus Error INT\n");
 
 		if ((ints & 0x00000100) && debug)
-			DTE_PRINTK(INFO, "Receive Stopped INT\n");
+			dev_info(&wc->pdev->dev, "Receive Stopped INT\n");
 
 		if ((ints & 0x00000080) && debug) {
-			DTE_PRINTK(INFO, "Receive Desciptor Unavailable INT " \
-			    "(%d)\n", wctc4xxp_getcount(wc->rxd));
+			dev_info(&wc->pdev->dev,
+				 "Receive Desciptor Unavailable INT " \
+				 "(%d)\n", wctc4xxp_getcount(wc->rxd));
 		}
 
 		if ((ints & 0x00000020) && debug)
-			DTE_PRINTK(INFO, "Transmit Under-flow INT\n");
+			dev_info(&wc->pdev->dev, "Transmit Under-flow INT\n");
 
 		if ((ints & 0x00000008) && debug)
-			DTE_PRINTK(INFO, "Jabber Timer Time-out INT\n");
+			dev_info(&wc->pdev->dev, "Jabber Timer Time-out INT\n");
 
-		if ((ints & 0x00000002) && debug)
-			DTE_PRINTK(INFO, "Transmit Processor Stopped INT\n");
+		if ((ints & 0x00000002) && debug) {
+			dev_info(&wc->pdev->dev,
+				 "Transmit Processor Stopped INT\n");
+		}
 
 		/* Clear all the pending interrupts. */
 		__wctc4xxp_setctl(wc, 0x0028, ints);
@@ -2741,11 +2739,10 @@ wctc4xxp_start_dma(struct wcdte *wc)
 }
 
 static void
-wctc4xxp_stop_dma(struct wcdte *wc)
+_wctc4xxp_stop_dma(struct wcdte *wc)
 {
 	/* Disable interrupts and reset */
 	unsigned int reg;
-	unsigned long newjiffies;
 	/* Disable interrupts */
 	wctc4xxp_setintmask(wc, 0x00000000);
 	wctc4xxp_setctl(wc, 0x0084, 0x00000000);
@@ -2754,7 +2751,14 @@ wctc4xxp_stop_dma(struct wcdte *wc)
 	reg = wctc4xxp_getctl(wc, 0x0000);
 	reg |= 0x00000001;
 	wctc4xxp_setctl(wc, 0x0000, reg);
+}
 
+static void
+wctc4xxp_stop_dma(struct wcdte *wc)
+{
+	unsigned long newjiffies;
+
+	_wctc4xxp_stop_dma(wc);
 	newjiffies = jiffies + HZ; /* One second timeout */
 	/* We'll wait here for the part to come out of reset */
 	while (((wctc4xxp_getctl(wc, 0x0000)) & 0x00000001) &&
@@ -2762,7 +2766,6 @@ wctc4xxp_stop_dma(struct wcdte *wc)
 			msleep(1);
 }
 
-
 #define MDIO_SHIFT_CLK		0x10000
 #define MDIO_DATA_WRITE1 	0x20000
 #define MDIO_ENB		0x00000
@@ -2857,7 +2860,8 @@ wctc4xxp_wait_for_link(struct wcdte *wc)
 		delay_count++;
 
 		if (delay_count >= 5000) {
-			DTE_PRINTK(ERR, "Failed to link to DTE processor!\n");
+			dev_err(&wc->pdev->dev,
+				"Failed to link to DTE processor!\n");
 			return -EIO;
 		}
 	} while ((reg & 0xE0000000) != 0xE0000000);
@@ -2868,7 +2872,6 @@ static int
 wctc4xxp_load_firmware(struct wcdte *wc, const struct firmware *firmware)
 {
 	unsigned int byteloc;
-	unsigned int last_byteloc;
 	unsigned int length;
 	struct tcb *cmd;
 
@@ -2883,7 +2886,6 @@ wctc4xxp_load_firmware(struct wcdte *wc, const struct firmware *firmware)
 #endif
 
 	while (byteloc < (firmware->size-20)) {
-		last_byteloc = byteloc;
 		length = (firmware->data[byteloc] << 8) |
 				firmware->data[byteloc+1];
 		byteloc += 2;
@@ -2896,7 +2898,7 @@ wctc4xxp_load_firmware(struct wcdte *wc, const struct firmware *firmware)
 		wait_for_completion(&cmd->complete);
 		if (cmd->flags & DTE_CMD_TIMEOUT) {
 			free_cmd(cmd);
-			DTE_PRINTK(ERR, "Failed to load firmware.\n");
+			dev_err(&wc->pdev->dev, "Failed to load firmware.\n");
 #if defined(CONFIG_WCTC4XXP_POLLING)
 			wctc4xxp_disable_polling(wc);
 #endif
@@ -2905,7 +2907,7 @@ wctc4xxp_load_firmware(struct wcdte *wc, const struct firmware *firmware)
 	}
 	free_cmd(cmd);
 	if (!wait_event_timeout(wc->waitq, wctc4xxp_is_ready(wc), 15*HZ)) {
-		DTE_PRINTK(ERR, "Failed to boot firmware.\n");
+		dev_err(&wc->pdev->dev, "Failed to boot firmware.\n");
 #if defined(CONFIG_WCTC4XXP_POLLING)
 		wctc4xxp_disable_polling(wc);
 #endif
@@ -3018,10 +3020,6 @@ wctc4xxp_create_channel_pair(struct wcdte *wc, struct channel_pvt *cpvt,
 		complicated = temp;
 	}
 
-	DTE_DEBUG(DTE_DEBUG_CHANNEL_SETUP,
-	   "DTE is using the following channels encoder_channel: " \
-	   "%d decoder_channel: %d\n", encoder_channel, decoder_channel);
-
 	BUG_ON(encoder_timeslot/2 >= wc->numchannels);
 	BUG_ON(decoder_timeslot/2 >= wc->numchannels);
 	encoder_pvt = wc->uencode->channels[encoder_timeslot/2].pvt;
@@ -3040,6 +3038,10 @@ wctc4xxp_create_channel_pair(struct wcdte *wc, struct channel_pvt *cpvt,
 		&decoder_channel))
 		goto error_exit;
 
+	DTE_DEBUG(DTE_DEBUG_CHANNEL_SETUP,
+	   "DTE is using the following channels encoder_channel: " \
+	   "%d decoder_channel: %d\n", encoder_channel, decoder_channel);
+
 	length = (DTE_FORMAT_G729A == complicated) ? G729_LENGTH :
 		(DTE_FORMAT_G723_1 == complicated) ? G723_LENGTH : 0;
 
@@ -3100,7 +3102,7 @@ wctc4xxp_destroy_channel_pair(struct wcdte *wc, struct channel_pvt *cpvt)
 	}
 
 	if (timeslot1/2 >= wc->numchannels || timeslot2/2 >= wc->numchannels) {
-		DTE_PRINTK(WARNING,
+		dev_warn(&wc->pdev->dev,
 		 "Invalid channel numbers in %s. chan1:%d chan2: %d\n",
 		 __func__, timeslot1/2, timeslot2/2);
 		return 0;
@@ -3307,13 +3309,20 @@ wctc4xxp_watchdog(unsigned long data)
 		if (time_after(jiffies, cmd->timeout)) {
 			if (++cmd->retries > MAX_RETRIES) {
 				if (!(cmd->flags & TX_COMPLETE)) {
+
+					cmd->flags |= DTE_CMD_TIMEOUT;
+					list_del_init(&cmd->node);
+					complete(&cmd->complete);
+
 					set_bit(DTE_SHUTDOWN, &wc->flags);
 					spin_unlock(&wc->cmd_list_lock);
-					wctc4xxp_stop_dma(wc);
-					DTE_PRINTK(ERR,
+					_wctc4xxp_stop_dma(wc);
+					dev_err(&wc->pdev->dev,
 					  "Board malfunctioning.  " \
 					  "Halting operation.\n");
-					return;
+					reschedule_timer = 0;
+					spin_lock(&wc->cmd_list_lock);
+					break;
 				}
 				/* ERROR:  We've retried the command and
 				 * haven't received the ACK or the response.
@@ -3333,7 +3342,7 @@ wctc4xxp_watchdog(unsigned long data)
 				 * off any lists, lets just reset the timeout
 				 * and tell the hardware to look for another
 				 * command . */
-				DTE_PRINTK(WARNING,
+				dev_warn(&wc->pdev->dev,
 				  "Retrying command that was " \
 				  "still on descriptor list.\n");
 				cmd->timeout = jiffies + HZ/4;
@@ -3456,8 +3465,6 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 #else
 	INIT_WORK(&wc->deferred_work, deferred_work_func);
 #endif
-	DTE_PRINTK(INFO, "Attached to device at %s.\n", pci_name(wc->pdev));
-
 	init_waitqueue_head(&wc->waitq);
 
 	if (pci_set_dma_mask(wc->pdev, DMA_BIT_MASK(32))) {
@@ -3465,7 +3472,7 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 			pci_resource_len(wc->pdev, 1));
 		if (wc->iobase)
 			pci_iounmap(wc->pdev, wc->iobase);
-		DTE_PRINTK(WARNING, "No suitable DMA available.\n");
+		dev_warn(&wc->pdev->dev, "No suitable DMA available.\n");
 		return -EIO;
 	}
 
@@ -3494,7 +3501,7 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 #if defined(HOTPLUG_FIRMWARE)
 	res = request_firmware(&firmware, tc400m_firmware, &wc->pdev->dev);
 	if (res || !firmware) {
-		DTE_PRINTK(ERR,
+		dev_err(&wc->pdev->dev,
 		  "Firmware %s not available from userspace. (%d)\n",
 		  tc400m_firmware, res);
 		goto error_exit_swinit;
@@ -3564,7 +3571,7 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 
 	res = pci_enable_device(pdev);
 	if (res) {
-		DTE_PRINTK(ERR, "Failed to enable device.\n");
+		dev_err(&wc->pdev->dev, "Failed to enable device.\n");
 		goto error_exit_swinit;;
 	}
 	pci_set_master(pdev);
@@ -3572,7 +3579,8 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 	res = request_irq(pdev->irq, wctc4xxp_interrupt,
 		DAHDI_IRQ_SHARED, wc->board_name, wc);
 	if (res) {
-		DTE_PRINTK(ERR, "Unable to request IRQ %d\n", pdev->irq);
+		dev_err(&wc->pdev->dev,
+			"Unable to request IRQ %d\n", pdev->irq);
 		if (firmware != &embedded_firmware)
 			release_firmware(firmware);
 		goto error_exit_hwinit;
@@ -3599,7 +3607,7 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 		goto error_exit_hwinit;
 
 	/* \todo Read firmware version directly from tc400b.*/
-	DTE_PRINTK(INFO, "(%s) Transcoder support LOADED " \
+	dev_info(&wc->pdev->dev, "(%s) Transcoder support LOADED " \
 	   "(firm ver = %d.%d)\n", wc->complexname, wctc4xxp_firmware_ver,
 	   wctc4xxp_firmware_ver_minor);
 
@@ -3608,7 +3616,7 @@ wctc4xxp_init_one(struct pci_dev *pdev, const struct pci_device_id *ent)
 	DTE_DEBUG(DTE_DEBUG_GENERAL,
 	   "debug: (post-boot) Reg fc is %08x\n", reg);
 
-	DTE_PRINTK(INFO, "Installed a Wildcard TC: %s \n", wc->variety);
+	dev_info(&wc->pdev->dev, "Installed a Wildcard TC: %s\n", wc->variety);
 	DTE_DEBUG(DTE_DEBUG_GENERAL, "Operating in DEBUG mode.\n");
 	dahdi_transcoder_register(wc->uencode);
 	dahdi_transcoder_register(wc->udecode);
@@ -3722,11 +3730,17 @@ static DEFINE_PCI_DEVICE_TABLE(wctc4xxp_pci_tbl) = {
 
 MODULE_DEVICE_TABLE(pci, wctc4xxp_pci_tbl);
 
+static int wctc4xxp_suspend(struct pci_dev *pdev, pm_message_t state)
+{
+	return -ENOSYS;
+}
+
 static struct pci_driver wctc4xxp_driver = {
 	.name = "wctc4xxp",
 	.probe = wctc4xxp_init_one,
 	.remove = __devexit_p(wctc4xxp_remove_one),
 	.id_table = wctc4xxp_pci_tbl,
+	.suspend = wctc4xxp_suspend,
 };
 
 static int __init wctc4xxp_init(void)
@@ -3767,7 +3781,7 @@ static void __exit wctc4xxp_cleanup(void)
 }
 
 module_param(debug, int, S_IRUGO | S_IWUSR);
-module_param(mode, charp, S_IRUGO | S_IWUSR);
+module_param(mode, charp, S_IRUGO);
 MODULE_PARM_DESC(mode, "'g729', 'g723.1', or 'any'.  Default 'any'.");
 MODULE_DESCRIPTION("Wildcard TC400P+TC400M Driver");
 MODULE_AUTHOR("Digium Incorporated <support@digium.com>");
