@@ -27,7 +27,6 @@
 #include <linux/errno.h>
 #include <linux/module.h>
 #include <linux/init.h>
-#include <linux/errno.h>
 #include <linux/pci.h>
 #include <linux/interrupt.h>
 #include <linux/moduleparam.h>
@@ -620,13 +619,11 @@ static unsigned char wctdm_getreg(struct wctdm *wc, int card, unsigned char reg)
 static int __wait_access(struct wctdm *wc, int card)
 {
     unsigned char data = 0;
-    long origjiffies;
     int count = 0;
 
     #define MAX 6000 /* attempts */
 
 
-    origjiffies = jiffies;
     /* Wait for indirect access */
     while (count++ < MAX)
 	 {
@@ -1713,7 +1710,7 @@ static int wctdm_init_voicedaa(struct wctdm *wc, int card, int fast, int manual,
 
 	/* Misc. DAA parameters */
 	if (fastpickup)
-		reg31 = 0xb3;
+		reg31 = 0xe3;
 	else
 		reg31 = 0xa3;
 
@@ -2371,7 +2368,7 @@ static int wctdm_initialize(struct wctdm *wc)
 	snprintf(wc->span.location, sizeof(wc->span.location) - 1,
 		 "PCI Bus %02d Slot %02d", wc->dev->bus->number, PCI_SLOT(wc->dev->devfn) + 1);
 	wc->span.manufacturer = "Digium";
-	dahdi_copy_string(wc->span.devicetype, wc->variety, sizeof(wc->span.devicetype));
+	strlcpy(wc->span.devicetype, wc->variety, sizeof(wc->span.devicetype));
 	if (alawoverride) {
 		printk(KERN_INFO "ALAW override parameter detected.  Device will be operating in ALAW\n");
 		wc->span.deflaw = DAHDI_LAW_ALAW;
@@ -2389,7 +2386,6 @@ static int wctdm_initialize(struct wctdm *wc)
 	wc->span.channels = NUM_CARDS;
 	wc->span.irq = wc->dev->irq;
 	wc->span.flags = DAHDI_FLAG_RBS;
-	init_waitqueue_head(&wc->span.maintq);
 	wc->span.ops = &wctdm_span_ops;
 
 	if (dahdi_register(&wc->span, 0)) {
@@ -2767,12 +2763,16 @@ static DEFINE_PCI_DEVICE_TABLE(wctdm_pci_tbl) = {
 
 MODULE_DEVICE_TABLE(pci, wctdm_pci_tbl);
 
+static int wctdm_suspend(struct pci_dev *pdev, pm_message_t state)
+{
+	return -ENOSYS;
+}
+
 static struct pci_driver wctdm_driver = {
 	.name = "wctdm",
 	.probe = wctdm_init_one,
 	.remove =__devexit_p(wctdm_remove_one),
-	.suspend = NULL,
-	.resume = NULL,
+	.suspend = wctdm_suspend,
 	.id_table = wctdm_pci_tbl,
 };
 
