@@ -28,7 +28,6 @@
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/usb.h>
-#include <linux/errno.h>
 #include <linux/pci.h>
 #include <linux/spinlock.h>
 #include <linux/moduleparam.h>
@@ -468,9 +467,6 @@ static int t1xxp_maint(struct dahdi_span *span, int cmd)
 		case DAHDI_MAINT_LOOPDOWN:
 			printk(KERN_INFO "XXX Send loopdown code E1 XXX\n");
 			break;
-		case DAHDI_MAINT_LOOPSTOP:
-			printk(KERN_INFO "XXX Stop sending loop codes E1 XXX\n");
-			break;
 		default:
 			printk(KERN_NOTICE "TE110P: Unknown E1 maint command: %d\n", cmd);
 			break;
@@ -479,6 +475,7 @@ static int t1xxp_maint(struct dahdi_span *span, int cmd)
 		switch(cmd) {
 	    case DAHDI_MAINT_NONE:
 			printk(KERN_INFO "XXX Turn off local and remote loops T1 XXX\n");
+			t1_framer_out(wc, 0x21, 0x40);
 			break;
 	    case DAHDI_MAINT_LOCALLOOP:
 			printk(KERN_INFO "XXX Turn on local loop and no remote loop XXX\n");
@@ -492,9 +489,6 @@ static int t1xxp_maint(struct dahdi_span *span, int cmd)
 	    case DAHDI_MAINT_LOOPDOWN:
 			t1_framer_out(wc, 0x21, 0x60);	/* FMR5: Nothing but RBS mode */
 			break;
-	    case DAHDI_MAINT_LOOPSTOP:
-			t1_framer_out(wc, 0x21, 0x40);	/* FMR5: Nothing but RBS mode */
-			break;
 	    default:
 			printk(KERN_NOTICE "TE110P: Unknown T1 maint command: %d\n", cmd);
 			break;
@@ -884,7 +878,7 @@ static void t1xxp_framer_start(struct t1 *wc, struct dahdi_span *span)
 }
 
 
-static int t1xxp_startup(struct dahdi_span *span)
+static int t1xxp_startup(struct file *file, struct dahdi_span *span)
 {
 	struct t1 *wc = t1_from_span(span);
 
@@ -926,7 +920,8 @@ static int t1xxp_shutdown(struct dahdi_span *span)
 }
 
 
-static int t1xxp_chanconfig(struct dahdi_chan *chan, int sigtype)
+static int
+t1xxp_chanconfig(struct file *file, struct dahdi_chan *chan, int sigtype)
 {
 	struct t1 *wc = chan->pvt;
 	unsigned long flags;
@@ -941,7 +936,9 @@ static int t1xxp_chanconfig(struct dahdi_chan *chan, int sigtype)
 	return 0;
 }
 
-static int t1xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
+static int
+t1xxp_spanconfig(struct file *file, struct dahdi_span *span,
+		 struct dahdi_lineconfig *lc)
 {
 	struct t1 *wc = t1_from_span(span);
 
@@ -949,7 +946,7 @@ static int t1xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc
 	wc->sync = (lc->sync) ? 1 : 0;
 	/* If already running, apply changes immediately */
 	if (span->flags & DAHDI_FLAG_RUNNING)
-		return t1xxp_startup(span);
+		return t1xxp_startup(file, span);
 
 	return 0;
 }
@@ -984,7 +981,7 @@ static int t1xxp_software_init(struct t1 *wc)
 	sprintf(wc->span.name, "WCT1/%d", wc->num);
 	snprintf(wc->span.desc, sizeof(wc->span.desc) - 1, "%s Card %d", wc->variety, wc->num);
 	wc->span.manufacturer = "Digium";
-	dahdi_copy_string(wc->span.devicetype, wc->variety, sizeof(wc->span.devicetype));
+	strlcpy(wc->span.devicetype, wc->variety, sizeof(wc->span.devicetype));
 	snprintf(wc->span.location, sizeof(wc->span.location) - 1,
 		 "PCI Bus %02d Slot %02d", wc->dev->bus->number, PCI_SLOT(wc->dev->devfn) + 1);
 	wc->span.irq = wc->dev->irq;
@@ -1004,7 +1001,6 @@ static int t1xxp_software_init(struct t1 *wc)
 	}
 	wc->span.chans = wc->chans;
 	wc->span.flags = DAHDI_FLAG_RBS;
-	init_waitqueue_head(&wc->span.maintq);
 	for (x=0;x<wc->span.channels;x++) {
 		sprintf(wc->chans[x]->name, "WCT1/%d/%d", wc->num, x + 1);
 		wc->chans[x]->sigcap = DAHDI_SIG_EM | DAHDI_SIG_CLEAR | DAHDI_SIG_EM_E1 | 
@@ -1247,7 +1243,7 @@ static void t1_check_alarms(struct t1 *wc)
 	if (wc->span.lineconfig & DAHDI_CONFIG_NOTOPEN) {
 		for (x=0,j=0;x < wc->span.channels;x++)
 			if ((wc->span.chans[x]->flags & DAHDI_FLAG_OPEN) ||
-			    (wc->span.chans[x]->flags & DAHDI_FLAG_NETDEV))
+			    dahdi_have_netdev(wc->span.chans[x]))
 				j++;
 		if (!j)
 			alarms |= DAHDI_ALARM_NOTOPEN;
