@@ -33,7 +33,7 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_bri.c 8545 2010-04-19 19:42:53Z tzafrir $";
+static const char rcsid[] = "$Id: card_bri.c 10021 2011-07-04 14:05:19Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(uint, poll_interval, 500, 0644, "Poll channel state interval in milliseconds (0 - disable)");
@@ -56,6 +56,7 @@ enum xhfc_states {
 	ST_NT_DEACTIVTING	= 4,	/* G4	*/
 };
 
+#ifdef CONFIG_PROC_FS
 static const char *xhfc_state_name(bool is_nt, enum xhfc_states state)
 {
 	const char	*p;
@@ -80,18 +81,19 @@ static const char *xhfc_state_name(bool is_nt, enum xhfc_states state)
 	};
 #undef	_E
 	if(is_nt) {
-		if ((state < ST_RESET) || (state > ST_NT_DEACTIVTING))
+		if (state > ST_NT_DEACTIVTING)
 			p = "NT ???";
 		else
 			p = nt_names[state];
 	} else {
-		if ((state < ST_RESET) || (state > ST_TE_LOST_FRAMING))
+		if (state > ST_TE_LOST_FRAMING)
 			p = "TE ???";
 		else
 			p = te_names[state];
 	}
 	return p;
 }
+#endif
 
 /* xhfc Layer1 physical commands */
 #define HFC_L1_ACTIVATE_TE		0x01
@@ -145,7 +147,7 @@ typedef union {
 #endif
 #define	BRI_BCHAN_SIGCAP	(DAHDI_SIG_CLEAR | DAHDI_SIG_DACS)
 
-#define	IS_NT(xpd)		((xpd)->direction == TO_PHONE)
+#define	IS_NT(xpd)		(PHONEDEV(xpd).direction == TO_PHONE)
 #define	BRI_PORT(xpd)		((xpd)->addr.subunit)
 
 /* shift in PCM highway */
@@ -157,10 +159,14 @@ typedef union {
 static int write_state_register(xpd_t *xpd, byte value);
 static bool bri_packet_is_valid(xpacket_t *pack);
 static void bri_packet_dump(const char *msg, xpacket_t *pack);
+#ifdef	CONFIG_PROC_FS
 static int proc_bri_info_read(char *page, char **start, off_t off, int count, int *eof, void *data);
-static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc);
-static int bri_chanconfig(struct dahdi_chan *chan, int sigtype);
-static int bri_startup(struct dahdi_span *span);
+#endif
+static int bri_spanconfig(struct file *file, struct dahdi_span *span,
+			  struct dahdi_lineconfig *lc);
+static int bri_chanconfig(struct file *file, struct dahdi_chan *chan,
+			  int sigtype);
+static int bri_startup(struct file *file, struct dahdi_span *span);
 static int bri_shutdown(struct dahdi_span *span);
 
 #define	PROC_REGISTER_FNAME	"slics"
@@ -340,14 +346,12 @@ static void layer1_state(xpd_t *xpd, bool up)
 static void te_activation(xpd_t *xpd, bool on)
 {
 	struct BRI_priv_data	*priv;
-	xbus_t			*xbus;
 	byte			curr_state;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	curr_state = priv->state_register.bits.v_su_sta;
-	xbus = xpd->xbus;
 	XPD_DBG(SIGNAL, xpd, "%s\n", (on)?"ON":"OFF");
 	if(on) {
 		if(curr_state == ST_TE_DEACTIVATED) {
@@ -385,14 +389,12 @@ static void te_activation(xpd_t *xpd, bool on)
 static void nt_activation(xpd_t *xpd, bool on)
 {
 	struct BRI_priv_data	*priv;
-	xbus_t			*xbus;
 	byte			curr_state;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	curr_state = priv->state_register.bits.v_su_sta;
-	xbus = xpd->xbus;
 	XPD_DBG(SIGNAL, xpd, "%s\n", (on)?"ON":"OFF");
 	if(on) {
 		switch(curr_state) {
@@ -537,7 +539,6 @@ static void bri_hdlc_finish(xpd_t *xpd, struct dahdi_chan *dchan)
 #ifdef	CONFIG_DAHDI_BRI_DCHANS
 static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 {
-	xbus_t			*xbus;
 	struct BRI_priv_data	*priv;
 	byte			*src;
 	byte			*dst;
@@ -559,9 +560,6 @@ static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	xbus = xpd->xbus;
-#ifdef XPP_DEBUGFS
-	xbus_log(xbus, xpd, 0, regcmd, sizeof(reg_cmd_t));		/* 0 = RX */
-#endif
 	dchan = XPD_CHAN(xpd, 2);
 	if(!IS_OFFHOOK(xpd, 2)) {	/* D-chan is used? */
 		static int rate_limit;
@@ -617,7 +615,6 @@ out:
 #else
 static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 {
-	xbus_t			*xbus;
 	struct BRI_priv_data	*priv;
 	byte			*src;
 	struct dahdi_chan	*dchan;
@@ -635,10 +632,6 @@ static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	xbus = xpd->xbus;
-#ifdef XPP_DEBUGFS
-	xbus_log(xbus, xpd, 0, regcmd, sizeof(reg_cmd_t));		/* 0 = RX */
-#endif
 	dchan = XPD_CHAN(xpd, 2);
 	if(!IS_OFFHOOK(xpd, 2)) {	/* D-chan is used? */
 		static int rate_limit;
@@ -728,7 +721,7 @@ static int tx_dchan(xpd_t *xpd)
 	if(atomic_read(&priv->hdlc_pending) == 0)
 		return 0;
 #endif
-	if(!SPAN_REGISTERED(xpd) || !(xpd->span.flags & DAHDI_FLAG_RUNNING))
+	if(!SPAN_REGISTERED(xpd) || !(PHONEDEV(xpd).span.flags & DAHDI_FLAG_RUNNING))
 		return 0;
 	dchan = XPD_CHAN(xpd, 2);
 	len = ARRAY_SIZE(priv->dchan_tbuf);
@@ -744,7 +737,7 @@ static int tx_dchan(xpd_t *xpd)
 	}
 	if(!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags) && !test_bit(HFC_L1_ACTIVATING, &priv->l1_flags)) {
 		XPD_DBG(SIGNAL, xpd, "Want to transmit: Kick D-Channel transmiter\n");
-		if(xpd->direction == TO_PSTN)
+		if(! IS_NT(xpd))
 			te_activation(xpd, 1);
 		else
 			nt_activation(xpd, 1);
@@ -799,14 +792,12 @@ static int bri_proc_create(xbus_t *xbus, xpd_t *xpd)
 	priv->bri_info = create_proc_read_entry(PROC_BRI_INFO_FNAME, 0444, xpd->proc_xpd_dir, proc_bri_info_read, xpd);
 	if(!priv->bri_info) {
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_BRI_INFO_FNAME);
-		goto err;
+		bri_proc_remove(xbus, xpd);
+		return -EINVAL;
 	}
 	SET_PROC_DIRENTRY_OWNER(priv->bri_info);
 #endif
 	return 0;
-err:
-	bri_proc_remove(xbus, xpd);
-	return -EINVAL;
 }
 
 static xpd_t *BRI_card_new(xbus_t *xbus, int unit, int subunit, const xproto_table_t *proto_table,
@@ -823,7 +814,7 @@ static xpd_t *BRI_card_new(xbus_t *xbus, int unit, int subunit, const xproto_tab
 	xpd = xpd_alloc(xbus, unit, subunit, subtype, subunits, sizeof(struct BRI_priv_data), proto_table, channels);
 	if(!xpd)
 		return NULL;
-	xpd->direction = (to_phone) ? TO_PHONE : TO_PSTN;
+	PHONEDEV(xpd).direction = (to_phone) ? TO_PHONE : TO_PSTN;
 	xpd->type_name = (to_phone) ? "BRI_NT" : "BRI_TE";
 	if(bri_proc_create(xbus, xpd) < 0)
 		goto err;
@@ -850,15 +841,36 @@ static int BRI_card_init(xbus_t *xbus, xpd_t *xpd)
 
 static int BRI_card_remove(xbus_t *xbus, xpd_t *xpd)
 {
-	struct BRI_priv_data	*priv;
-
 	BUG_ON(!xpd);
-	priv = xpd->priv;
 	XPD_DBG(GENERAL, xpd, "\n");
 	bri_proc_remove(xbus, xpd);
 	return 0;
 }
 
+static const struct dahdi_span_ops BRI_span_ops = {
+	.owner = THIS_MODULE,
+	.spanconfig = bri_spanconfig,
+	.chanconfig = bri_chanconfig,
+	.startup = bri_startup,
+	.shutdown = bri_shutdown,
+#ifndef	CONFIG_DAHDI_BRI_DCHANS
+	.hdlc_hard_xmit = bri_hdlc_hard_xmit,
+#endif
+	.open = xpp_open,
+	.close = xpp_close,
+	.hooksig = xpp_hooksig,	/* Only with RBS bits */
+	.ioctl = xpp_ioctl,
+	.maint = xpp_maint,
+	.echocan_create = xpp_echocan_create,
+	.echocan_name = xpp_echocan_name,
+#ifdef	DAHDI_SYNC_TICK
+	.sync_tick = dahdi_sync_tick,
+#endif
+#ifdef	CONFIG_DAHDI_WATCHDOG
+	.watchdog = xpp_watchdog,
+#endif
+};
+
 static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
@@ -874,11 +886,10 @@ static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 		/* Nothing to do yet */
 		return 0;
 	}
-	xpd->span.owner = THIS_MODULE;
-	xpd->span.spantype = "BRI";
-	xpd->span.linecompat = DAHDI_CONFIG_AMI | DAHDI_CONFIG_CCS;
-	xpd->span.deflaw = DAHDI_LAW_ALAW;
-	BIT_SET(xpd->digital_signalling, 2);	/* D-Channel */
+	PHONEDEV(xpd).span.spantype = "BRI";
+	PHONEDEV(xpd).span.linecompat = DAHDI_CONFIG_AMI | DAHDI_CONFIG_CCS;
+	PHONEDEV(xpd).span.deflaw = DAHDI_LAW_ALAW;
+	BIT_SET(PHONEDEV(xpd).digital_signalling, 2);	/* D-Channel */
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*cur_chan = XPD_CHAN(xpd, i);
 
@@ -909,31 +920,23 @@ static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 			cur_chan->sigcap = BRI_BCHAN_SIGCAP;
 		}
 	}
-	CALL_XMETHOD(card_pcm_recompute, xbus, xpd, 0);
-	xpd->span.spanconfig = bri_spanconfig;
-	xpd->span.chanconfig = bri_chanconfig;
-	xpd->span.startup = bri_startup;
-	xpd->span.shutdown = bri_shutdown;
-#ifndef	CONFIG_DAHDI_BRI_DCHANS
-	xpd->span.hdlc_hard_xmit = bri_hdlc_hard_xmit;
-#endif
+	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
+	PHONEDEV(xpd).span.ops = &BRI_span_ops;
 	return 0;
 }
 
 static int BRI_card_dahdi_postregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
-	struct BRI_priv_data	*priv;
 	
 	BUG_ON(!xpd);
 	xbus = xpd->xbus;
-	priv = xpd->priv;
 	BUG_ON(!xbus);
 	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"on":"off");
 	return(0);
 }
 
-static int BRI_card_hooksig(xbus_t *xbus, xpd_t *xpd, int pos, enum dahdi_txsig txsig)
+static int BRI_card_hooksig(xpd_t *xpd, int pos, enum dahdi_txsig txsig)
 {
 	LINE_DBG(SIGNAL, xpd, pos, "%s\n", txsig2str(txsig));
 	return 0;
@@ -1128,16 +1131,13 @@ static int BRI_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 
 static int BRI_card_open(xpd_t *xpd, lineno_t pos)
 {
-	struct BRI_priv_data	*priv;
-
 	BUG_ON(!xpd);
-	priv = xpd->priv;
 	if(pos == 2) {
 		LINE_DBG(SIGNAL, xpd, pos, "OFFHOOK the whole span\n");
-		BIT_SET(xpd->offhook_state, 0);
-		BIT_SET(xpd->offhook_state, 1);
-		BIT_SET(xpd->offhook_state, 2);
-		CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+		BIT_SET(PHONEDEV(xpd).offhook_state, 0);
+		BIT_SET(PHONEDEV(xpd).offhook_state, 1);
+		BIT_SET(PHONEDEV(xpd).offhook_state, 2);
+		CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	}
 	return 0;
 }
@@ -1156,10 +1156,10 @@ static int BRI_card_close(xpd_t *xpd, lineno_t pos)
 #endif
 	if(pos == 2) {
 		LINE_DBG(SIGNAL, xpd, pos, "ONHOOK the whole span\n");
-		BIT_CLR(xpd->offhook_state, 0);
-		BIT_CLR(xpd->offhook_state, 1);
-		BIT_CLR(xpd->offhook_state, 2);
-		CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, 0);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, 1);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, 2);
+		CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	}
 	return 0;
 }
@@ -1167,9 +1167,11 @@ static int BRI_card_close(xpd_t *xpd, lineno_t pos)
 /*
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
-static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
+static int bri_spanconfig(struct file *file, struct dahdi_span *span,
+			  struct dahdi_lineconfig *lc)
 {
-	xpd_t		*xpd = span->pvt;
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	const char	*framingstr = "";
 	const char	*codingstr = "";
 	const char	*crcstr = "";
@@ -1211,7 +1213,8 @@ static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
  * Called from dahdi with spinlock held on chan. Must not call back
  * dahdi functions.
  */
-static int bri_chanconfig(struct dahdi_chan *chan, int sigtype)
+static int bri_chanconfig(struct file *file, struct dahdi_chan *chan,
+			  int sigtype)
 {
 	DBG(GENERAL, "channel %d (%s) -> %s\n", chan->channo, chan->name, sig2str(sigtype));
 	// FIXME: sanity checks:
@@ -1223,9 +1226,10 @@ static int bri_chanconfig(struct dahdi_chan *chan, int sigtype)
 /*
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
-static int bri_startup(struct dahdi_span *span)
+static int bri_startup(struct file *file, struct dahdi_span *span)
 {
-	xpd_t			*xpd = span->pvt;
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct BRI_priv_data	*priv;
 	struct dahdi_chan	*dchan;
 
@@ -1238,7 +1242,7 @@ static int bri_startup(struct dahdi_span *span)
 	}
 	XPD_DBG(GENERAL, xpd, "STARTUP\n");
 	// Turn on all channels
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 1);
+	CALL_PHONE_METHOD(card_state, xpd, 1);
 	if(SPAN_REGISTERED(xpd)) {
 		dchan = XPD_CHAN(xpd, 2);
 		span->flags |= DAHDI_FLAG_RUNNING;
@@ -1261,7 +1265,8 @@ static int bri_startup(struct dahdi_span *span)
  */
 static int bri_shutdown(struct dahdi_span *span)
 {
-	xpd_t			*xpd = span->pvt;
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct BRI_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -1273,11 +1278,11 @@ static int bri_shutdown(struct dahdi_span *span)
 	}
 	XPD_DBG(GENERAL, xpd, "SHUTDOWN\n");
 	// Turn off all channels
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 0);
+	CALL_PHONE_METHOD(card_state, xpd, 0);
 	return 0;
 }
 
-static void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
+static void BRI_card_pcm_recompute(xpd_t *xpd,
 		xpp_line_t dont_care)
 {
 	int		i;
@@ -1288,7 +1293,7 @@ static void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
 	unsigned long	flags;
 
 	BUG_ON(!xpd);
-	main_xpd = xpd_byaddr(xbus, xpd->addr.unit, 0);
+	main_xpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, 0);
 	if(!main_xpd) {
 		XPD_DBG(DEVICES, xpd, "Unit 0 is already gone. Ignore request\n");
 		return;
@@ -1297,15 +1302,15 @@ static void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
 	 * We calculate all subunits, so use the main lock
 	 * as a mutex for the whole operation.
 	 */
-	spin_lock_irqsave(&main_xpd->lock_recompute_pcm, flags);
+	spin_lock_irqsave(&PHONEDEV(main_xpd).lock_recompute_pcm, flags);
 	line_count = 0;
 	pcm_mask = 0;
 	for(i = 0; i < MAX_SUBUNIT; i++) {
-		xpd_t		*sub_xpd = xpd_byaddr(xbus, main_xpd->addr.unit, i);
+		xpd_t		*sub_xpd = xpd_byaddr(xpd->xbus, main_xpd->addr.unit, i);
 
 		if(sub_xpd) {
 			xpp_line_t	lines =
-				sub_xpd->offhook_state & ~sub_xpd->digital_signalling;
+				PHONEDEV(sub_xpd).offhook_state & ~(PHONEDEV(sub_xpd).digital_signalling);
 
 			if(lines) {
 				pcm_mask |= PCM_SHIFT(lines, i);
@@ -1332,10 +1337,10 @@ static void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
 		? RPACKET_HEADERSIZE + sizeof(xpp_line_t) + line_count * DAHDI_CHUNKSIZE
 		: 0L;
 	update_wanted_pcm_mask(main_xpd, pcm_mask, pcm_len);
-	spin_unlock_irqrestore(&main_xpd->lock_recompute_pcm, flags);
+	spin_unlock_irqrestore(&PHONEDEV(main_xpd).lock_recompute_pcm, flags);
 }
 
-static void BRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+static void BRI_card_pcm_fromspan(xpd_t *xpd, xpacket_t *pack)
 {
 	byte		*pcm;
 	unsigned long	flags;
@@ -1345,18 +1350,17 @@ static void BRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	xpp_line_t	wanted_lines;
 
 
-	BUG_ON(!xbus);
 	BUG_ON(!xpd);
 	BUG_ON(!pack);
 	pcm = RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, pcm);
 	for(subunit = 0; subunit < MAX_SUBUNIT; subunit++) {
 		xpd_t		*tmp_xpd;
 
-		tmp_xpd = xpd_byaddr(xbus, xpd->addr.unit, subunit);
+		tmp_xpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, subunit);
 		if(!tmp_xpd || !tmp_xpd->card_present)
 			continue;
 		spin_lock_irqsave(&tmp_xpd->lock, flags);
-		wanted_lines = tmp_xpd->wanted_pcm_mask;
+		wanted_lines = PHONEDEV(tmp_xpd).wanted_pcm_mask;
 		for_each_line(tmp_xpd, i) {
 			struct dahdi_chan	*chan = XPD_CHAN(tmp_xpd, i);
 
@@ -1382,7 +1386,7 @@ static void BRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, lines) = pcm_mask;
 }
 
-static void BRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+static void BRI_card_pcm_tospan(xpd_t *xpd, xpacket_t *pack)
 {
 	byte		*pcm;
 	xpp_line_t	pcm_mask;
@@ -1404,7 +1408,7 @@ static void BRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 
 		if(!pcm_mask)
 			break;	/* optimize */
-		tmp_xpd = xpd_byaddr(xbus, xpd->addr.unit, subunit);
+		tmp_xpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, subunit);
 		if(!tmp_xpd || !tmp_xpd->card_present || !SPAN_REGISTERED(tmp_xpd))
 			continue;
 		spin_lock_irqsave(&tmp_xpd->lock, flags);
@@ -1424,27 +1428,36 @@ static void BRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	}
 }
 
-/*---------------- BRI: HOST COMMANDS -------------------------------------*/
+int BRI_echocancel_timeslot(xpd_t *xpd, int pos)
+{
+	return xpd->addr.subunit * 4 + pos;
+}
 
-static /* 0x0F */ HOSTCMD(BRI, XPD_STATE, bool on)
+static int BRI_echocancel_setmask(xpd_t *xpd, xpp_line_t ec_mask)
 {
 	struct BRI_priv_data	*priv;
+	int i;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
-	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"ON":"OFF");
-	if(on) {
-		if(!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags)) {
-			if(xpd->direction == TO_PSTN)
-				te_activation(xpd, 1);
-			else
-				nt_activation(xpd, 1);
-		}
-	} else if(IS_NT(xpd))
-		nt_activation(xpd, 0);
+	BUG_ON(!priv);
+	XPD_DBG(GENERAL, xpd, "0x%8X\n", ec_mask);
+	if (!ECHOOPS(xpd->xbus)) {
+		XPD_DBG(GENERAL, xpd,
+				"No echo canceller in XBUS: Doing nothing.\n");
+		return -EINVAL;
+	}
+	for (i = 0; i < PHONEDEV(xpd).channels - 1; i++) {
+		int	on = BIT(i) & ec_mask;
+
+		CALL_EC_METHOD(ec_set, xpd->xbus, xpd, i, on);
+	}
+	CALL_EC_METHOD(ec_update, xpd->xbus, xpd->xbus);
 	return 0;
 }
 
+/*---------------- BRI: HOST COMMANDS -------------------------------------*/
+
 static /* 0x33 */ HOSTCMD(BRI, SET_LED, enum bri_led_names which_led, enum led_state to_led_state)
 {
 	int			ret = 0;
@@ -1491,14 +1504,12 @@ static int write_state_register(xpd_t *xpd, byte value)
 /*---------------- BRI: Astribank Reply Handlers --------------------------*/
 static void su_new_state(xpd_t *xpd, byte reg_x30)
 {
-	xbus_t			*xbus;
 	struct BRI_priv_data	*priv;
 	su_rd_sta_t		new_state;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	xbus = xpd->xbus;
 	if(!priv->initialized) {
 		XPD_ERR(xpd, "%s called on uninitialized AB\n", __FUNCTION__);
 		return;
@@ -1628,7 +1639,7 @@ static int BRI_card_register_reply(xbus_t *xbus, xpd_t *xpd, reg_cmd_t *info)
 		ret = rx_dchan(xpd, info);
 		if (ret < 0) {
 			priv->dchan_rx_drops++;
-			if(atomic_read(&xpd->open_counter) > 0)
+			if(atomic_read(&PHONEDEV(xpd).open_counter) > 0)
 				XPD_NOTICE(xpd, "Multibyte Drop: errno=%d\n", ret);
 		} 
 		goto end;
@@ -1650,6 +1661,49 @@ end:
 	return 0;
 }
 
+static int BRI_card_state(xpd_t *xpd, bool on)
+{
+	struct BRI_priv_data	*priv;
+
+	BUG_ON(!xpd);
+	priv = xpd->priv;
+	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"ON":"OFF");
+	if(on) {
+		if(!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags)) {
+			if( ! IS_NT(xpd))
+				te_activation(xpd, 1);
+			else
+				nt_activation(xpd, 1);
+		}
+	} else if(IS_NT(xpd))
+		nt_activation(xpd, 0);
+	return 0;
+}
+
+static const struct xops	bri_xops = {
+	.card_new	= BRI_card_new,
+	.card_init	= BRI_card_init,
+	.card_remove	= BRI_card_remove,
+	.card_tick	= BRI_card_tick,
+	.card_register_reply	= BRI_card_register_reply,
+};
+
+static const struct phoneops	bri_phoneops = {
+	.card_dahdi_preregistration	= BRI_card_dahdi_preregistration,
+	.card_dahdi_postregistration	= BRI_card_dahdi_postregistration,
+	.card_hooksig	= BRI_card_hooksig,
+	.card_pcm_recompute	= BRI_card_pcm_recompute,
+	.card_pcm_fromspan	= BRI_card_pcm_fromspan,
+	.card_pcm_tospan	= BRI_card_pcm_tospan,
+	.card_timing_priority	= generic_timing_priority,
+	.echocancel_timeslot	= BRI_echocancel_timeslot,
+	.echocancel_setmask	= BRI_echocancel_setmask,
+	.card_ioctl	= BRI_card_ioctl,
+	.card_open	= BRI_card_open,
+	.card_close	= BRI_card_close,
+	.card_state	= BRI_card_state,
+};
+
 static xproto_table_t PROTO_TABLE(BRI) = {
 	.owner = THIS_MODULE,
 	.entries = {
@@ -1658,24 +1712,8 @@ static xproto_table_t PROTO_TABLE(BRI) = {
 	.name = "BRI",	/* protocol name */
 	.ports_per_subunit = 1,
 	.type = XPD_TYPE_BRI,
-	.xops = {
-		.card_new	= BRI_card_new,
-		.card_init	= BRI_card_init,
-		.card_remove	= BRI_card_remove,
-		.card_dahdi_preregistration	= BRI_card_dahdi_preregistration,
-		.card_dahdi_postregistration	= BRI_card_dahdi_postregistration,
-		.card_hooksig	= BRI_card_hooksig,
-		.card_tick	= BRI_card_tick,
-		.card_pcm_recompute	= BRI_card_pcm_recompute,
-		.card_pcm_fromspan	= BRI_card_pcm_fromspan,
-		.card_pcm_tospan	= BRI_card_pcm_tospan,
-		.card_ioctl	= BRI_card_ioctl,
-		.card_open	= BRI_card_open,
-		.card_close	= BRI_card_close,
-		.card_register_reply	= BRI_card_register_reply,
-
-		.XPD_STATE	= XPROTO_CALLER(BRI, XPD_STATE),
-	},
+	.xops = &bri_xops,
+	.phoneops = &bri_phoneops,
 	.packet_is_valid = bri_packet_is_valid,
 	.packet_dump = bri_packet_dump,
 };
@@ -1694,6 +1732,7 @@ static void bri_packet_dump(const char *msg, xpacket_t *pack)
 }
 /*------------------------- REGISTER Handling --------------------------*/
 
+#ifdef	CONFIG_PROC_FS
 static int proc_bri_info_read(char *page, char **start, off_t off, int count, int *eof, void *data)
 {
 	int			len = 0;
@@ -1756,6 +1795,7 @@ static int proc_bri_info_read(char *page, char **start, off_t off, int count, in
 		len = 0;
 	return len;
 }
+#endif
 
 static DRIVER_ATTR_READER(dchan_hardhdlc_show, drv,buf)
 {
