@@ -30,7 +30,7 @@
 #include "xbus-core.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: card_global.c 8545 2010-04-19 19:42:53Z tzafrir $";
+static const char rcsid[] = "$Id: card_global.c 9916 2011-05-23 13:37:40Z tzafrir $";
 
 DEF_PARM(charp,initdir, "/usr/share/dahdi", 0644, "The directory of card initialization scripts");
 
@@ -40,59 +40,6 @@ extern	int debug;
 
 /*---------------- GLOBAL PROC handling -----------------------------------*/
 
-#ifdef	OLD_PROC
-static int proc_xpd_register_read(char *page, char **start, off_t off, int count, int *eof, void *data)
-{
-	int			len = 0;
-	unsigned long		flags;
-	xpd_t			*xpd = data;
-	reg_cmd_t		*info;
-	bool			do_datah;
-	char			datah_str[50];
-
-	if(!xpd)
-		return -ENODEV;
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] read from /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	spin_lock_irqsave(&xpd->lock, flags);
-	info = &xpd->last_reply;
-	len += sprintf(page + len, "# Writing bad data into this file may damage your hardware!\n");
-	len += sprintf(page + len, "# Consult firmware docs first\n");
-	len += sprintf(page + len, "#\n");
-	do_datah = REG_FIELD(info, do_datah) ? 1 : 0;
-	if(do_datah) {
-		snprintf(datah_str, ARRAY_SIZE(datah_str), "\t%02X",
-			REG_FIELD(info, data_high));
-	} else
-		datah_str[0] = '\0';
-	if(REG_FIELD(info, do_subreg)) {
-		len += sprintf(page + len, "#CH\tOP\tReg.\tSub\tDL%s\n",
-				(do_datah) ? "\tDH" : "");
-		len += sprintf(page + len, "%2d\tRS\t%02X\t%02X\t%02X%s\n",
-				info->portnum,
-				REG_FIELD(info, regnum), REG_FIELD(info, subreg),
-				REG_FIELD(info, data_low), datah_str);
-	} else {
-		len += sprintf(page + len, "#CH\tOP\tReg.\tDL%s\n",
-				(do_datah) ? "\tDH" : "");
-		len += sprintf(page + len, "%2d\tRD\t%02X\t%02X%s\n",
-				info->portnum,
-				REG_FIELD(info, regnum),
-				REG_FIELD(info, data_low), datah_str);
-	}
-	spin_unlock_irqrestore(&xpd->lock, flags);
-	if (len <= off+count)
-		*eof = 1;
-	*start = page + off;
-	len -= off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-	return len;
-}
-#endif
-
 static int parse_hexbyte(const char *buf)
 {
 	char		*endp;
@@ -342,78 +289,6 @@ out:
 	return ret;
 }
 
-#ifdef	OLD_PROC
-static int proc_xpd_register_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
-{
-	xpd_t		*xpd = data;
-	char		buf[MAX_PROC_WRITE];
-	char		*p;
-	int		i;
-	int		ret;
-
-	if(!xpd)
-		return -ENODEV;
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] wrote to /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	for(i = 0; i < count; /* noop */) {
-		for(p = buf; p < buf + MAX_PROC_WRITE; p++) {	/* read a line */
-			if(i >= count)
-				break;
-			if(get_user(*p, buffer + i))
-				return -EFAULT;
-			i++;
-			if(*p == '\n' || *p == '\r')	/* whatever */
-				break;
-		}
-		if(p >= buf + MAX_PROC_WRITE)
-			return -E2BIG;
-		*p = '\0';
-		ret = parse_chip_command(xpd, buf);
-		if(ret < 0) {
-			XPD_NOTICE(xpd, "Failed writing command: '%s'\n", buf);
-			return ret;
-		}
-		/* Don't flood command_queue */
-		if(xframe_queue_count(&xpd->xbus->command_queue) > 5)
-			msleep(6);
-	}
-	return count;
-}
-
-void chip_proc_remove(xbus_t *xbus, xpd_t *xpd)
-{
-	BUG_ON(!xpd);
-#ifdef	CONFIG_PROC_FS
-	if(xpd->proc_xpd_chipregs) {
-		XBUS_DBG(PROC, xbus, "UNIT %d: Removing %s\n", xpd->addr.unit, CHIP_REGISTERS);
-		xpd->proc_xpd_chipregs->data = NULL;
-		remove_proc_entry(CHIP_REGISTERS, xpd->proc_xpd_dir);
-	}
-#endif
-}
-
-int chip_proc_create(xbus_t *xbus, xpd_t *xpd)
-{
-	BUG_ON(!xpd);
-#ifdef	CONFIG_PROC_FS
-	XBUS_DBG(PROC, xbus, "UNIT %d: Creating %s\n", xpd->addr.unit, CHIP_REGISTERS);
-	xpd->proc_xpd_chipregs = create_proc_entry(CHIP_REGISTERS, 0644, xpd->proc_xpd_dir);
-	if(!xpd->proc_xpd_chipregs) {
-		XPD_ERR(xpd, "Failed to create proc file '%s'\n", CHIP_REGISTERS);
-		goto err;
-	}
-	SET_PROC_DIRENTRY_OWNER(xpd->proc_xpd_chipregs);
-	xpd->proc_xpd_chipregs->write_proc = proc_xpd_register_write;
-	xpd->proc_xpd_chipregs->read_proc = proc_xpd_register_read;
-	xpd->proc_xpd_chipregs->data = xpd;
-#endif
-	return 0;
-err:
-	chip_proc_remove(xbus, xpd);
-	return -EINVAL;
-}
-#endif
-
 /*---------------- GLOBAL Protocol Commands -------------------------------*/
 
 static bool global_packet_is_valid(xpacket_t *pack);
@@ -431,6 +306,8 @@ static void global_packet_dump(const char *msg, xpacket_t *pack);
 		DBG(DEVICES, "NO XBUS\n");
 		return -EINVAL;
 	}
+	if (xbus_check_unique(xbus))
+		return -EBUSY;
 	XFRAME_NEW_CMD(xframe, pack, xbus, GLOBAL, AB_REQUEST, 0);
 	RPACKET_FIELD(pack, GLOBAL, AB_REQUEST, rev) = XPP_PROTOCOL_VERSION;
 	RPACKET_FIELD(pack, GLOBAL, AB_REQUEST, reserved) = 0;
@@ -576,13 +453,17 @@ HANDLER_DEF(GLOBAL, NULL_REPLY)
 
 HANDLER_DEF(GLOBAL, AB_DESCRIPTION)	/* 0x08 */
 {
-	struct xbus_workqueue	*worker = xbus->worker;
+	struct xbus_workqueue	*worker;
 	byte			rev;
 	struct unit_descriptor	*units;
 	int			count_units;
 	int			i;
 	int			ret = 0;
 
+	if (!xbus) {
+		NOTICE("%s: xbus is gone!!!\n", __func__);
+		goto out;
+	}
 	rev = RPACKET_FIELD(pack, GLOBAL, AB_DESCRIPTION, rev);
 	units = RPACKET_FIELD(pack, GLOBAL, AB_DESCRIPTION, unit_descriptor);
 	count_units = XPACKET_LEN(pack) - ((byte *)units - (byte *)pack);
@@ -610,11 +491,14 @@ HANDLER_DEF(GLOBAL, AB_DESCRIPTION)	/* 0x08 */
 		goto proto_err;
 	}
 	XBUS_INFO(xbus, "DESCRIPTOR: %d cards, protocol revision %d\n", count_units, rev);
+	if (xbus_check_unique(xbus))
+		return -EBUSY;
 	xbus->revision = rev;
-	if(!worker) {
-		XBUS_ERR(xbus, "missing worker\n");
+	worker = &xbus->worker;
+	if (!worker->wq) {
+		XBUS_ERR(xbus, "missing worker thread\n");
 		ret = -ENODEV;
-		goto err;
+		goto out;
 	}
 	for(i = 0; i < count_units; i++) {
 		struct unit_descriptor	*this_unit = &units[i];
@@ -624,7 +508,7 @@ HANDLER_DEF(GLOBAL, AB_DESCRIPTION)	/* 0x08 */
 		if((card_desc = KZALLOC(sizeof(struct card_desc_struct), GFP_ATOMIC)) == NULL) {
 			XBUS_ERR(xbus, "Card description allocation failed.\n");
 			ret = -ENOMEM;
-			goto err;
+			goto out;
 		}
 		card_desc->magic = CARD_DESC_MAGIC;
 		INIT_LIST_HEAD(&card_desc->card_list);
@@ -650,12 +534,14 @@ HANDLER_DEF(GLOBAL, AB_DESCRIPTION)	/* 0x08 */
 		list_add_tail(&card_desc->card_list, &worker->card_list);
 		spin_unlock_irqrestore(&worker->worker_lock, flags);
 	}
-	if(!xbus_process_worker(xbus))
-		return -ENODEV;
-	return 0;
+	if (!xbus_process_worker(xbus)) {
+		ret = -ENODEV;
+		goto out;
+	}
+	goto out;
 proto_err:
 	dump_packet("AB_DESCRIPTION", pack, DBG_ANY);
-err:
+out:
 	return ret;
 }
 
@@ -674,7 +560,11 @@ HANDLER_DEF(GLOBAL, REGISTER_REPLY)
 		dump_reg_cmd("REG_REPLY", 0, xbus, xpd->addr.unit, reg->portnum, reg);
 		dump_packet("REG_REPLY", pack, 1);
 	}
-	return CALL_XMETHOD(card_register_reply, xbus, xpd, reg);
+	if (! XMETHOD(card_register_reply, xpd)) {
+		XPD_ERR(xpd, "REGISTER_REPLY: without card_register_reply() method\n");
+		return -EINVAL;
+	}
+	return CALL_XMETHOD(card_register_reply, xpd, reg);
 }
 
 HANDLER_DEF(GLOBAL, SYNC_REPLY)
@@ -792,7 +682,8 @@ int run_initialize_registers(xpd_t *xpd)
 	xbus = xpd->xbus;
 	if(!initdir || !initdir[0]) {
 		XPD_NOTICE(xpd, "Missing initdir parameter\n");
-		return -EINVAL;
+		ret = -EINVAL;
+		goto err;
 	}
 	if(!xpd_setstate(xpd, XPD_STATE_INIT_REGS)) {
 		ret = -EINVAL;
@@ -808,7 +699,7 @@ int run_initialize_registers(xpd_t *xpd)
 				xpd->subunits, i);
 			continue;
 		}
-		direction_mask |= (su->direction == TO_PHONE) ? BIT(i) : 0;
+		direction_mask |= (PHONEDEV(su).direction == TO_PHONE) ? BIT(i) : 0;
 	}
 	snprintf(busstr, MAX_ENV_STR, "XBUS_NAME=%s", xbus->busname);
 	snprintf(busnumstr, MAX_ENV_STR, "XBUS_NUMBER=%d", xbus->num);
