@@ -31,10 +31,9 @@
 #include "xpp_dahdi.h"
 #include "card_pri.h"
 #include "dahdi_debug.h"
-#include "xpd.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_pri.c 9680 2011-01-21 05:31:11Z sruffell $";
+static const char rcsid[] = "$Id: card_pri.c 10053 2011-07-20 16:49:07Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(uint, poll_interval, 500, 0644, "Poll channel state interval in milliseconds (0 - disable)");
@@ -76,20 +75,13 @@ static bool is_sigtype_dchan(int sigtype)
 
 static bool pri_packet_is_valid(xpacket_t *pack);
 static void pri_packet_dump(const char *msg, xpacket_t *pack);
-#ifdef	OLD_PROC
-static int proc_pri_info_read(char *page, char **start, off_t off, int count, int *eof, void *data);
-static int proc_pri_info_write(struct file *file, const char __user *buffer, unsigned long count, void *data);
-#endif
-static int pri_startup(struct dahdi_span *span);
+static int pri_startup(struct file *file, struct dahdi_span *span);
 static int pri_shutdown(struct dahdi_span *span);
 static int pri_rbsbits(struct dahdi_chan *chan, int bits);
 static int pri_lineconfig(xpd_t *xpd, int lineconfig);
 static void send_idlebits(xpd_t *xpd, bool saveold);
 
 #define	PROC_REGISTER_FNAME	"slics"
-#ifdef	OLD_PROC
-#define	PROC_PRI_INFO_FNAME	"pri_info"
-#endif
 
 enum pri_protocol {
 	PRI_PROTO_0  	= 0,
@@ -315,9 +307,6 @@ struct pri_leds {
 
 struct PRI_priv_data {
 	bool				clock_source;
-#ifdef	OLD_PROC
-	struct proc_dir_entry		*pri_info;
-#endif
 	enum pri_protocol		pri_protocol;
 	xpp_line_t			rbslines;
 	int				deflaw;
@@ -495,50 +484,6 @@ static int write_cas_reg(xpd_t *xpd, int rsnum, byte val)
 	return 0;
 }
 
-#ifdef	OLD_PROC
-static void pri_proc_remove(xbus_t *xbus, xpd_t *xpd)
-{
-	struct PRI_priv_data	*priv;
-
-	BUG_ON(!xpd);
-	priv = xpd->priv;
-	XPD_DBG(PROC, xpd, "\n");
-#ifdef	CONFIG_PROC_FS
-	if(priv->pri_info) {
-		XPD_DBG(PROC, xpd, "Removing xpd PRI_INFO file\n");
-		remove_proc_entry(PROC_PRI_INFO_FNAME, xpd->proc_xpd_dir);
-	}
-#endif
-}
-#endif
-
-#ifdef	OLD_PROC
-static int pri_proc_create(xbus_t *xbus, xpd_t *xpd)
-{
-	struct PRI_priv_data	*priv;
-
-	BUG_ON(!xpd);
-	priv = xpd->priv;
-	XPD_DBG(PROC, xpd, "\n");
-#ifdef	CONFIG_PROC_FS
-	XPD_DBG(PROC, xpd, "Creating PRI_INFO file\n");
-	priv->pri_info = create_proc_entry(PROC_PRI_INFO_FNAME, 0644, xpd->proc_xpd_dir);
-	if(!priv->pri_info) {
-		XPD_ERR(xpd, "Failed to create proc '%s'\n", PROC_PRI_INFO_FNAME);
-		goto err;
-	}
-	SET_PROC_DIRENTRY_OWNER(priv->pri_info);
-	priv->pri_info->write_proc = proc_pri_info_write;
-	priv->pri_info->read_proc = proc_pri_info_read;
-	priv->pri_info->data = xpd;
-#endif
-	return 0;
-err:
-	pri_proc_remove(xbus, xpd);
-	return -EINVAL;
-}
-#endif
-
 static bool valid_pri_modes(const xpd_t *xpd)
 {
 	struct PRI_priv_data	*priv;
@@ -553,7 +498,7 @@ static bool valid_pri_modes(const xpd_t *xpd)
 	return 1;
 }
 
-static void PRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
+static void PRI_card_pcm_recompute(xpd_t *xpd,
 		xpp_line_t pcm_mask)
 {
 	struct PRI_priv_data	*priv;
@@ -564,10 +509,10 @@ static void PRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
-	spin_lock_irqsave(&xpd->lock_recompute_pcm, flags);
+	spin_lock_irqsave(&PHONEDEV(xpd).lock_recompute_pcm, flags);
 	//XPD_DBG(SIGNAL, xpd, "pcm_mask=0x%X\n", pcm_mask);
 	/* Add/remove all the trivial cases */
-	pcm_mask |= xpd->offhook_state;
+	pcm_mask |= PHONEDEV(xpd).offhook_state;
 	for_each_line(xpd, i)
 		if (IS_SET(pcm_mask, i))
 			line_count++;
@@ -591,7 +536,7 @@ static void PRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
 			line_count * DAHDI_CHUNKSIZE
 		: 0L;
 	update_wanted_pcm_mask(xpd, pcm_mask, pcm_len);
-	spin_unlock_irqrestore(&xpd->lock_recompute_pcm, flags);
+	spin_unlock_irqrestore(&PHONEDEV(xpd).lock_recompute_pcm, flags);
 }
 
 /*
@@ -611,7 +556,7 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 	priv = xpd->priv;
 	if(SPAN_REGISTERED(xpd)) {
 		XPD_NOTICE(xpd, "Registered as span %d. Cannot do setup pri protocol (%s)\n",
-			xpd->span.spanno, __FUNCTION__);
+			PHONEDEV(xpd).span.spanno, __FUNCTION__);
 		return -EBUSY;
 	}
 	if(priv->pri_protocol != PRI_PROTO_0) {
@@ -651,16 +596,16 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 	}
 	priv->pri_protocol = set_proto;
 	priv->is_cas = -1;
-	xpd->channels = pri_num_channels(set_proto);
-	xpd->offhook_state = BITMASK(xpd->channels);
-	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+	PHONEDEV(xpd).channels = pri_num_channels(set_proto);
+	PHONEDEV(xpd).offhook_state = BITMASK(PHONEDEV(xpd).channels);
+	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	priv->deflaw = deflaw;
 	priv->dchan_num = dchan_num;
 	priv->local_loopback = 0;
 	xpd->type_name = type_name(priv->pri_protocol);
 	XPD_DBG(GENERAL, xpd, "%s, channels=%d, dchan_num=%d, deflaw=%d\n",
 			pri_protocol_name(set_proto),
-			xpd->channels,
+			PHONEDEV(xpd).channels,
 			priv->dchan_num,
 			priv->deflaw
 			);
@@ -693,18 +638,18 @@ static void dahdi_update_syncsrc(xpd_t *xpd)
 		if(priv->clock_source && priv->alarms == 0) {
 			if(best_spanno)
 				XPD_ERR(xpd, "Duplicate XPD's with clock_source=1\n");
-			best_spanno = subxpd->span.spanno;
+			best_spanno = PHONEDEV(subxpd).span.spanno;
 		}
 	}
 	for(i = 0; i < MAX_SLAVES; i++) {
 		subxpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, i);
 		if(!subxpd)
 			continue;
-		if(subxpd->span.syncsrc == best_spanno)
+		if(PHONEDEV(subxpd).span.syncsrc == best_spanno)
 			XPD_DBG(SYNC, xpd, "Setting SyncSource to span %d\n", best_spanno);
 		else
 			XPD_DBG(SYNC, xpd, "Slaving to span %d\n", best_spanno);
-		subxpd->span.syncsrc = best_spanno;
+		PHONEDEV(subxpd).span.syncsrc = best_spanno;
 	}
 }
 
@@ -735,10 +680,10 @@ static void set_clocking(xpd_t *xpd)
 		priv = subxpd->priv;
 		if(priv->alarms != 0)
 			continue;
-		if(subxpd->timing_priority > 0 && subxpd->timing_priority < best_subunit_prio) {
+		if(PHONEDEV(subxpd).timing_priority > 0 && PHONEDEV(subxpd).timing_priority < best_subunit_prio) {
 			best_xpd = subxpd;
 			best_subunit = i;
-			best_subunit_prio = subxpd->timing_priority;
+			best_subunit_prio = PHONEDEV(subxpd).timing_priority;
 		}
 	}
 	/* Now set it */
@@ -780,7 +725,7 @@ static void set_reg_lim0(const char *msg, xpd_t *xpd)
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	is_master_mode = xpd->timing_priority == 0;
+	is_master_mode = PHONEDEV(xpd).timing_priority == 0;
 	localloop = priv->local_loopback;
 	lim0 |= (localloop) ? REG_LIM0_LL : 0;
 	if(is_master_mode)
@@ -802,11 +747,8 @@ static void set_reg_lim0(const char *msg, xpd_t *xpd)
  */
 static int set_master_mode(const char *msg, xpd_t *xpd)
 {
-	struct PRI_priv_data	*priv;
-
 	BUG_ON(!xpd);
 	XPD_DBG(SIGNAL, xpd, "\n");
-	priv = xpd->priv;
 	set_reg_lim0(__FUNCTION__, xpd);
 	set_clocking(xpd);
 	return 0;
@@ -820,7 +762,7 @@ static int set_localloop(xpd_t *xpd, bool localloop)
 	priv = xpd->priv;
 	if(SPAN_REGISTERED(xpd)) {
 		XPD_NOTICE(xpd, "Registered as span %d. Cannot do %s\n",
-			xpd->span.spanno, __FUNCTION__);
+			PHONEDEV(xpd).span.spanno, __FUNCTION__);
 		return -EBUSY;
 	}
 	priv->local_loopback = localloop;
@@ -870,7 +812,7 @@ static void set_rbslines(xpd_t *xpd, int channo)
 		else
 			BIT_SET(new_rbslines, i);
 	}
-	new_rbslines &= BITMASK(xpd->channels);
+	new_rbslines &= BITMASK(PHONEDEV(xpd).channels);
 	modified_lines = priv->rbslines ^ new_rbslines;
 	XPD_DBG(DEVICES, xpd, "RBSLINES-%d(%s): 0x%X\n",
 		channo, pri_protocol_name(priv->pri_protocol), new_rbslines);
@@ -1038,7 +980,7 @@ static int pri_lineconfig(xpd_t *xpd, int lineconfig)
 		force_cas = 1;
 		set_mode_cas(xpd, 1);
 	}
-	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	/*
 	 * E1's can enable CRC checking
 	 * CRC4 is legal only for E1, and it is checked by pri_linecompat()
@@ -1091,17 +1033,19 @@ bad_lineconfig:
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
 
-static int pri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
+static int pri_spanconfig(struct file *file, struct dahdi_span *span,
+			  struct dahdi_lineconfig *lc)
 {
-	xpd_t			*xpd = container_of(span, struct xpd, span);
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct PRI_priv_data	*priv;
 	int			ret;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
-	if(lc->span != xpd->span.spanno) {
+	if(lc->span != PHONEDEV(xpd).span.spanno) {
 		XPD_ERR(xpd, "I am span %d but got spanconfig for span %d\n",
-			xpd->span.spanno, lc->span);
+			PHONEDEV(xpd).span.spanno, lc->span);
 		return -EINVAL;
 	}
 	/*
@@ -1113,7 +1057,7 @@ static int pri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
 	ret = pri_lineconfig(xpd, lc->lineconfig);
 	if(!ret) {
 		span->lineconfig = lc->lineconfig;
-		xpd->timing_priority = lc->sync;
+		PHONEDEV(xpd).timing_priority = lc->sync;
 		set_master_mode("spanconfig", xpd);
 		elect_syncer("PRI-master_mode");
 	}
@@ -1125,9 +1069,11 @@ static int pri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
  * Called from dahdi with spinlock held on chan. Must not call back
  * dahdi functions.
  */
-static int pri_chanconfig(struct dahdi_chan *chan, int sigtype)
+static int pri_chanconfig(struct file *file, struct dahdi_chan *chan,
+			  int sigtype)
 {
-	xpd_t			*xpd = container_of(chan->span, struct xpd, span);
+	struct phonedev	*phonedev = container_of(chan->span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct PRI_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -1139,16 +1085,16 @@ static int pri_chanconfig(struct dahdi_chan *chan, int sigtype)
 	 *        and prevent DChan definitions with CAS.
 	 */
 	if(is_sigtype_dchan(sigtype)) {
-		if(VALID_DCHAN(priv) && DCHAN(priv) != chan->channo) {
+		if (VALID_DCHAN(priv) && DCHAN(priv) != chan->channo) {
 			ERR("channel %d (%s) marked DChan but also channel %d.\n",
 				chan->channo, chan->name, DCHAN(priv));
 			return -EINVAL;
-		} else {
-			XPD_DBG(GENERAL, xpd, "channel %d (%s) marked as DChan\n", chan->channo, chan->name);
-			SET_DCHAN(priv, chan->channo);
-			/* In T1, we don't know before-hand */
-			if(priv->pri_protocol != PRI_PROTO_E1 && priv->is_cas != 0)
-				set_mode_cas(xpd, 0);
+		}
+		XPD_DBG(GENERAL, xpd, "channel %d (%s) marked as DChan\n", chan->channo, chan->name);
+		SET_DCHAN(priv, chan->channo);
+		/* In T1, we don't know before-hand */
+		if(priv->pri_protocol != PRI_PROTO_E1 && priv->is_cas != 0) {
+			set_mode_cas(xpd, 0);
 		}
 	} else {
 		if(chan->channo == 1) {
@@ -1159,7 +1105,7 @@ static int pri_chanconfig(struct dahdi_chan *chan, int sigtype)
 		if(priv->pri_protocol != PRI_PROTO_E1 && priv->is_cas != 1)
 			set_mode_cas(xpd, 1);
 	}
-	if(xpd->span.flags & DAHDI_FLAG_RUNNING) {
+	if(PHONEDEV(xpd).span.flags & DAHDI_FLAG_RUNNING) {
 		XPD_DBG(DEVICES, xpd, "Span is RUNNING. Updating rbslines.\n");
 		set_rbslines(xpd, chan->channo);
 	}
@@ -1188,12 +1134,6 @@ static xpd_t *PRI_card_new(xbus_t *xbus, int unit, int subunit, const xproto_tab
 	priv->pri_protocol = PRI_PROTO_0;		/* Default, changes in set_pri_proto() */
 	priv->deflaw = DAHDI_LAW_DEFAULT;		/* Default, changes in set_pri_proto() */
 	xpd->type_name = type_name(priv->pri_protocol);
-#ifdef	OLD_PROC
-	if(pri_proc_create(xbus, xpd) < 0) {
-		xpd_free(xpd);
-		return NULL;
-	}
-#endif
 	xbus->sync_mode_default = SYNC_MODE_AB;
 	return xpd;
 }
@@ -1202,12 +1142,10 @@ static int PRI_card_init(xbus_t *xbus, xpd_t *xpd)
 {
 	struct PRI_priv_data	*priv;
 	int			ret = 0;
-	xproto_table_t		*proto_table;
 
 	BUG_ON(!xpd);
 	XPD_DBG(GENERAL, xpd, "\n");
 	xpd->type = XPD_TYPE_PRI;
-	proto_table = &PROTO_TABLE(PRI);
 	priv = xpd->priv;
 	if(priv->pri_protocol == PRI_PROTO_0) {
 		/*
@@ -1228,9 +1166,9 @@ static int PRI_card_init(xbus_t *xbus, xpd_t *xpd)
 		goto err;
 	}
 	xpd->type_name = type_name(priv->pri_protocol);
-	xpd->direction = TO_PSTN;
+	PHONEDEV(xpd).direction = TO_PSTN;
 	XPD_DBG(DEVICES, xpd, "%s\n", xpd->type_name);
-	xpd->timing_priority = 1;		/* High priority SLAVE */
+	PHONEDEV(xpd).timing_priority = 1;		/* High priority SLAVE */
 	set_master_mode(__FUNCTION__, xpd);
 	for(ret = 0; ret < NUM_LEDS; ret++) {
 		DO_LED(xpd, ret, PRI_LED_ON);
@@ -1240,23 +1178,14 @@ static int PRI_card_init(xbus_t *xbus, xpd_t *xpd)
 	priv->initialized = 1;
 	return 0;
 err:
-#ifdef	OLD_PROC
-	pri_proc_remove(xbus, xpd);
-#endif
 	XPD_ERR(xpd, "Failed initializing registers (%d)\n", ret);
 	return ret;
 }
 
 static int PRI_card_remove(xbus_t *xbus, xpd_t *xpd)
 {
-	struct PRI_priv_data	*priv;
-
 	BUG_ON(!xpd);
-	priv = xpd->priv;
 	XPD_DBG(GENERAL, xpd, "\n");
-#ifdef	OLD_PROC
-	pri_proc_remove(xbus, xpd);
-#endif
 	return 0;
 }
 
@@ -1284,6 +1213,8 @@ static const struct dahdi_span_ops PRI_span_ops = {
 	.close = xpp_close,
 	.ioctl = xpp_ioctl,
 	.maint = xpp_maint,
+	.echocan_create = xpp_echocan_create,
+	.echocan_name = xpp_echocan_name,
 #ifdef	DAHDI_SYNC_TICK
 	.sync_tick = dahdi_sync_tick,
 #endif
@@ -1309,15 +1240,16 @@ static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	XPD_DBG(GENERAL, xpd, "%s (proto=%s, channels=%d, deflaw=%d)\n",
 		(on)?"on":"off",
 		pri_protocol_name(priv->pri_protocol),
-		xpd->channels,
+		PHONEDEV(xpd).channels,
 		priv->deflaw);
 	if(!on) {
 		/* Nothing to do yet */
 		return 0;
 	}
-	xpd->span.spantype = pri_protocol_name(priv->pri_protocol);
-	xpd->span.linecompat = pri_linecompat(priv->pri_protocol);
-	xpd->span.deflaw = priv->deflaw;
+	PHONEDEV(xpd).span.spantype = pri_protocol_name(priv->pri_protocol);
+	PHONEDEV(xpd).span.linecompat = pri_linecompat(priv->pri_protocol);
+	PHONEDEV(xpd).span.deflaw = priv->deflaw;
+	PHONEDEV(xpd).span.alarms = DAHDI_ALARM_NONE;
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*cur_chan = XPD_CHAN(xpd, i);
 		bool		is_dchan = i == PRI_DCHAN_IDX(priv);
@@ -1334,19 +1266,17 @@ static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 			cur_chan->flags &= ~DAHDI_FLAG_HDLC;
 		}
 	}
-	xpd->offhook_state = xpd->wanted_pcm_mask;
-	xpd->span.ops = &PRI_span_ops;
+	PHONEDEV(xpd).offhook_state = PHONEDEV(xpd).wanted_pcm_mask;
+	PHONEDEV(xpd).span.ops = &PRI_span_ops;
 	return 0;
 }
 
 static int PRI_card_dahdi_postregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
-	struct PRI_priv_data	*priv;
 	
 	BUG_ON(!xpd);
 	xbus = xpd->xbus;
-	priv = xpd->priv;
 	BUG_ON(!xbus);
 	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"on":"off");
 	dahdi_update_syncsrc(xpd);
@@ -1373,7 +1303,7 @@ static void dchan_state(xpd_t *xpd, bool up)
 	} else {
 		int	d = PRI_DCHAN_IDX(priv);
 
-		if(SPAN_REGISTERED(xpd) && d >= 0 && d < xpd->channels) {
+		if(SPAN_REGISTERED(xpd) && d >= 0 && d < PHONEDEV(xpd).channels) {
 			byte	*pcm;
 
 			pcm = (byte *)XPD_CHAN(xpd, d)->readchunk;
@@ -1407,7 +1337,7 @@ static void handle_leds(xbus_t *xbus, xpd_t *xpd)
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	if(xpd->timing_priority == 0) {
+	if(PHONEDEV(xpd).timing_priority == 0) {
 		which_led = TOP_RED_LED;
 		other_led = BOTTOM_GREEN_LED;
 	} else {
@@ -1529,9 +1459,10 @@ static int PRI_card_close(xpd_t *xpd, lineno_t pos)
 /*
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
-static int pri_startup(struct dahdi_span *span)
+static int pri_startup(struct file *file, struct dahdi_span *span)
 {
-	xpd_t			*xpd = container_of(span, struct xpd, span);
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct PRI_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -1543,7 +1474,7 @@ static int pri_startup(struct dahdi_span *span)
 	}
 	XPD_DBG(GENERAL, xpd, "STARTUP\n");
 	// Turn on all channels
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 1);
+	CALL_PHONE_METHOD(card_state, xpd, 1);
 	set_rbslines(xpd, 0);
 	write_subunit(xpd, REG_XPM2, 0x00);
 	return 0;
@@ -1554,7 +1485,8 @@ static int pri_startup(struct dahdi_span *span)
  */
 static int pri_shutdown(struct dahdi_span *span)
 {
-	xpd_t			*xpd = container_of(span, struct xpd, span);
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct PRI_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -1566,7 +1498,7 @@ static int pri_shutdown(struct dahdi_span *span)
 	}
 	XPD_DBG(GENERAL, xpd, "SHUTDOWN\n");
 	// Turn off all channels
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 0);
+	CALL_PHONE_METHOD(card_state, xpd, 0);
 	return 0;
 }
 
@@ -1615,7 +1547,7 @@ static int encode_rbsbits_t1(xpd_t *xpd, int pos, int bits)
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	BUG_ON(priv->pri_protocol != PRI_PROTO_T1);
-	if(pos < 0 || pos >= xpd->channels) {
+	if(pos < 0 || pos >= PHONEDEV(xpd).channels) {
 		XPD_ERR(xpd, "%s: Bad pos=%d\n", __FUNCTION__, pos);
 		return 0;
 	}
@@ -1687,9 +1619,6 @@ static int pri_rbsbits(struct dahdi_chan *chan, int bits)
 	pos = chan->chanpos - 1;
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	if(!priv->layer1_up) {
-		XPD_DBG(SIGNAL, xpd, "RBS: TX: No layer1 yet. Keep going.\n");
-	}
 	if(!priv->is_cas) {
 		XPD_DBG(SIGNAL, xpd, "RBS: TX: not in CAS mode. Ignore.\n");
 		return 0;
@@ -1700,6 +1629,9 @@ static int pri_rbsbits(struct dahdi_chan *chan, int bits)
 				sig2str(chan->sig), bits);
 		return 0;
 	}
+	if(!priv->layer1_up) {
+		XPD_DBG(SIGNAL, xpd, "RBS: TX: No layer1 yet. Keep going.\n");
+	}
 	if(priv->pri_protocol == PRI_PROTO_E1) {
 		if(encode_rbsbits_e1(xpd, pos, bits) < 0)
 			return -EINVAL;
@@ -1725,7 +1657,7 @@ static int pri_rbsbits(struct dahdi_chan *chan, int bits)
  * send 31 channels to the device, but they should be called 1-31 rather
  * than 0-30 .
  */
-static void PRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+static void PRI_card_pcm_fromspan(xpd_t *xpd, xpacket_t *pack)
 {
 	struct PRI_priv_data	*priv;
 	byte			*pcm;
@@ -1735,14 +1667,13 @@ static void PRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	int			physical_chan;
 	int			physical_mask = 0;
 
-	BUG_ON(!xbus);
 	BUG_ON(!xpd);
 	BUG_ON(!pack);
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	pcm = RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, pcm);
 	spin_lock_irqsave(&xpd->lock, flags);
-	wanted_lines = xpd->wanted_pcm_mask;
+	wanted_lines = PHONEDEV(xpd).wanted_pcm_mask;
 	physical_chan = 0;
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*chan = XPD_CHAN(xpd, i);
@@ -1799,7 +1730,7 @@ static void PRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
  *
  * \see PRI_card_pcm_fromspan
  */
-static void PRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+static void PRI_card_pcm_tospan(xpd_t *xpd, xpacket_t *pack)
 {
 	struct PRI_priv_data	*priv;
 	byte			*pcm;
@@ -1852,27 +1783,53 @@ static void PRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	spin_unlock_irqrestore(&xpd->lock, flags);
 }
 
-int PRI_timing_priority(xbus_t *xbus, xpd_t *xpd)
+static int PRI_timing_priority(xpd_t *xpd)
 {
 	struct PRI_priv_data	*priv;
 
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	if (priv->layer1_up)
-		return xpd->timing_priority;
+		return PHONEDEV(xpd).timing_priority;
 	XPD_DBG(SYNC, xpd, "No timing priority (no layer1)\n");
 	return -ENOENT;
 }
 
-/*---------------- PRI: HOST COMMANDS -------------------------------------*/
-
-static /* 0x0F */ HOSTCMD(PRI, XPD_STATE, bool on)
+static int PRI_echocancel_timeslot(xpd_t *xpd, int pos)
 {
+	/*
+	 * Skip ts=0 (used for PRI sync)
+	 */
+	return (1 + pos) * 4 + xpd->addr.subunit;
+}
+
+static int PRI_echocancel_setmask(xpd_t *xpd, xpp_line_t ec_mask)
+{
+	struct PRI_priv_data *priv;
+	int i;
+
 	BUG_ON(!xpd);
-	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"on":"off");
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	XPD_DBG(GENERAL, xpd, "0x%8X\n", ec_mask);
+	if (!ECHOOPS(xpd->xbus)) {
+		XPD_DBG(GENERAL, xpd,
+				"No echo canceller in XBUS: Doing nothing.\n");
+		return -EINVAL;
+	}
+	for (i = 0; i < PHONEDEV(xpd).channels; i++) {
+		int	on = BIT(i) & ec_mask;
+
+		if (i == PRI_DCHAN_IDX(priv))
+			on = 0;
+		CALL_EC_METHOD(ec_set, xpd->xbus, xpd, i, on);
+	}
+	CALL_EC_METHOD(ec_update, xpd->xbus, xpd->xbus);
 	return 0;
 }
 
+/*---------------- PRI: HOST COMMANDS -------------------------------------*/
+
 static /* 0x33 */ HOSTCMD(PRI, SET_LED, enum pri_led_selectors led_sel, enum pri_led_state to_led_state)
 {
 	int			ret = 0;
@@ -1938,24 +1895,24 @@ static void layer1_state(xpd_t *xpd, byte data_low)
 			query_subunit(xpd, regbase + i);
 	}
 
-	if(SPAN_REGISTERED(xpd) && xpd->span.alarms != alarms) {
+	if(SPAN_REGISTERED(xpd) && PHONEDEV(xpd).span.alarms != alarms) {
 		char	str1[MAX_PROC_WRITE];
 		char	str2[MAX_PROC_WRITE];
 
-		alarm2str(xpd->span.alarms, str1, sizeof(str1));
+		alarm2str(PHONEDEV(xpd).span.alarms, str1, sizeof(str1));
 		alarm2str(alarms, str2, sizeof(str2));
 		XPD_NOTICE(xpd, "Alarms: 0x%X (%s) => 0x%X (%s)\n",
-				xpd->span.alarms, str1,
+				PHONEDEV(xpd).span.alarms, str1,
 				alarms, str2);
 		if (priv->is_cas) {
 			if (alarms == DAHDI_ALARM_NONE)
 				send_oldbits(xpd);
-			else if (xpd->span.alarms == DAHDI_ALARM_NONE)
+			else if (PHONEDEV(xpd).span.alarms == DAHDI_ALARM_NONE)
 				send_idlebits(xpd, 1);
 		}
-		xpd->span.alarms = alarms;
+		PHONEDEV(xpd).span.alarms = alarms;
 		elect_syncer("LAYER1");
-		dahdi_alarm_notify(&xpd->span);
+		dahdi_alarm_notify(&PHONEDEV(xpd).span);
 		set_clocking(xpd);
 	}
 	priv->reg_frs0 = data_low;
@@ -1977,18 +1934,18 @@ static int decode_cas_e1(xpd_t *xpd, byte regnum, byte data_low)
 	BUG_ON(!priv->is_cas);
 	BUG_ON(priv->pri_protocol != PRI_PROTO_E1);
 	XPD_DBG(SIGNAL, xpd, "RBS: RX: data_low=0x%02X\n", data_low);
-	if(pos < 0 || pos >= NUM_CAS_RS_E) {
+	if(pos >= NUM_CAS_RS_E) {
 		XPD_ERR(xpd, "%s: got bad pos=%d [0-%d]\n", __FUNCTION__, pos, NUM_CAS_RS_E);
 		return -EINVAL;
 	}
-	if(chan1 < 0 || chan1 > xpd->channels) {
+	if(chan1 < 0 || chan1 > PHONEDEV(xpd).channels) {
 		XPD_NOTICE(xpd, "%s: %s CAS: Bad chan1 number (%d)\n",
 			__FUNCTION__,
 			pri_protocol_name(priv->pri_protocol),
 			chan1);
 		return -EINVAL;
 	}
-	if(chan2 < 0 || chan2 > xpd->channels) {
+	if(chan2 < 0 || chan2 > PHONEDEV(xpd).channels) {
 		XPD_NOTICE(xpd, "%s: %s CAS: Bad chan2 number (%d)\n",
 			__FUNCTION__,
 			pri_protocol_name(priv->pri_protocol),
@@ -2018,7 +1975,7 @@ static int decode_cas_t1(xpd_t *xpd, byte regnum, byte data_low)
 	BUG_ON(!priv->is_cas);
 	BUG_ON(priv->pri_protocol != PRI_PROTO_T1);
 	rsnum = regnum - REG_RS1_E;
-	if(rsnum < 0 || rsnum >= 12) {
+	if(rsnum >= 12) {
 		XPD_ERR(xpd, "Bad rsnum=%d\n", rsnum);
 		return 0;
 	}
@@ -2039,7 +1996,7 @@ static int decode_cas_t1(xpd_t *xpd, byte regnum, byte data_low)
 		if (!priv->is_esf)
 			rxsig <<= 2;
 		pos = rsnum * chan_per_reg + chan_per_reg - i - 1;
-		if(pos < 0 || pos >= xpd->channels) {
+		if(pos < 0 || pos >= PHONEDEV(xpd).channels) {
 			XPD_ERR(xpd, "%s: Bad pos=%d\n", __FUNCTION__, pos);
 			continue;
 		}
@@ -2166,6 +2123,35 @@ end:
 	return 0;
 }
 
+static int PRI_card_state(xpd_t *xpd, bool on)
+{
+	BUG_ON(!xpd);
+	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"on":"off");
+	return 0;
+}
+
+static const struct xops	pri_xops = {
+	.card_new	= PRI_card_new,
+	.card_init	= PRI_card_init,
+	.card_remove	= PRI_card_remove,
+	.card_tick	= PRI_card_tick,
+	.card_register_reply	= PRI_card_register_reply,
+};
+
+static const struct phoneops	pri_phoneops = {
+	.card_dahdi_preregistration	= PRI_card_dahdi_preregistration,
+	.card_dahdi_postregistration	= PRI_card_dahdi_postregistration,
+	.card_pcm_recompute	= PRI_card_pcm_recompute,
+	.card_pcm_fromspan	= PRI_card_pcm_fromspan,
+	.card_pcm_tospan	= PRI_card_pcm_tospan,
+	.echocancel_timeslot	= PRI_echocancel_timeslot,
+	.echocancel_setmask	= PRI_echocancel_setmask,
+	.card_timing_priority	= PRI_timing_priority,
+	.card_ioctl	= PRI_card_ioctl,
+	.card_close	= PRI_card_close,
+	.card_state	= PRI_card_state,
+};
+
 static xproto_table_t PROTO_TABLE(PRI) = {
 	.owner = THIS_MODULE,
 	.entries = {
@@ -2174,23 +2160,8 @@ static xproto_table_t PROTO_TABLE(PRI) = {
 	.name = "PRI",	/* protocol name */
 	.ports_per_subunit = 1,
 	.type = XPD_TYPE_PRI,
-	.xops = {
-		.card_new	= PRI_card_new,
-		.card_init	= PRI_card_init,
-		.card_remove	= PRI_card_remove,
-		.card_dahdi_preregistration	= PRI_card_dahdi_preregistration,
-		.card_dahdi_postregistration	= PRI_card_dahdi_postregistration,
-		.card_tick	= PRI_card_tick,
-		.card_pcm_recompute	= PRI_card_pcm_recompute,
-		.card_pcm_fromspan	= PRI_card_pcm_fromspan,
-		.card_pcm_tospan	= PRI_card_pcm_tospan,
-		.card_timing_priority	= PRI_timing_priority,
-		.card_ioctl	= PRI_card_ioctl,
-		.card_close	= PRI_card_close,
-		.card_register_reply	= PRI_card_register_reply,
-
-		.XPD_STATE	= XPROTO_CALLER(PRI, XPD_STATE),
-	},
+	.xops = &pri_xops,
+	.phoneops = &pri_phoneops,
 	.packet_is_valid = pri_packet_is_valid,
 	.packet_dump = pri_packet_dump,
 };
@@ -2208,154 +2179,6 @@ static void pri_packet_dump(const char *msg, xpacket_t *pack)
 	DBG(GENERAL, "%s\n", msg);
 }
 /*------------------------- REGISTER Handling --------------------------*/
-#ifdef	OLD_PROC
-static int proc_pri_info_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
-{
-	xpd_t			*xpd = data;
-	struct PRI_priv_data	*priv;
-	char			buf[MAX_PROC_WRITE];
-	char			*p;
-	char			*tok;
-	int			ret = 0;
-	bool			got_localloop = 0;
-	bool			got_nolocalloop = 0;
-	bool			got_e1 = 0;
-	bool			got_t1 = 0;
-	bool			got_j1 = 0;
-
-	if(!xpd)
-		return -ENODEV;
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] write to /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	priv = xpd->priv;
-	if(count >= MAX_PROC_WRITE) {	/* leave room for null */
-		XPD_ERR(xpd, "write too long (%ld)\n", count);
-		return -E2BIG;
-	}
-	if(copy_from_user(buf, buffer, count)) {
-		XPD_ERR(xpd, "Failed reading user data\n");
-		return -EFAULT;
-	}
-	buf[count] = '\0';
-	XPD_DBG(PROC, xpd, "PRI-SETUP: got %s\n", buf);
-	/*
-	 * First parse. Act only of *everything* is good.
-	 */
-	p = buf;
-	while((tok = strsep(&p, " \t\v\n")) != NULL) {
-		if(*tok == '\0')
-			continue;
-		XPD_DBG(PROC, xpd, "Got token='%s'\n", tok);
-		if(strnicmp(tok, "LOCALLOOP", 8) == 0)
-			got_localloop = 1;
-		else if(strnicmp(tok, "NOLOCALLOOP", 8) == 0)
-			got_nolocalloop = 1;
-		else if(strnicmp(tok, "E1", 2) == 0)
-			got_e1 = 1;
-		else if(strnicmp(tok, "T1", 2) == 0)
-			got_t1 = 1;
-		else if(strnicmp(tok, "J1", 2) == 0) {
-			got_j1 = 1;
-		} else {
-			XPD_NOTICE(xpd, "PRI-SETUP: unknown keyword: '%s'\n", tok);
-			return -EINVAL;
-		}
-	}
-	if(got_e1)
-		ret = set_pri_proto(xpd, PRI_PROTO_E1);
-	else if(got_t1)
-		ret = set_pri_proto(xpd, PRI_PROTO_T1);
-	else if(got_j1)
-		ret = set_pri_proto(xpd, PRI_PROTO_J1);
-	if(priv->pri_protocol == PRI_PROTO_0) {
-		XPD_ERR(xpd,
-			"Must set PRI protocol (E1/T1/J1) before setting other parameters\n");
-		return -EINVAL;
-	}
-	if(got_localloop)
-		ret = set_localloop(xpd, 1);
-	if(got_nolocalloop)
-		ret = set_localloop(xpd, 0);
-	return (ret) ? ret : count;
-}
-
-
-static int proc_pri_info_read(char *page, char **start, off_t off, int count, int *eof, void *data)
-{
-	int			len = 0;
-	unsigned long		flags;
-	xpd_t			*xpd = data;
-	struct PRI_priv_data	*priv;
-	int			i;
-
-	DBG(PROC, "\n");
-	if(!xpd)
-		return -ENODEV;
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] read from /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	spin_lock_irqsave(&xpd->lock, flags);
-	priv = xpd->priv;
-	BUG_ON(!priv);
-	len += sprintf(page + len, "PRI: %s %s%s (deflaw=%d, dchan=%d)\n",
-		(priv->clock_source) ? "MASTER" : "SLAVE",
-		pri_protocol_name(priv->pri_protocol),
-		(priv->local_loopback) ? " LOCALLOOP" : "",
-		priv->deflaw, priv->dchan_num);
-	len += sprintf(page + len, "%05d Layer1: ", priv->layer1_replies);
-	if(priv->poll_noreplies > 1)
-		len += sprintf(page + len, "No Replies [%d]\n",
-			priv->poll_noreplies);
-	else {
-		len += sprintf(page + len, "%s\n",
-				((priv->layer1_up) ?  "UP" : "DOWN"));
-		len += sprintf(page + len,
-				"Framer Status: FRS0=0x%02X, FRS1=0x%02X ALARMS:",
-				priv->reg_frs0, priv->reg_frs1);
-		if(priv->reg_frs0 & REG_FRS0_LOS)
-			len += sprintf(page + len, " RED");
-		if(priv->reg_frs0 & REG_FRS0_AIS)
-			len += sprintf(page + len, " BLUE");
-		if(priv->reg_frs0 & REG_FRS0_RRA)
-			len += sprintf(page + len, " YELLOW");
-		len += sprintf(page + len, "\n");
-	}
-	if(priv->is_cas) {
-		len += sprintf(page + len,
-			"CAS: replies=%d\n", priv->cas_replies);
-		len += sprintf(page + len, "   CAS-TS: ");
-		for(i = 0; i < NUM_CAS_RS_E; i++) {
-			len += sprintf(page + len, " %02X", priv->cas_ts_e[i]);
-		}
-		len += sprintf(page + len, "\n");
-		len += sprintf(page + len, "   CAS-RS: ");
-		for(i = 0; i < NUM_CAS_RS_E; i++) {
-			len += sprintf(page + len, " %02X", priv->cas_rs_e[i]);
-		}
-		len += sprintf(page + len, "\n");
-	}
-	len += sprintf(page + len, "D-Channel: TX=[%5d] (0x%02X)   RX=[%5d] (0x%02X) ",
-			priv->dchan_tx_counter, priv->dchan_tx_sample,
-			priv->dchan_rx_counter, priv->dchan_rx_sample);
-	if(priv->dchan_alive) {
-		len += sprintf(page + len, "(alive %d K-ticks)\n",
-			priv->dchan_alive_ticks/1000);
-	} else {
-		len += sprintf(page + len, "(dead)\n");
-	}
-	for(i = 0; i < NUM_LEDS; i++)
-		len += sprintf(page + len, "LED #%d: %d\n", i, priv->ledstate[i]);
-	spin_unlock_irqrestore(&xpd->lock, flags);
-	if (len <= off+count)
-		*eof = 1;
-	*start = page + off;
-	len -= off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-	return len;
-}
-#endif
 
 /*------------------------- sysfs stuff --------------------------------*/
 static DEVICE_ATTR_READER(pri_protocol_show, dev, buf)
@@ -2498,7 +2321,7 @@ static DEVICE_ATTR_READER(pri_alarms_show, dev, buf)
 	struct PRI_priv_data	*priv;
 	unsigned long		flags;
 	int			len = 0;
-	const static struct {
+	static const struct {
 		byte		bits;
 		const char	*name;
 	} alarm_types[] = {
@@ -2617,7 +2440,6 @@ static DEVICE_ATTR_READER(pri_clocking_show, dev, buf)
 
 static	DEVICE_ATTR(pri_clocking, S_IRUGO, pri_clocking_show, NULL);
 
-
 static int pri_xpd_probe(struct device *dev)
 {
 	xpd_t	*xpd;
