@@ -40,20 +40,14 @@
 #include "xpp_dahdi.h"
 #include "xbus-core.h"
 #include "card_global.h"
-#ifdef	XPP_DEBUGFS
-#include "xpp_log.h"
-#endif
 #include "dahdi_debug.h"
 
-static const char rcsid[] = "$Id: xbus-core.c 9676 2011-01-21 05:30:39Z sruffell $";
+static const char rcsid[] = "$Id: xbus-core.c 10055 2011-07-20 16:49:53Z tzafrir $";
 
 /* Defines */
 #define	INITIALIZATION_TIMEOUT	(90*HZ)		/* in jiffies */
 #define	PROC_XBUSES		"xbuses"
 #define	PROC_XBUS_SUMMARY	"summary"
-#ifdef	OLD_PROC
-#define	PROC_XBUS_WAITFOR_XPDS	"waitfor_xpds"
-#endif
 
 #ifdef	PROTOCOL_DEBUG
 #ifdef	CONFIG_PROC_FS
@@ -64,21 +58,18 @@ static int proc_xbus_command_write(struct file *file, const char __user *buffer,
 
 /* Command line parameters */
 extern int debug;
-static DEF_PARM(uint, command_queue_length, 800, 0444, "Maximal command queue length");
+static DEF_PARM(uint, command_queue_length, 1000, 0444, "Maximal command queue length");
 static DEF_PARM(uint, poll_timeout, 1000, 0644, "Timeout (in jiffies) waiting for units to reply");
 static DEF_PARM_BOOL(rx_tasklet, 0, 0644, "Use receive tasklets");
 
 #ifdef	CONFIG_PROC_FS
 static int xbus_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data);
-#ifdef	OLD_PROC
-static int xbus_read_waitfor_xpds(char *page, char **start, off_t off, int count, int *eof, void *data);
-#endif
 #endif
 static void transport_init(xbus_t *xbus, struct xbus_ops *ops, ushort max_send_size, struct device *transport_device, void *priv);
 static void transport_destroy(xbus_t *xbus);
 
 /* Data structures */
-static spinlock_t		xbuses_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(xbuses_lock);
 #ifdef	CONFIG_PROC_FS
 static struct proc_dir_entry	*proc_xbuses = NULL;
 #endif
@@ -104,7 +95,7 @@ int xbus_check_unique(xbus_t *xbus)
 {
 	if (!xbus)
 		return -ENOENT;
-	if (xbus->label) {
+	if (xbus->label && *(xbus->label)) {
 		xbus_t	*xbus_old;
 
 		XBUS_DBG(DEVICES, xbus, "Checking LABEL='%s'\n", xbus->label);
@@ -118,7 +109,7 @@ int xbus_check_unique(xbus_t *xbus)
 			return -EBUSY;
 		}
 	} else {
-		XBUS_NOTICE(xbus, "MISSING BOARD LABEL!!!\n");
+		XBUS_NOTICE(xbus, "Missing board label (old Astribank?)\n");
 	}
 	return 0;
 }
@@ -219,155 +210,6 @@ int refcount_xbus(xbus_t *xbus)
 	return atomic_read(&kref->refcount);
 }
 
-/*------------------------- Debugfs Handling -----------------------*/
-#ifdef	XPP_DEBUGFS
-
-#define DEBUGFS_BUFSIZ		4096	/* must be power of two, otherwise POS_IN_BUF will have to use '%' instead of '&' */
-#define POS_IN_BUF(x)		((x) & (DEBUGFS_BUFSIZ-1))
-
-struct debugfs_data {
-	spinlock_t lock;
-	xbus_t *xbus;
-	char buffer[DEBUGFS_BUFSIZ];
-	unsigned long head, tail;	/* reading and writing are performed at position (head % BUF_SIZ) and (tail % BUF_SIZ) */
-	wait_queue_head_t queue;
-};
-
-static unsigned long add_to_buf(struct debugfs_data *d, unsigned long tail, const void *buf, unsigned long len)
-{
-	unsigned long count = min(len, (unsigned long)(DEBUGFS_BUFSIZ - POS_IN_BUF(tail)));
-	memcpy(d->buffer + POS_IN_BUF(tail), buf, count);		/* fill starting at position tail */
-	memcpy(d->buffer, (u_char *)buf + count, len - count);		/* fill leftover */
-	return len;
-}
-
-int xbus_log(xbus_t *xbus, xpd_t *xpd, int direction, const void *buf, unsigned long len)
-{
-	unsigned long tail;
-	unsigned long flags;
-	struct debugfs_data *d;
-	struct log_header header;
-	int ret = 0;
-	
-	BUG_ON(!xbus);
-	BUG_ON(!xpd);
-	BUG_ON(sizeof(struct log_header) + len > DEBUGFS_BUFSIZ);
-	d = xbus->debugfs_data;
-	if (!d)			/* no consumer process */
-		return ret;
-	spin_lock_irqsave(&d->lock, flags);
-	if (sizeof(struct log_header) + len > DEBUGFS_BUFSIZ - (d->tail - d->head)) {
-		ret = -ENOSPC;
-		XPD_DBG(GENERAL, xpd, "Dropping debugfs data of len %lu, free space is %lu\n", sizeof(struct log_header) + len,
-				DEBUGFS_BUFSIZ - (d->tail - d->head));
-		goto out;
-	}
-	header.len = sizeof(struct log_header) + len;
-	header.time = jiffies_to_msecs(jiffies);
-	header.xpd_num = xpd->xbus_idx;
-	header.direction = (char)direction;
-	tail = d->tail;
-	tail += add_to_buf(d, tail, &header, sizeof(header));
-	tail += add_to_buf(d, tail, buf, len);
-	d->tail = tail;
-	wake_up_interruptible(&d->queue);
-out:
-	spin_unlock_irqrestore(&d->lock, flags);
-	return ret;
-}
-
-static struct dentry	*debugfs_root = NULL;
-static int debugfs_open(struct inode *inode, struct file *file);
-static ssize_t debugfs_read(struct file *file, char __user *buf, size_t nbytes, loff_t *ppos);
-static int debugfs_release(struct inode *inode, struct file *file);
-
-static struct file_operations debugfs_operations = {
-	.open		= debugfs_open,
-	.read		= debugfs_read,
-	.release	= debugfs_release,
-};
-
-/*
- * As part of the "inode diet" the private data member of struct inode
- * has changed in 2.6.19. However, Fedore Core 6 adopted this change
- * a bit earlier (2.6.18). If you use such a kernel, Change the 
- * following test from 2,6,19 to 2,6,18.
- */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
-#define	I_PRIVATE(inode)	((inode)->u.generic_ip)
-#else
-#define	I_PRIVATE(inode)	((inode)->i_private)
-#endif
-
-static int debugfs_open(struct inode *inode, struct file *file)
-{
-	xbus_t	*xbus = I_PRIVATE(inode);
-	struct debugfs_data *d;
-	struct log_global_header gheader;
-
-	BUG_ON(!xbus);
-	XBUS_DBG(GENERAL, xbus, "\n");
-	if (xbus->debugfs_data)
-		return -EBUSY;
-	d = KZALLOC(sizeof(struct debugfs_data), GFP_KERNEL);
-	if (!d)
-		return -ENOMEM;
-	try_module_get(THIS_MODULE);
-	spin_lock_init(&d->lock);
-	d->xbus = xbus;
-	d->head = d->tail = 0;
-	init_waitqueue_head(&d->queue);
-	file->private_data = d;
-
-	gheader.magic = XPP_LOG_MAGIC;
-	gheader.version = 1;
-	d->tail += add_to_buf(d, d->tail, &gheader, sizeof(gheader));
-
-	xbus->debugfs_data = d;
-	return 0;
-}
-
-static ssize_t debugfs_read(struct file *file, char __user *buf, size_t nbytes, loff_t *ppos)
-{
-	struct debugfs_data *d = file->private_data;
-	size_t len;
-
-	BUG_ON(!d);
-	BUG_ON(!d->xbus);
-	XBUS_DBG(GENERAL, d->xbus, "\n");
-	while (d->head == d->tail) {
-		if (wait_event_interruptible(d->queue, d->head != d->tail))
-			return -EAGAIN;
-	}
-	len = min(nbytes, (size_t)(d->tail - d->head));
-	if (copy_to_user(buf, d->buffer + POS_IN_BUF(d->head), len))
-		return -EFAULT;
-	d->head += len;
-	/* optimization to avoid future buffer wraparound */
-	if (d->head == d->tail) {
-		unsigned long flags;
-		spin_lock_irqsave(&d->lock, flags);
-		if (d->head == d->tail)
-			d->head = d->tail = 0;
-		spin_unlock_irqrestore(&d->lock, flags);
-	}
-	return len;
-}
-
-static int debugfs_release(struct inode *inode, struct file *file)
-{
-	struct debugfs_data *d = file->private_data;
-
-	BUG_ON(!d);
-	BUG_ON(!d->xbus);
-	XBUS_DBG(GENERAL, d->xbus, "\n");
-	d->xbus->debugfs_data = NULL;
-	KZFREE(d);
-	module_put(THIS_MODULE);
-	return 0;
-}
-#endif
-
 /*------------------------- Frame  Handling ------------------------*/
 
 void xframe_init(xbus_t *xbus, xframe_t *xframe, void *buf, size_t maxsize, void *priv)
@@ -405,7 +247,7 @@ xpacket_t *xframe_next_packet(xframe_t *frm, int len)
 	return (xpacket_t *)(frm->packets + newlen - len);
 }
 
-static spinlock_t serialize_dump_xframe = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(serialize_dump_xframe);
 
 static void do_hexdump(const char msg[], byte *data, uint16_t len)
 {
@@ -784,11 +626,11 @@ static int new_card(xbus_t *xbus,
 		byte port_dir)
 {
 	const xproto_table_t	*proto_table;
-	const xops_t		*xops;
 	int			i;
 	int			subunits;
 	int			ret = 0;
 	int			remaining_ports;
+	const struct echoops	*echoops;
 
 	proto_table = xproto_get(type);
 	if(!proto_table) {
@@ -797,6 +639,18 @@ static int new_card(xbus_t *xbus,
 			unit, type);
 		return -EINVAL;
 	}
+	echoops = proto_table->echoops;
+	if (echoops) {
+		XBUS_INFO(xbus, "Detected ECHO Canceler (%d)\n", unit);
+		if (ECHOOPS(xbus)) {
+			XBUS_NOTICE(xbus,
+				"CARD %d: tryies to define echoops (type %d) but we already have one. Ignored.\n",
+				unit, type);
+			return -EINVAL;
+		}
+		xbus->echo_state.echoops = echoops;
+		xbus->echo_state.xpd_idx = XPD_IDX(unit, 0);
+	}
 	remaining_ports = ports;
 	subunits = (ports + proto_table->ports_per_subunit - 1) /
 			proto_table->ports_per_subunit;
@@ -810,8 +664,10 @@ static int new_card(xbus_t *xbus,
 			subunits,
 			port_dir
 		);
-	xops = &proto_table->xops;
-	BUG_ON(!xops);
+	if (type == XPD_TYPE_PRI || type == XPD_TYPE_BRI)
+		xbus->quirks.has_digital_span = 1;
+	if (type == XPD_TYPE_FXO)
+		xbus->quirks.has_fxo = 1;
 	xbus->worker.num_units += subunits - 1;
 	for(i = 0; i < subunits; i++) {
 		int	subunit_ports = proto_table->ports_per_subunit;
@@ -843,12 +699,6 @@ static int new_card(xbus_t *xbus,
 				i,
 				type,
 				subtype, subunit_ports);
-		if(!XBUS_IS(xbus, RECVD_DESC)) {
-			XBUS_ERR(xbus, "Aborting creation -- In bad state %s\n",
-				xbus_statename(XBUS_STATE(xbus)));
-			ret = -ENODEV;
-			goto out;
-		}
 		ret = create_xpd(xbus, proto_table, unit, i, type, subtype, subunits, subunit_ports, port_dir);
 		if(ret < 0) {
 			XBUS_ERR(xbus, "Creation of XPD=%d%d failed %d\n",
@@ -909,13 +759,19 @@ static int xpd_initialize(xpd_t *xpd)
 {
 	int	ret = -ENODEV;
 
-	if(CALL_XMETHOD(card_init, xpd->xbus, xpd) < 0) {
+	if(CALL_XMETHOD(card_init, xpd) < 0) {
 		XPD_ERR(xpd, "Card Initialization failed\n");
 		goto out;
 	}
-	//CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 0);	/* Turn off all channels */
 	xpd->card_present = 1;
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 1);		/* Turn on all channels */
+	if (IS_PHONEDEV(xpd)) {
+		/*
+		 * Set echo canceler channels (off)
+		 * Asterisk will tell us when/if it's needed.
+		 */
+		CALL_PHONE_METHOD(echocancel_setmask, xpd, 0);
+		CALL_PHONE_METHOD(card_state, xpd, 1);	/* Turn on all channels */
+	}
 	if(!xpd_setstate(xpd, XPD_STATE_READY)) {
 		goto out;
 	}
@@ -926,6 +782,34 @@ out:
 	return ret;
 }
 
+static int xbus_echocancel(xbus_t *xbus, int on)
+{
+	int unit;
+	int subunit;
+	xpd_t *xpd;
+
+	if (!ECHOOPS(xbus))
+		return 0;
+	for (unit = 0; unit < MAX_UNIT; unit++) {
+		xpd = xpd_byaddr(xbus, unit, 0);
+		if (!xpd || !IS_PHONEDEV(xpd))
+			continue;
+		for (subunit = 0; subunit < MAX_SUBUNIT; subunit++) {
+			int	ret;
+
+			xpd = xpd_byaddr(xbus, unit, subunit);
+			if (!xpd || !IS_PHONEDEV(xpd))
+				continue;
+			ret = echocancel_xpd(xpd, on);
+			if (ret < 0) {
+				XPD_ERR(xpd, "Fail in xbus_echocancel()\n");
+				return ret;
+			}
+		}
+	}
+	return 0;
+}
+
 static int xbus_initialize(xbus_t *xbus)
 {
 	int	unit;
@@ -973,6 +857,7 @@ static int xbus_initialize(xbus_t *xbus)
 				goto err;
 		}
 	}
+	xbus_echocancel(xbus, 1);
 	do_gettimeofday(&time_end);
 	timediff = usec_diff(&time_end, &time_start);
 	timediff /= 1000*100;
@@ -1139,16 +1024,6 @@ static void worker_destroy(xbus_t *xbus)
 		worker->wq = NULL;
 		XBUS_DBG(DEVICES, xbus, "destroying workqueue -- done\n");
 	}
-#ifdef CONFIG_PROC_FS
-#ifdef	OLD_PROC
-	if (xbus->proc_xbus_dir && worker->proc_xbus_waitfor_xpds) {
-		XBUS_DBG(PROC, xbus, "Removing proc '%s'\n",
-				PROC_XBUS_WAITFOR_XPDS);
-		remove_proc_entry(PROC_XBUS_WAITFOR_XPDS, xbus->proc_xbus_dir);
-		worker->proc_xbus_waitfor_xpds = NULL;
-	}
-#endif
-#endif
 	XBUS_DBG(DEVICES, xbus, "detach worker\n");
 	put_xbus(__func__, xbus);	/* got from worker_run() */
 }
@@ -1184,22 +1059,6 @@ static int worker_run(xbus_t *xbus)
 	BUG_ON(worker->wq);	/* Hmmm... nested workers? */
 	XBUS_DBG(DEVICES, xbus, "\n");
 	/* poll related variables */
-#ifdef CONFIG_PROC_FS
-#ifdef	OLD_PROC
-	if(xbus->proc_xbus_dir) {
-		worker->proc_xbus_waitfor_xpds = create_proc_read_entry(
-				PROC_XBUS_WAITFOR_XPDS, 0444,
-				xbus->proc_xbus_dir,
-				xbus_read_waitfor_xpds,
-				xbus);
-		if (!worker->proc_xbus_waitfor_xpds) {
-			XBUS_ERR(xbus, "Failed to create proc file '%s'\n", PROC_XBUS_WAITFOR_XPDS);
-			goto err;
-		}
-		SET_PROC_DIRENTRY_OWNER(worker->proc_xbus_waitfor_xpds);
-	}
-#endif
-#endif
 	worker->wq = create_singlethread_workqueue(xbus->busname);
 	if(!worker->wq) {
 		XBUS_ERR(xbus, "Failed to create worker workqueue.\n");
@@ -1346,6 +1205,7 @@ void xbus_deactivate(xbus_t *xbus)
 		return;
 	xbus_request_sync(xbus, SYNC_MODE_NONE);	/* no more ticks */
 	elect_syncer("deactivate");
+	xbus_echocancel(xbus, 0);
 	xbus_request_removal(xbus);
 	XBUS_DBG(DEVICES, xbus, "[%s] Waiting for queues\n", xbus->label);
 	xbus_command_queue_clean(xbus);
@@ -1421,16 +1281,6 @@ void xbus_free(xbus_t *xbus)
 	BUG_ON(!xbuses_array[num].xbus);
 	BUG_ON(xbus != xbuses_array[num].xbus);
 	spin_unlock_irqrestore(&xbuses_lock, flags);
-#ifdef	XPP_DEBUGFS
-	if(xbus->debugfs_dir) {
-		if(xbus->debugfs_file) {
-			XBUS_DBG(GENERAL, xbus, "Removing debugfs file\n");
-			debugfs_remove(xbus->debugfs_file);
-		}
-		XBUS_DBG(GENERAL, xbus, "Removing debugfs directory\n");
-		debugfs_remove(xbus->debugfs_dir);
-	}
-#endif
 #ifdef CONFIG_PROC_FS
 	if(xbus->proc_xbus_dir) {
 		if(xbus->proc_xbus_summary) {
@@ -1525,18 +1375,6 @@ xbus_t *xbus_new(struct xbus_ops *ops, ushort max_send_size, struct device *tran
 	xbus->proc_xbus_command->data = xbus;
 	SET_PROC_DIRENTRY_OWNER(xbus->proc_xbus_command);
 #endif
-#endif
-#ifdef	XPP_DEBUGFS
-	xbus->debugfs_dir = debugfs_create_dir(xbus->busname, debugfs_root);
-	if(!xbus->debugfs_dir) {
-		XBUS_ERR(xbus, "Failed to create debugfs directory\n");
-		goto nobus;
-	}
-	xbus->debugfs_file = debugfs_create_file("dchannel", S_IFREG|S_IRUGO|S_IWUSR, xbus->debugfs_dir, xbus, &debugfs_operations);
-	if(!xbus->debugfs_file) {
-		XBUS_ERR(xbus, "Failed to create dchannel file\n");
-		goto nobus;
-	}
 #endif
 	xframe_queue_init(&xbus->command_queue, 10, command_queue_length, "command_queue", xbus);
 	xframe_queue_init(&xbus->receive_queue, 10, 50, "receive_queue", xbus);
@@ -1667,7 +1505,6 @@ static int xbus_fill_proc_queue(char *p, struct xframe_queue *q)
 static int xbus_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data)
 {
 	xbus_t			*xbus;
-	struct xbus_workqueue	*worker;
 	unsigned long		flags;
 	int			len = 0;
 	int			i = (int)((unsigned long)data);
@@ -1676,7 +1513,6 @@ static int xbus_read_proc(char *page, char **start, off_t off, int count, int *e
 	if(!xbus)
 		goto out;
 	spin_lock_irqsave(&xbus->lock, flags);
-	worker = &xbus->worker;
 
 	len += sprintf(page + len, "%s: CONNECTOR=%s LABEL=[%s] STATUS=%s\n",
 			xbus->busname,
@@ -1738,35 +1574,6 @@ out:
 
 }
 
-#ifdef	OLD_PROC
-static int xbus_read_waitfor_xpds(char *page, char **start, off_t off, int count, int *eof, void *data)
-{
-	int			len = 0;
-	int			i = (int)((unsigned long)data);
-	xbus_t			*xbus;
-
-	xbus = get_xbus(__func__, i);
-	if (xbus != NULL) {
-		XBUS_NOTICE(xbus, "%s: DEPRECATED: %s[%d] read from /proc interface instead of /sys\n",
-			__func__, current->comm, current->tgid);
-		/* first handle special cases */
-		if (count && !off)
-			len = waitfor_xpds(xbus, page);
-		put_xbus(__func__, xbus);
-	}
-	if (len <= off+count)
-		*eof = 1;
-	*start = page + off;
-	len -= off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-	return len;
-
-}
-#endif
-
 #ifdef	PROTOCOL_DEBUG
 static int proc_xbus_command_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
 {
@@ -1944,12 +1751,6 @@ void transportops_put(xbus_t *xbus)
 static void xbus_core_cleanup(void)
 {
 	finalize_xbuses_array();
-#ifdef	XPP_DEBUGFS
-	if(debugfs_root) {
-		DBG(GENERAL, "Removing xpp from debugfs\n");
-		debugfs_remove(debugfs_root);
-	}
-#endif
 #ifdef CONFIG_PROC_FS
 	if(proc_xbuses) {
 		DBG(PROC, "Removing " PROC_XBUSES " from proc\n");
@@ -1967,9 +1768,6 @@ int __init xbus_core_init(void)
 #ifdef PROTOCOL_DEBUG
 	INFO("FEATURE: with PROTOCOL_DEBUG\n");
 #endif
-#ifdef	XPP_DEBUGFS
-	INFO("FEATURE: with XPP_DEBUGFS support\n");
-#endif
 #ifdef CONFIG_PROC_FS
 	proc_xbuses = create_proc_read_entry(PROC_XBUSES, 0444, xpp_proc_toplevel, read_proc_xbuses, NULL);
 	if (!proc_xbuses) {
@@ -1978,15 +1776,6 @@ int __init xbus_core_init(void)
 		goto err;
 	}
 	SET_PROC_DIRENTRY_OWNER(proc_xbuses);
-#endif
-#ifdef	XPP_DEBUGFS
-	DBG(GENERAL, "Creating debugfs xpp root\n");
-	debugfs_root = debugfs_create_dir("xpp", NULL);
-	if(!debugfs_root) {
-		ERR("Failed to create debugfs root\n");
-		ret = -EFAULT;
-		goto err;
-	}
 #endif
 	if((ret = xpp_driver_init()) < 0)
 		goto err;
@@ -2028,6 +1817,3 @@ EXPORT_SYMBOL(xframe_init);
 EXPORT_SYMBOL(transportops_get);
 EXPORT_SYMBOL(transportops_put);
 EXPORT_SYMBOL(xbus_command_queue_tick);
-#ifdef XPP_DEBUGFS
-EXPORT_SYMBOL(xbus_log);
-#endif
