@@ -34,9 +34,6 @@
 
 /* forward declarations */
 struct xbus_workqueue;
-#ifdef XPP_DEBUGFS
-struct debugfs_data;
-#endif
 
 #ifdef	__KERNEL__
 
@@ -135,11 +132,6 @@ struct xbus_workqueue {
 	int			num_units;
 	int			num_units_initialized;
 	wait_queue_head_t	wait_for_xpd_initialization;
-#ifdef	CONFIG_PROC_FS
-#ifdef	OLD_PROC
-	struct proc_dir_entry	*proc_xbus_waitfor_xpds;
-#endif
-#endif
 	spinlock_t		worker_lock;
 	struct semaphore	running_initialization;
 };
@@ -163,6 +155,28 @@ xbus_t	*get_xbus(const char *msg, uint num);
 void	put_xbus(const char *msg, xbus_t *xbus);
 int	refcount_xbus(xbus_t *xbus);
 
+/*
+ * Echo canceller related data
+ */
+#define	ECHO_TIMESLOTS	128
+
+struct echoops {
+	int	(*ec_set)(xpd_t *xpd, int pos, bool on);
+	int	(*ec_get)(xpd_t *xpd, int pos);
+	int	(*ec_update)(xbus_t *xbus);
+	void	(*ec_dump)(xbus_t *xbus);
+};
+
+struct xbus_echo_state {
+	const struct echoops	*echoops;
+	byte			timeslots[ECHO_TIMESLOTS];
+	int			xpd_idx;
+	struct device_attribute	*da[MAX_XPDS];
+};
+#define	ECHOOPS(xbus)			((xbus)->echo_state.echoops)
+#define	EC_METHOD(name, xbus)		(ECHOOPS(xbus)->name)
+#define	CALL_EC_METHOD(name, xbus, ...)	(EC_METHOD(name, (xbus))(__VA_ARGS__))
+
 /*
  * An xbus is a transport layer for Xorcom Protocol commands
  */
@@ -177,6 +191,7 @@ struct xbus {
 
 	int			num;
 	struct xpd		*xpds[MAX_XPDS];
+	struct xbus_echo_state	echo_state;
 
 	int			command_tick_counter;
 	int			usec_nosend;		/* Firmware flow control */
@@ -192,6 +207,10 @@ struct xbus {
 	int			cpu_rcv_intr[NR_CPUS];
 	int			cpu_rcv_tasklet[NR_CPUS];
 
+	struct quirks {
+		unsigned int has_fxo:1;
+		unsigned int has_digital_span:1;
+	}			quirks;
 	bool			self_ticking;
 	enum sync_mode		sync_mode;
 	/* Managed by low-level drivers: */
@@ -240,11 +259,6 @@ struct xbus {
 
 	atomic_t		num_xpds;
 
-#ifdef	XPP_DEBUGFS
-	struct dentry		*debugfs_dir;
-	struct dentry		*debugfs_file;
-	struct debugfs_data	*debugfs_data;
-#endif
 #ifdef CONFIG_PROC_FS
 	struct proc_dir_entry	*proc_xbus_dir;
 	struct proc_dir_entry	*proc_xbus_summary;
@@ -284,11 +298,6 @@ void xframe_init(xbus_t *xbus, xframe_t *xframe, void *buf, size_t maxsize, void
 int xbus_core_init(void);		/* Initializer */
 void xbus_core_shutdown(void);		/* Terminator */
 
-#ifdef XPP_DEBUGFS
-/* Debugfs handling */
-int xbus_log(xbus_t *xbus, xpd_t *xpd, int direction, const void *buf, unsigned long len);
-#endif
-
 /* Frame handling */
 void dump_xframe(const char msg[], const xbus_t *xbus, const xframe_t *xframe, int debug);
 int send_cmd_frame(xbus_t *xbus, xframe_t *xframe);
@@ -329,6 +338,7 @@ int	xbus_xpd_unbind(xbus_t *xbus, xpd_t *xpd);
 /* sysfs */
 int	xpd_device_register(xbus_t *xbus, xpd_t *xpd);
 void	xpd_device_unregister(xpd_t *xpd);
+int	echocancel_xpd(xpd_t *xpd, int on);
 
 int	xpp_driver_init(void);
 void	xpp_driver_exit(void);
