@@ -33,7 +33,7 @@
 #include "dahdi_debug.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xbus-pcm.c 8984 2010-07-26 00:30:39Z sruffell $";
+static const char rcsid[] = "$Id: xbus-pcm.c 9993 2011-06-28 18:23:00Z tzafrir $";
 
 extern int debug;
 #ifdef	OPTIMIZE_CHANMUTE
@@ -55,8 +55,8 @@ static struct xpp_ticker	dahdi_ticker;
  * I.e: one of our AB or dahdi_ticker
  */
 static struct xpp_ticker	*ref_ticker = NULL;
-static spinlock_t		ref_ticker_lock = SPIN_LOCK_UNLOCKED;
-static spinlock_t		elect_syncer_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(ref_ticker_lock);
+static DEFINE_SPINLOCK(elect_syncer_lock);
 static bool			force_dahdi_sync = 0;	/* from /sys/bus/astribanks/drivers/xppdrv/sync */
 static xbus_t			*global_ticker;
 static struct xpp_ticker	global_ticks_series;
@@ -494,9 +494,10 @@ static void global_tick(void)
 }
 
 #ifdef	DAHDI_SYNC_TICK
-int dahdi_sync_tick(struct dahdi_span *span, int is_master)
+void dahdi_sync_tick(struct dahdi_span *span, int is_master)
 {
-	xpd_t		*xpd = container_of(span, struct xpd, span);
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	static int	redundant_ticks;	/* for extra spans */
 	struct timeval	now;
 
@@ -536,9 +537,8 @@ int dahdi_sync_tick(struct dahdi_span *span, int is_master)
 	xpp_ticker_step(&dahdi_ticker, &now);
 	dahdi_tick_count++;
 	//flip_parport_bit(1);
-	return 0;
 noop:
-	return 0;	/* No auto sync from dahdi */
+	return;
 }
 #endif
 
@@ -628,10 +628,9 @@ void elect_syncer(const char *msg)
 				xpd_t	*xpd = xpd_of(xbus, j);
 				int	prio;
 
-				if(!xpd || !xpd->card_present)
+				if(!xpd || !xpd->card_present || !IS_PHONEDEV(xpd))
 					continue;
-				prio = CALL_XMETHOD(card_timing_priority,
-						xbus, xpd);
+				prio = CALL_PHONE_METHOD(card_timing_priority, xpd);
 				if (prio < 0) {
 					DBG(SYNC, "%s/%s: skip sync\n",
 						xbus->busname, xpd->xpdname);
@@ -669,10 +668,10 @@ void elect_syncer(const char *msg)
  */
 void update_wanted_pcm_mask(xpd_t *xpd, xpp_line_t new_mask, uint new_pcm_len)
 {
-	xpd->pcm_len = new_pcm_len;
-	xpd->wanted_pcm_mask = new_mask;
+	PHONEDEV(xpd).pcm_len = new_pcm_len;
+	PHONEDEV(xpd).wanted_pcm_mask = new_mask;
 	XPD_DBG(SIGNAL, xpd, "pcm_len=%d wanted_pcm_mask=0x%X\n",
-		xpd->pcm_len, xpd->wanted_pcm_mask);
+		PHONEDEV(xpd).pcm_len, PHONEDEV(xpd).wanted_pcm_mask);
 }
 
 /*
@@ -680,20 +679,20 @@ void update_wanted_pcm_mask(xpd_t *xpd, xpp_line_t new_mask, uint new_pcm_len)
  * channels which should be *added* to the automatic calculation.
  * Normally, this argument is 0.
  */
-void generic_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t pcm_mask)
+void generic_card_pcm_recompute(xpd_t *xpd, xpp_line_t pcm_mask)
 {
 	int		i;
 	int		line_count = 0;
 	unsigned long	flags;
 	uint		pcm_len;
 
-	spin_lock_irqsave(&xpd->lock_recompute_pcm, flags);
+	spin_lock_irqsave(&PHONEDEV(xpd).lock_recompute_pcm, flags);
 	//XPD_DBG(SIGNAL, xpd, "pcm_mask=0x%X\n", pcm_mask);
 	/* Add/remove all the trivial cases */
-	pcm_mask |= xpd->offhook_state;
-	pcm_mask |= xpd->oht_pcm_pass;
-	pcm_mask &= ~xpd->digital_inputs;
-	pcm_mask &= ~xpd->digital_outputs;
+	pcm_mask |= PHONEDEV(xpd).offhook_state;
+	pcm_mask |= PHONEDEV(xpd).oht_pcm_pass;
+	pcm_mask &= ~(PHONEDEV(xpd).digital_inputs);
+	pcm_mask &= ~(PHONEDEV(xpd).digital_outputs);
 	for_each_line(xpd, i)
 		if(IS_SET(pcm_mask, i))
 			line_count++;
@@ -709,7 +708,7 @@ void generic_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t pcm_mask)
 		? RPACKET_HEADERSIZE + sizeof(xpp_line_t) + line_count * DAHDI_CHUNKSIZE
 		: 0L;
 	update_wanted_pcm_mask(xpd, pcm_mask, pcm_len);
-	spin_unlock_irqrestore(&xpd->lock_recompute_pcm, flags);
+	spin_unlock_irqrestore(&PHONEDEV(xpd).lock_recompute_pcm, flags);
 }
 
 void fill_beep(u_char *buf, int num, int duration)
@@ -742,16 +741,16 @@ static void do_ec(xpd_t *xpd)
 {
 	int	i;
 
-	for (i = 0;i < xpd->span.channels; i++) {
+	for (i = 0;i < PHONEDEV(xpd).span.channels; i++) {
 		struct dahdi_chan	*chan = XPD_CHAN(xpd, i);
 
-		if(unlikely(IS_SET(xpd->digital_signalling, i)))	/* Don't echo cancel BRI D-chans */
+		if(unlikely(IS_SET(PHONEDEV(xpd).digital_signalling, i)))	/* Don't echo cancel BRI D-chans */
 			continue;
-		if(!IS_SET(xpd->wanted_pcm_mask, i))			/* No ec for unwanted PCM */
+		if(!IS_SET(PHONEDEV(xpd).wanted_pcm_mask, i))			/* No ec for unwanted PCM */
 			continue;
-		dahdi_ec_chunk(chan, chan->readchunk, xpd->ec_chunk2[i]);
-		memcpy(xpd->ec_chunk2[i], xpd->ec_chunk1[i], DAHDI_CHUNKSIZE);
-		memcpy(xpd->ec_chunk1[i], chan->writechunk, DAHDI_CHUNKSIZE);
+		dahdi_ec_chunk(chan, chan->readchunk, PHONEDEV(xpd).ec_chunk2[i]);
+		memcpy(PHONEDEV(xpd).ec_chunk2[i], PHONEDEV(xpd).ec_chunk1[i], DAHDI_CHUNKSIZE);
+		memcpy(PHONEDEV(xpd).ec_chunk1[i], chan->writechunk, DAHDI_CHUNKSIZE);
 	}
 }
 
@@ -855,21 +854,20 @@ dropit:
  * Generic implementations of card_pcmfromspan()/card_pcmtospan()
  * For FXS/FXO
  */
-void generic_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+void generic_card_pcm_fromspan(xpd_t *xpd, xpacket_t *pack)
 {
 	byte		*pcm;
 	unsigned long	flags;
 	xpp_line_t	wanted_lines;
 	int		i;
 
-	BUG_ON(!xbus);
 	BUG_ON(!xpd);
 	BUG_ON(!pack);
-	wanted_lines = xpd->wanted_pcm_mask;
+	wanted_lines = PHONEDEV(xpd).wanted_pcm_mask;
 	RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, lines) = wanted_lines;
 	pcm = RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, pcm);
 	spin_lock_irqsave(&xpd->lock, flags);
-	for (i = 0; i < xpd->channels; i++) {
+	for (i = 0; i < PHONEDEV(xpd).channels; i++) {
 		struct dahdi_chan	*chan = XPD_CHAN(xpd, i);
 
 		if(IS_SET(wanted_lines, i)) {
@@ -891,7 +889,7 @@ void generic_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	spin_unlock_irqrestore(&xpd->lock, flags);
 }
 
-void generic_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+void generic_card_pcm_tospan(xpd_t *xpd, xpacket_t *pack)
 {
 	byte		*pcm;
 	xpp_line_t	pcm_mask;
@@ -905,11 +903,11 @@ void generic_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	/*
 	 * Calculate the channels we want to mute
 	 */
-	pcm_mute = ~xpd->wanted_pcm_mask;
-	pcm_mute |= xpd->mute_dtmf | xpd->silence_pcm;
+	pcm_mute = ~(PHONEDEV(xpd).wanted_pcm_mask);
+	pcm_mute |= PHONEDEV(xpd).mute_dtmf | PHONEDEV(xpd).silence_pcm;
 	if(!SPAN_REGISTERED(xpd))
 		goto out;
-	for (i = 0; i < xpd->channels; i++) {
+	for (i = 0; i < PHONEDEV(xpd).channels; i++) {
 		volatile u_char	*r = XPD_CHAN(xpd, i)->readchunk;
 		bool		got_data = IS_SET(pcm_mask, i);
 
@@ -917,16 +915,16 @@ void generic_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 			/* We have and want real data */
 			// memset((u_char *)r, 0x5A, DAHDI_CHUNKSIZE);	// DEBUG
 			memcpy((u_char *)r, pcm, DAHDI_CHUNKSIZE);
-		} else if(IS_SET(xpd->wanted_pcm_mask | xpd->silence_pcm, i)) {
+		} else if(IS_SET(PHONEDEV(xpd).wanted_pcm_mask | PHONEDEV(xpd).silence_pcm, i)) {
 			/* Inject SILENCE */
 			memset((u_char *)r, 0x7F, DAHDI_CHUNKSIZE);
-			if(IS_SET(xpd->silence_pcm, i)) {
+			if(IS_SET(PHONEDEV(xpd).silence_pcm, i)) {
 				/*
 				 * This will clear the EC buffers until next tick
 				 * So we don't have noise residues from the past.
 				 */
-				memset(xpd->ec_chunk2[i], 0x7F, DAHDI_CHUNKSIZE);
-				memset(xpd->ec_chunk1[i], 0x7F, DAHDI_CHUNKSIZE);
+				memset(PHONEDEV(xpd).ec_chunk2[i], 0x7F, DAHDI_CHUNKSIZE);
+				memset(PHONEDEV(xpd).ec_chunk1[i], 0x7F, DAHDI_CHUNKSIZE);
 			}
 		}
 		if(got_data)
@@ -937,6 +935,32 @@ out:
 	spin_unlock_irqrestore(&xpd->lock, flags);
 }
 
+int generic_echocancel_timeslot(xpd_t *xpd, int pos)
+{
+	return xpd->addr.unit * 32 + pos;
+}
+EXPORT_SYMBOL(generic_echocancel_timeslot);
+
+int generic_echocancel_setmask(xpd_t *xpd, xpp_line_t ec_mask)
+{
+	int i;
+
+	BUG_ON(!xpd);
+	XPD_DBG(GENERAL, xpd, "0x%8X\n", ec_mask);
+	if (!ECHOOPS(xpd->xbus)) {
+		XPD_DBG(GENERAL, xpd,
+				"No echo canceller in XBUS: Doing nothing.\n");
+		return -EINVAL;
+	}
+	for (i = 0; i < PHONEDEV(xpd).channels; i++) {
+		int on = BIT(i) & ec_mask;
+		CALL_EC_METHOD(ec_set, xpd->xbus, xpd, i, on);
+	}
+	CALL_EC_METHOD(ec_update, xpd->xbus, xpd->xbus);
+	return 0;
+}
+EXPORT_SYMBOL(generic_echocancel_setmask);
+
 static int copy_pcm_tospan(xbus_t *xbus, xframe_t *xframe)
 {
 	byte		*xframe_end;
@@ -994,7 +1018,7 @@ static int copy_pcm_tospan(xbus_t *xbus, xframe_t *xframe)
 			goto out;
 		if(SPAN_REGISTERED(xpd)) {
 			XBUS_COUNTER(xbus, RX_PACK_PCM)++;
-			CALL_XMETHOD(card_pcm_tospan, xbus, xpd, pack);
+			CALL_PHONE_METHOD(card_pcm_tospan, xpd, pack);
 		}
 	} while(p < xframe_end);
 	ret = 0;	/* all good */
@@ -1004,9 +1028,9 @@ out:
 	return ret;
 }
 
-int generic_timing_priority(xbus_t *xbus, xpd_t *xpd)
+int generic_timing_priority(xpd_t *xpd)
 {
-	return xpd->timing_priority;
+	return PHONEDEV(xpd).timing_priority;
 }
 
 static void xbus_tick(xbus_t *xbus)
@@ -1025,10 +1049,10 @@ static void xbus_tick(xbus_t *xbus)
 		if(xpd && SPAN_REGISTERED(xpd)) {
 #ifdef	OPTIMIZE_CHANMUTE
 			int		j;
-			xpp_line_t	xmit_mask = xpd->wanted_pcm_mask;
+			xpp_line_t	xmit_mask = PHONEDEV(xpd).wanted_pcm_mask;
 			
-			xmit_mask |= xpd->silence_pcm;
-			xmit_mask |= xpd->digital_signalling;
+			xmit_mask |= PHONEDEV(xpd).silence_pcm;
+			xmit_mask |= PHONEDEV(xpd).digital_signalling;
 			for_each_line(xpd, j) {
 				XPD_CHAN(xpd, j)->chanmute = (optimize_chanmute)
 					? !IS_SET(xmit_mask, j)
@@ -1039,19 +1063,20 @@ static void xbus_tick(xbus_t *xbus)
 			 * calls to dahdi_transmit should be out of spinlocks, as it may call back
 			 * our hook setting methods.
 			 */
-			dahdi_transmit(&xpd->span);
+			dahdi_transmit(&PHONEDEV(xpd).span);
 		}
 	}
 	/*
 	 * Fill xframes
 	 */
 	for(i = 0; i < MAX_XPDS; i++) {
-		size_t		pcm_len;
-
 		if((xpd = xpd_of(xbus, i)) == NULL)
 			continue;
-		pcm_len = xpd->pcm_len;
+		if (!IS_PHONEDEV(xpd))
+			continue;
 		if(SPAN_REGISTERED(xpd)) {
+			size_t	pcm_len = PHONEDEV(xpd).pcm_len;
+
 			if(pcm_len && xpd->card_present) {
 				do {
 					// pack = NULL;		/* FORCE single packet frames */
@@ -1080,7 +1105,7 @@ static void xbus_tick(xbus_t *xbus)
 					XPACKET_ADDR_SYNC(pack) = 1;
 					sent_sync_bit = 1;
 				}
-				CALL_XMETHOD(card_pcm_fromspan, xbus, xpd, pack);
+				CALL_PHONE_METHOD(card_pcm_fromspan, xpd, pack);
 				XBUS_COUNTER(xbus, TX_PACK_PCM)++;
 			}
 		}
@@ -1119,17 +1144,19 @@ static void xbus_tick(xbus_t *xbus)
 		xpd = xpd_of(xbus, i);
 		if(!xpd || !xpd->card_present)
 			continue;
-		if(SPAN_REGISTERED(xpd)) {
-			do_ec(xpd);
-			dahdi_receive(&xpd->span);
+		if (IS_PHONEDEV(xpd)) {
+			if(SPAN_REGISTERED(xpd)) {
+				do_ec(xpd);
+				dahdi_receive(&PHONEDEV(xpd).span);
+			}
+			PHONEDEV(xpd).silence_pcm = 0;	/* silence was injected */
 		}
-		xpd->silence_pcm = 0;	/* silence was injected */
 		xpd->timer_count = xbus->global_counter;
 		/*
 		 * Must be called *after* tx/rx so
 		 * D-Chan counters may be cleared
 		 */
-		CALL_XMETHOD(card_tick, xbus, xpd);
+		CALL_XMETHOD(card_tick, xpd);
 	}
 }
 
@@ -1218,73 +1245,6 @@ int fill_sync_string(char *buf, size_t count)
 	return len;
 }
 
-#ifdef	OLD_PROC
-#ifdef	CONFIG_PROC_FS
-static int proc_sync_read(char *page, char **start, off_t off, int count, int *eof, void *data)
-{
-	int		len = 0;
-	struct timeval	now;
-	unsigned int	counter = atomic_read(&xpp_tick_counter);
-	unsigned long	usec;
-
-	do_gettimeofday(&now);
-	NOTICE("%s: DEPRECATED: %s[%d] read from /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	len += sprintf(page + len, "# To modify sync source write into this file:\n");
-	len += sprintf(page + len, "#     DAHDI       - Another dahdi device provide sync\n");
-	len += sprintf(page + len, "#     SYNC=nn     - XBUS-nn provide sync\n");
-	len += sprintf(page + len, "#     QUERY=nn    - Query XBUS-nn for sync information (DEBUG)\n");
-	len += fill_sync_string(page + len, PAGE_SIZE - len);
-#ifdef	DAHDI_SYNC_TICK
-	if(force_dahdi_sync) {
-		len += sprintf(page + len,
-			"Dahdi Reference Sync (%d registered spans):\n",
-			total_registered_spans());
-		len += sprintf(page + len, "\tdahdi_tick: #%d\n", dahdi_tick_count);
-		len += sprintf(page + len, "\ttick - dahdi_tick = %d\n",
-				counter - dahdi_tick_count);
-	} else {
-		len += sprintf(page + len,
-				"Dahdi Reference Sync Not activated\n");
-	}
-#endif
-	usec = usec_diff(&now, &global_ticks_series.last_sample.tv);
-	len += sprintf(page + len, "\ntick: #%d\n", counter);
-	len += sprintf(page + len,
-		"tick duration: %d usec (measured %ld.%ld msec ago)\n",
-		global_ticks_series.tick_period,
-		usec / 1000, usec % 1000);
-	if (len <= off+count)
-		*eof = 1;
-	*start = page + off;
-	len -= off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-	return len;
-}
-
-static int proc_sync_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
-{
-	char		buf[MAX_PROC_WRITE];
-
-	// DBG(SYNC, "%s: count=%ld\n", __FUNCTION__, count);
-	NOTICE("%s: DEPRECATED: %s[%d] write to /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	if(count >= MAX_PROC_WRITE)
-		return -EINVAL;
-	if(copy_from_user(buf, buffer, count))
-		return -EFAULT;
-	buf[count] = '\0';
-	return exec_sync_command(buf, count);
-}
-
-static struct proc_dir_entry	*top;
-
-#endif
-#endif	/* OLD_PROC */
-
 int xbus_pcm_init(void *toplevel)
 {
 	int			ret = 0;
@@ -1300,34 +1260,11 @@ int xbus_pcm_init(void *toplevel)
 #endif
 	xpp_ticker_init(&global_ticks_series);
 	xpp_ticker_init(&dahdi_ticker);
-#ifdef	OLD_PROC
-#ifdef	CONFIG_PROC_FS
-	{
-		struct proc_dir_entry	*ent;
-
-		top = toplevel;
-		ent = create_proc_entry(PROC_SYNC, 0644, top);
-		if(ent) {
-			ent->read_proc = proc_sync_read;
-			ent->write_proc = proc_sync_write;
-			ent->data = NULL;
-		} else {
-			ret = -EFAULT;
-		}
-	}
-#endif
-#endif	/* OLD_PROC */
 	return ret;
 }
 
 void xbus_pcm_shutdown(void)
 {
-#ifdef	OLD_PROC
-#ifdef CONFIG_PROC_FS
-	DBG(GENERAL, "Removing '%s' from proc\n", PROC_SYNC);
-	remove_proc_entry(PROC_SYNC, top);
-#endif
-#endif	/* OLD_PROC */
 }
 
 
