@@ -105,10 +105,12 @@ int		send_pcm_frame(xbus_t *xbus, xframe_t *xframe);
 void		pcm_recompute(xpd_t *xpd, xpp_line_t tmp_pcm_mask);
 void		xframe_receive_pcm(xbus_t *xbus, xframe_t *xframe);
 void		update_wanted_pcm_mask(xpd_t *xpd, xpp_line_t new_mask, uint new_pcm_len);
-void		generic_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t pcm_mask);
-void		generic_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack);
-void		generic_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack);
-int		generic_timing_priority(xbus_t *xbus, xpd_t *xpd);
+void		generic_card_pcm_recompute(xpd_t *xpd, xpp_line_t pcm_mask);
+void		generic_card_pcm_fromspan(xpd_t *xpd, xpacket_t *pack);
+void		generic_card_pcm_tospan(xpd_t *xpd, xpacket_t *pack);
+int		generic_timing_priority(xpd_t *xpd);
+int		generic_echocancel_timeslot(xpd_t *xpd, int pos);
+int		generic_echocancel_setmask(xpd_t *xpd, xpp_line_t ec_mask);
 void		fill_beep(u_char *buf, int num, int duration);
 const char	*sync_mode_name(enum sync_mode mode);
 void		xbus_set_command_timer(xbus_t *xbus, bool on);
@@ -120,7 +122,7 @@ void		elect_syncer(const char *msg);
 int		exec_sync_command(const char *buf, size_t count);
 int		fill_sync_string(char *buf, size_t count);
 #ifdef	DAHDI_SYNC_TICK
-int		dahdi_sync_tick(struct dahdi_span *span, int is_master);
+void		dahdi_sync_tick(struct dahdi_span *span, int is_master);
 #endif
 
 #ifdef	DEBUG_PCMTX
