@@ -38,12 +38,9 @@
 #include "xpd.h"
 #include "xpp_dahdi.h"
 #include "xbus-core.h"
-#ifdef	XPP_DEBUGFS
-#include "xpp_log.h"
-#endif
 #include "dahdi_debug.h"
 
-static const char rcsid[] = "$Id: xbus-sysfs.c 7625 2009-11-22 21:17:16Z tzafrir $";
+static const char rcsid[] = "$Id: xbus-sysfs.c 9993 2011-06-28 18:23:00Z tzafrir $";
 
 /* Command line parameters */
 extern int debug;
@@ -205,6 +202,16 @@ static DEVICE_ATTR_READER(waitfor_xpds_show, dev, buf)
 	return len;
 }
 
+static DEVICE_ATTR_READER(refcount_xbus_show, dev, buf)
+{
+	xbus_t			*xbus;
+	int			len;
+
+	xbus = dev_to_xbus(dev);
+	len = sprintf(buf, "%d\n", refcount_xbus(xbus));
+	return len;
+}
+
 static DEVICE_ATTR_READER(driftinfo_show, dev, buf)
 {
 	xbus_t			*xbus;
@@ -290,6 +297,7 @@ static struct device_attribute xbus_dev_attrs[] = {
         __ATTR_RO(label),
 	__ATTR_RO(status),
 	__ATTR_RO(timing),
+	__ATTR_RO(refcount_xbus),
 	__ATTR_RO(waitfor_xpds),
 	__ATTR_RO(driftinfo),
 	__ATTR(cls,		S_IWUSR, NULL, cls_store),
@@ -430,6 +438,16 @@ static void astribank_release(struct device *dev)
 	xbus_free(xbus);
 }
 
+static void toplevel_release(struct device *dev)
+{
+	NOTICE("%s\n", __func__);
+}
+
+static struct device toplevel_device = {
+	.release	= toplevel_release,
+	/* No Parent */
+};
+
 static struct bus_type toplevel_bus_type = {
 	.name           = "astribanks",
 	.match          = astribank_match,
@@ -600,7 +618,7 @@ static DEVICE_ATTR_READER(span_show, dev, buf)
 	if(!xpd)
 		return -ENODEV;
 	spin_lock_irqsave(&xpd->lock, flags);
-	len += sprintf(buf, "%d\n", SPAN_REGISTERED(xpd) ? xpd->span.spanno : 0);
+	len += sprintf(buf, "%d\n", SPAN_REGISTERED(xpd) ? PHONEDEV(xpd).span.spanno : 0);
 	spin_unlock_irqrestore(&xpd->lock, flags);
 	return len;
 }
@@ -656,8 +674,8 @@ static DEVICE_ATTR_READER(offhook_show, dev, buf)
 	}
 	if(len) {
 		len--;	/* backout last space */
-		len += sprintf(buf + len, "\n");
 	}
+	len += sprintf(buf + len, "\n");
 	return len;
 }
 
@@ -672,11 +690,24 @@ static DEVICE_ATTR_READER(timing_priority_show, dev, buf)
 	if(!xpd)
 		return -ENODEV;
 	spin_lock_irqsave(&xpd->lock, flags);
-	len += sprintf(buf + len, "%d\n", xpd->timing_priority);
+	len += sprintf(buf + len, "%d\n", PHONEDEV(xpd).timing_priority);
 	spin_unlock_irqrestore(&xpd->lock, flags);
 	return len;
 }
 
+static DEVICE_ATTR_READER(refcount_xpd_show, dev, buf)
+{
+	xpd_t			*xpd;
+	int			len = 0;
+
+	BUG_ON(!dev);
+	xpd = dev_to_xpd(dev);
+	if (!xpd)
+		return -ENODEV;
+	len += sprintf(buf + len, "%d\n", refcount_xpd(xpd));
+	return len;
+}
+
 static int xpd_match(struct device *dev, struct device_driver *driver)
 {
 	struct xpd_driver	*xpd_driver;
@@ -701,6 +732,7 @@ static struct device_attribute xpd_dev_attrs[] = {
         __ATTR_RO(type),
         __ATTR_RO(offhook),
         __ATTR_RO(timing_priority),
+	__ATTR_RO(refcount_xpd),
         __ATTR_NULL,
 };
 
@@ -756,8 +788,6 @@ int xpd_device_register(xbus_t *xbus, xpd_t *xpd)
 		XPD_ERR(xpd, "%s: device_register failed: %d\n", __FUNCTION__, ret);
 		return ret;
 	}
-	get_xpd(__FUNCTION__, xpd);
-	BUG_ON(!xpd);
 	return 0;
 }
 
@@ -777,8 +807,140 @@ void xpd_device_unregister(xpd_t *xpd)
 	dev_set_drvdata(dev, NULL);
 }
 
+static DEVICE_ATTR_READER(echocancel_show, dev, buf)
+{
+	xpd_t *xpd;
+	unsigned long flags;
+	int len = 0;
+	xpp_line_t ec_mask = 0;
+	int i;
+	int ret;
+
+	BUG_ON(!dev);
+	xpd = dev_to_xpd(dev);
+	if (!xpd)
+		return -ENODEV;
+	if (!ECHOOPS(xpd->xbus))
+		return -ENODEV;
+	spin_lock_irqsave(&xpd->lock, flags);
+	for (i = 0; i < PHONEDEV(xpd).channels; i++) {
+		ret = CALL_EC_METHOD(ec_get, xpd->xbus, xpd, i);
+		if (ret < 0) {
+			LINE_ERR(xpd, i, "ec_get failed\n");
+			len = -ENODEV;
+			goto out;
+		}
+		if (ret)
+			ec_mask |= (1 << i);
+	}
+	len += sprintf(buf, "0x%08X\n", ec_mask);
+out:
+	spin_unlock_irqrestore(&xpd->lock, flags);
+	return len;
+}
+
+static DEVICE_ATTR_WRITER(echocancel_store, dev, buf, count)
+{
+	xpd_t *xpd;
+	char *endp;
+	unsigned long mask;
+	int channels;
+	int ret;
+
+	BUG_ON(!dev);
+	xpd = dev_to_xpd(dev);
+	XPD_DBG(GENERAL, xpd, "%s\n", buf);
+	if (!xpd)
+		return -ENODEV;
+	if (!ECHOOPS(xpd->xbus)) {
+		XPD_ERR(xpd, "No echo canceller in this XBUS\n");
+		return -ENODEV;
+	}
+	if (!IS_PHONEDEV(xpd)) {
+		XPD_ERR(xpd, "Not a phone device\n");
+		return -ENODEV;
+	}
+	channels = PHONEDEV(xpd).channels;
+	mask = simple_strtoul(buf, &endp, 0);
+	if (*endp != '\0' && *endp != '\n' && *endp != '\r') {
+		XPD_ERR(xpd, "Too many channels: %d\n", channels);
+		return -EINVAL;
+	}
+	if (mask != 0 && __ffs(mask) > channels) {
+		XPD_ERR(xpd,
+			"Channel mask (0x%lX) larger than available channels (%d)\n",
+			mask, channels);
+		return -EINVAL;
+	}
+	XPD_DBG(GENERAL, xpd, "ECHOCANCEL channels: 0x%lX\n", mask);
+	ret = CALL_PHONE_METHOD(echocancel_setmask, xpd, mask);
+	if (ret < 0) {
+		XPD_ERR(xpd, "echocancel_setmask failed\n");
+		return ret;
+	}
+	return count;
+}
+
+static	DEVICE_ATTR(echocancel, S_IRUGO | S_IWUSR, echocancel_show,
+		echocancel_store);
+
+int echocancel_xpd(xpd_t *xpd, int on)
+{
+	int ret;
+
+	XPD_DBG(GENERAL, xpd, "echocancel_xpd(%s)\n", (on) ? "on" : "off");
+	if (!on) {
+		device_remove_file(xpd->echocancel, &dev_attr_echocancel);
+		return 0;
+	}
+
+	ret = device_create_file(&xpd->xpd_dev, &dev_attr_echocancel);
+	if (ret)
+		XPD_ERR(xpd,
+			"%s: device_create_file(echocancel) failed: %d\n",
+			__func__, ret);
+
+	return ret;
+}
+EXPORT_SYMBOL(echocancel_xpd);
+
 /*--------- Sysfs Device handling ----*/
 
+void xbus_sysfs_transport_remove(xbus_t *xbus)
+{
+	struct device	*astribank;
+
+	BUG_ON(!xbus);
+	XBUS_DBG(DEVICES, xbus, "\n");
+	astribank = &xbus->astribank;
+	sysfs_remove_link(&astribank->kobj, "transport");
+}
+
+int xbus_sysfs_transport_create(xbus_t *xbus)
+{
+	struct device	*astribank;
+	struct device	*transport_device;
+	int		ret = 0;
+
+	BUG_ON(!xbus);
+	XBUS_DBG(DEVICES, xbus, "\n");
+	astribank = &xbus->astribank;
+	BUG_ON(!astribank);
+	transport_device = xbus->transport.transport_device;
+	if (!transport_device) {
+		XBUS_ERR(xbus, "%s: Missing transport_device\n", __func__);
+		return -ENODEV;
+	}
+	ret = sysfs_create_link(&astribank->kobj, &transport_device->kobj,
+			"transport");
+	if (ret < 0) {
+		XBUS_ERR(xbus, "%s: sysfs_create_link failed: %d\n",
+				__func__, ret);
+		dev_set_drvdata(astribank, NULL);
+	}
+	return ret;
+}
+
 void xbus_sysfs_remove(xbus_t *xbus)
 {
 	struct device	*astribank;
@@ -786,12 +948,11 @@ void xbus_sysfs_remove(xbus_t *xbus)
 	BUG_ON(!xbus);
 	XBUS_DBG(DEVICES, xbus, "\n");
 	astribank = &xbus->astribank;
-	BUG_ON(!astribank);
-	sysfs_remove_link(&astribank->kobj, "transport");
 	if(!dev_get_drvdata(astribank))
 		return;
 	BUG_ON(dev_get_drvdata(astribank) != xbus);
-	device_unregister(&xbus->astribank);
+	device_unregister(astribank);
+	dev_set_drvdata(astribank, NULL);
 }
 
 int xbus_sysfs_create(xbus_t *xbus)
@@ -801,10 +962,9 @@ int xbus_sysfs_create(xbus_t *xbus)
 
 	BUG_ON(!xbus);
 	astribank = &xbus->astribank;
-	BUG_ON(!astribank);
 	XBUS_DBG(DEVICES, xbus, "\n");
 	astribank->bus = &toplevel_bus_type;
-	astribank->parent = xbus->transport.transport_device;
+	astribank->parent = &toplevel_device;
 	dev_set_name(astribank, "xbus-%02d", xbus->num);
 	dev_set_drvdata(astribank, xbus);
 	astribank->release = astribank_release;
@@ -812,15 +972,7 @@ int xbus_sysfs_create(xbus_t *xbus)
 	if(ret) {
 		XBUS_ERR(xbus, "%s: device_register failed: %d\n", __FUNCTION__, ret);
 		dev_set_drvdata(astribank, NULL);
-		goto out;
 	}
-	ret = sysfs_create_link(&astribank->kobj, &astribank->parent->kobj, "transport");
-	if(ret < 0) {
-		XBUS_ERR(xbus, "%s: sysfs_create_link failed: %d\n", __FUNCTION__, ret);
-		dev_set_drvdata(astribank, NULL);
-		goto out;
-	}
-out:
 	return ret;
 }
 
@@ -829,6 +981,12 @@ int __init xpp_driver_init(void)
 	int	ret;
 
 	DBG(DEVICES, "SYSFS\n");
+	dev_set_name(&toplevel_device, "astribanks");
+	ret = device_register(&toplevel_device);
+	if (ret) {
+		ERR("%s: toplevel device_register failed: %d\n", __func__, ret);
+		goto failed_toplevel;
+	}
 	if((ret = bus_register(&toplevel_bus_type)) < 0) {
 		ERR("%s: bus_register(%s) failed. Error number %d",
 			__FUNCTION__, toplevel_bus_type.name, ret);
@@ -850,6 +1008,8 @@ failed_xpd_bus:
 failed_xpp_driver:
 	bus_unregister(&toplevel_bus_type);
 failed_bus:
+	device_unregister(&toplevel_device);
+failed_toplevel:
 	return ret;
 }
 
@@ -859,6 +1019,7 @@ void xpp_driver_exit(void)
 	bus_unregister(&xpd_type);
 	driver_unregister(&xpp_driver);
 	bus_unregister(&toplevel_bus_type);
+	device_unregister(&toplevel_device);
 }
 
 EXPORT_SYMBOL(xpd_driver_register);
