@@ -45,15 +45,11 @@
 #include "xpp_dahdi.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_dahdi.c 8986 2010-07-26 00:30:43Z sruffell $";
+static const char rcsid[] = "$Id: xpp_dahdi.c 10037 2011-07-12 18:08:42Z tzafrir $";
 
 #ifdef CONFIG_PROC_FS
 struct proc_dir_entry *xpp_proc_toplevel = NULL;
 #define	PROC_DIR		"xpp"
-#ifdef	OLD_PROC
-#define	PROC_XPD_ZTREGISTER	"dahdi_registration"
-#define	PROC_XPD_BLINK		"blink"
-#endif
 #define	PROC_XPD_SUMMARY	"summary"
 #endif
 
@@ -67,6 +63,8 @@ static DEF_PARM_BOOL(prefmaster, 0, 0644, "Do we want to be dahdi preferred sync
 
 #include "dahdi_debug.h"
 
+static void phonedev_cleanup(xpd_t *xpd);
+
 #ifdef	DEBUG_SYNC_PARPORT
 /*
  * Use parallel port to sample our PCM sync and diagnose quality and
@@ -112,12 +110,6 @@ int total_registered_spans(void)
 
 #ifdef	CONFIG_PROC_FS
 static int xpd_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data);
-#ifdef	OLD_PROC
-static int proc_xpd_ztregister_read(char *page, char **start, off_t off, int count, int *eof, void *data);
-static int proc_xpd_ztregister_write(struct file *file, const char __user *buffer, unsigned long count, void *data);
-static int proc_xpd_blink_read(char *page, char **start, off_t off, int count, int *eof, void *data);
-static int proc_xpd_blink_write(struct file *file, const char __user *buffer, unsigned long count, void *data);
-#endif
 #endif
 
 /*------------------------- XPD Management -------------------------*/
@@ -160,26 +152,11 @@ static void xpd_proc_remove(xbus_t *xbus, xpd_t *xpd)
 {
 #ifdef CONFIG_PROC_FS
 	if(xpd->proc_xpd_dir) {
-#ifdef	OLD_PROC
-		chip_proc_remove(xbus, xpd);
-#endif
 		if(xpd->proc_xpd_summary) {
 			XPD_DBG(PROC, xpd, "Removing proc '%s'\n", PROC_XPD_SUMMARY);
 			remove_proc_entry(PROC_XPD_SUMMARY, xpd->proc_xpd_dir);
 			xpd->proc_xpd_summary = NULL;
 		}
-#ifdef	OLD_PROC
-		if(xpd->proc_xpd_ztregister) {
-			XPD_DBG(PROC, xpd, "Removing proc '%s'\n", PROC_XPD_ZTREGISTER);
-			remove_proc_entry(PROC_XPD_ZTREGISTER, xpd->proc_xpd_dir);
-			xpd->proc_xpd_ztregister = NULL;
-		}
-		if(xpd->proc_xpd_blink) {
-			XPD_DBG(PROC, xpd, "Removing proc '%s'\n", PROC_XPD_BLINK);
-			remove_proc_entry(PROC_XPD_BLINK, xpd->proc_xpd_dir);
-			xpd->proc_xpd_blink = NULL;
-		}
-#endif
 		XPD_DBG(PROC, xpd, "Removing %s/%s proc directory\n",
 				xbus->busname, xpd->xpdname);
 		remove_proc_entry(xpd->xpdname, xbus->proc_xbus_dir);
@@ -204,28 +181,6 @@ static int xpd_proc_create(xbus_t *xbus, xpd_t *xpd)
 		goto err;
 	}
 	SET_PROC_DIRENTRY_OWNER(xpd->proc_xpd_summary);
-#ifdef	OLD_PROC
-	xpd->proc_xpd_ztregister = create_proc_entry(PROC_XPD_ZTREGISTER, 0644, xpd->proc_xpd_dir);
-	if (!xpd->proc_xpd_ztregister) {
-		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_XPD_ZTREGISTER);
-		goto err;
-	}
-	SET_PROC_DIRENTRY_OWNER(xpd->proc_xpd_ztregister);
-	xpd->proc_xpd_ztregister->data = xpd;
-	xpd->proc_xpd_ztregister->read_proc = proc_xpd_ztregister_read;
-	xpd->proc_xpd_ztregister->write_proc = proc_xpd_ztregister_write;
-	xpd->proc_xpd_blink = create_proc_entry(PROC_XPD_BLINK, 0644, xpd->proc_xpd_dir);
-	if (!xpd->proc_xpd_blink) {
-		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_XPD_BLINK);
-		goto err;
-	}
-	SET_PROC_DIRENTRY_OWNER(xpd->proc_xpd_blink);
-	xpd->proc_xpd_blink->data = xpd;
-	xpd->proc_xpd_blink->read_proc = proc_xpd_blink_read;
-	xpd->proc_xpd_blink->write_proc = proc_xpd_blink_write;
-	if(chip_proc_create(xbus, xpd) < 0)
-		goto err;
-#endif
 #endif
 	return 0;
 #ifdef	CONFIG_PROC_FS
@@ -238,7 +193,6 @@ err:
 void xpd_free(xpd_t *xpd)
 {
 	xbus_t		*xbus = NULL;
-	unsigned int	x;
 
 	if(!xpd)
 		return;
@@ -251,10 +205,7 @@ void xpd_free(xpd_t *xpd)
 	XPD_DBG(DEVICES, xpd, "\n");
 	xpd_proc_remove(xbus, xpd);
 	xbus_xpd_unbind(xbus, xpd);
-	for (x = 0; x < xpd->channels; x++) {
-		if (xpd->chans[x])
-			KZFREE(xpd->chans[x]);
-	}
+	phonedev_cleanup(xpd);
 	KZFREE(xpd);
 	DBG(DEVICES, "refcount_xbus=%d\n", refcount_xbus(xbus));
 	/*
@@ -293,7 +244,7 @@ int create_xpd(xbus_t *xbus, const xproto_table_t *proto_table,
 			subunit_ports, unit, subunit);
 		return 0;
 	}
-	xpd = proto_table->xops.card_new(xbus, unit, subunit, proto_table, subtype, subunits, subunit_ports, to_phone);
+	xpd = proto_table->xops->card_new(xbus, unit, subunit, proto_table, subtype, subunits, subunit_ports, to_phone);
 	if(!xpd) {
 		XBUS_NOTICE(xbus, "card_new(%d,%d,%d,%d,%d) failed. Ignored.\n",
 			unit, subunit, proto_table->type, subtype, to_phone);
@@ -324,41 +275,39 @@ static int xpd_read_proc(char *page, char **start, off_t off, int count, int *eo
 {
 	int		len = 0;
 	xpd_t		*xpd = data;
-	xbus_t		*xbus;
 	int		i;
 
 	if(!xpd)
 		goto out;
 
-	xbus = xpd->xbus;
 	len += sprintf(page + len, "%s (%s, card %s, span %d)\n"
 			"timing_priority: %d\n"
 			"timer_count: %d span->mainttimer=%d\n"
 			,
 			xpd->xpdname, xpd->type_name,
 			(xpd->card_present) ? "present" : "missing",
-			(SPAN_REGISTERED(xpd)) ? xpd->span.spanno : 0,
-			xpd->timing_priority,
-			xpd->timer_count, xpd->span.mainttimer
+			(SPAN_REGISTERED(xpd)) ? PHONEDEV(xpd).span.spanno : 0,
+			PHONEDEV(xpd).timing_priority,
+			xpd->timer_count, PHONEDEV(xpd).span.mainttimer
 			);
 	len += sprintf(page + len, "xpd_state: %s (%d)\n",
 		xpd_statename(xpd->xpd_state), xpd->xpd_state);
 	len += sprintf(page + len, "open_counter=%d refcount=%d\n",
-		atomic_read(&xpd->open_counter), refcount_xpd(xpd));
+		atomic_read(&PHONEDEV(xpd).open_counter), refcount_xpd(xpd));
 	len += sprintf(page + len, "Address: U=%d S=%d\n", xpd->addr.unit, xpd->addr.subunit);
 	len += sprintf(page + len, "Subunits: %d\n", xpd->subunits);
 	len += sprintf(page + len, "Type: %d.%d\n\n", xpd->type, xpd->subtype);
-	len += sprintf(page + len, "pcm_len=%d\n\n", xpd->pcm_len);
-	len += sprintf(page + len, "wanted_pcm_mask=0x%04X\n\n", xpd->wanted_pcm_mask);
-	len += sprintf(page + len, "mute_dtmf=0x%04X\n\n", xpd->mute_dtmf);
+	len += sprintf(page + len, "pcm_len=%d\n\n", PHONEDEV(xpd).pcm_len);
+	len += sprintf(page + len, "wanted_pcm_mask=0x%04X\n\n", PHONEDEV(xpd).wanted_pcm_mask);
+	len += sprintf(page + len, "mute_dtmf=0x%04X\n\n", PHONEDEV(xpd).mute_dtmf);
 	len += sprintf(page + len, "STATES:");
 	len += sprintf(page + len, "\n\t%-17s: ", "output_relays");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", IS_SET(xpd->digital_outputs, i));
+		len += sprintf(page + len, "%d ", IS_SET(PHONEDEV(xpd).digital_outputs, i));
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "input_relays");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", IS_SET(xpd->digital_inputs, i));
+		len += sprintf(page + len, "%d ", IS_SET(PHONEDEV(xpd).digital_inputs, i));
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "offhook");
 	for_each_line(xpd, i) {
@@ -366,19 +315,19 @@ static int xpd_read_proc(char *page, char **start, off_t off, int count, int *eo
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "oht_pcm_pass");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", IS_SET(xpd->oht_pcm_pass, i));
+		len += sprintf(page + len, "%d ", IS_SET(PHONEDEV(xpd).oht_pcm_pass, i));
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "msg_waiting");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", xpd->msg_waiting[i]);
+		len += sprintf(page + len, "%d ", PHONEDEV(xpd).msg_waiting[i]);
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "ringing");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", xpd->ringing[i]);
+		len += sprintf(page + len, "%d ", PHONEDEV(xpd).ringing[i]);
 	}
 	len += sprintf(page + len, "\n\t%-17s: ", "no_pcm");
 	for_each_line(xpd, i) {
-		len += sprintf(page + len, "%d ", IS_SET(xpd->no_pcm, i));
+		len += sprintf(page + len, "%d ", IS_SET(PHONEDEV(xpd).no_pcm, i));
 	}
 #if 1
 	if(SPAN_REGISTERED(xpd)) {
@@ -391,11 +340,11 @@ static int xpd_read_proc(char *page, char **start, off_t off, int count, int *eo
 			byte	*wp;
 			int j;
 
-			if(IS_SET(xpd->digital_outputs, i))
+			if(IS_SET(PHONEDEV(xpd).digital_outputs, i))
 				continue;
-			if(IS_SET(xpd->digital_inputs, i))
+			if(IS_SET(PHONEDEV(xpd).digital_inputs, i))
 				continue;
-			if(IS_SET(xpd->digital_signalling, i))
+			if(IS_SET(PHONEDEV(xpd).digital_signalling, i))
 				continue;
 			rp = chan->readchunk;
 			wp = chan->writechunk;
@@ -410,9 +359,9 @@ static int xpd_read_proc(char *page, char **start, off_t off, int count, int *eo
 				len += sprintf(page + len, "%02X ", wchunk[j]);
 			}
 			len += sprintf(page + len, " | %c",
-				(IS_SET(xpd->wanted_pcm_mask, i))?'+':' ');
+				(IS_SET(PHONEDEV(xpd).wanted_pcm_mask, i))?'+':' ');
 			len += sprintf(page + len, " %c",
-				(IS_SET(xpd->mute_dtmf, i))?'-':' ');
+				(IS_SET(PHONEDEV(xpd).mute_dtmf, i))?'-':' ');
 		}
 	}
 #endif
@@ -502,6 +451,57 @@ badstate:
 }
 
 
+/*
+ * Cleanup/initialize phonedev
+ */
+static void phonedev_cleanup(xpd_t *xpd)
+{
+	struct phonedev	*phonedev = &PHONEDEV(xpd);
+	unsigned int	x;
+
+	for (x = 0; x < phonedev->channels; x++) {
+		if (phonedev->chans[x]) {
+			KZFREE(phonedev->chans[x]);
+		}
+		if (phonedev->ec[x])
+			KZFREE(phonedev->ec[x]);
+	}
+}
+
+__must_check static int phonedev_init(xpd_t *xpd, const xproto_table_t *proto_table,
+	int channels, xpp_line_t no_pcm)
+{
+	struct phonedev	*phonedev = &PHONEDEV(xpd);
+	unsigned int	x;
+
+	spin_lock_init(&phonedev->lock_recompute_pcm);
+	phonedev->channels = channels;
+	phonedev->no_pcm = no_pcm;
+	phonedev->offhook_state = 0x0;	/* ONHOOK */
+	phonedev->phoneops = proto_table->phoneops;
+	phonedev->digital_outputs = 0;
+	phonedev->digital_inputs = 0;
+	atomic_set(&phonedev->dahdi_registered, 0);
+	atomic_set(&phonedev->open_counter, 0);
+	for (x = 0; x < phonedev->channels; x++) {
+		if (!(phonedev->chans[x] = KZALLOC(sizeof(*(phonedev->chans[x])), GFP_KERNEL))) {
+			ERR("%s: Unable to allocate channel %d\n", __FUNCTION__, x);
+			goto err;
+		}
+		phonedev->ec[x] = KZALLOC(sizeof(*(phonedev->ec[x])),
+				GFP_KERNEL);
+		if (!phonedev->ec[x]) {
+			ERR("%s: Unable to allocate ec state %d\n", __func__,
+					x);
+			goto err;
+		}
+	}
+	return 0;
+err:
+	phonedev_cleanup(xpd);
+	return -ENOMEM;
+}
+
 /*
  * xpd_alloc - Allocator for new XPD's
  *
@@ -514,7 +514,7 @@ __must_check xpd_t *xpd_alloc(xbus_t *xbus,
 	xpd_t		*xpd = NULL;
 	size_t		alloc_size = sizeof(xpd_t) + privsize;
 	int		type = proto_table->type;
-	unsigned int	x;
+	xpp_line_t	no_pcm = 0;
 
 	BUG_ON(!proto_table);
 	XBUS_DBG(DEVICES, xbus, "type=%d channels=%d (alloc_size=%zd)\n",
@@ -532,38 +532,23 @@ __must_check xpd_t *xpd_alloc(xbus_t *xbus,
 	}
 	xpd->priv = (byte *)xpd + sizeof(xpd_t);
 	spin_lock_init(&xpd->lock);
-	spin_lock_init(&xpd->lock_recompute_pcm);
-	xpd->channels = channels;
 	xpd->card_present = 0;
-	xpd->offhook_state = 0x0;	/* ONHOOK */
 	xpd->type = proto_table->type;
 	xpd->xproto = proto_table;
-	xpd->xops = &proto_table->xops;
-	xpd->digital_outputs = 0;
-	xpd->digital_inputs = 0;
+	xpd->xops = proto_table->xops;
 	xpd->xpd_state = XPD_STATE_START;
 	xpd->subtype = subtype;
 	xpd->subunits = subunits;
-
-	atomic_set(&xpd->dahdi_registered, 0);
-	atomic_set(&xpd->open_counter, 0);
 	kref_init(&xpd->kref);
 
 	/* For USB-1 disable some channels */
 	if(MAX_SEND_SIZE(xbus) < RPACKET_SIZE(GLOBAL, PCM_WRITE)) {
-		xpp_line_t	no_pcm;
-
-		no_pcm = 0x7F | xpd->digital_outputs | xpd->digital_inputs;
-		xpd->no_pcm = no_pcm;
+		no_pcm = 0x7F | PHONEDEV(xpd).digital_outputs | PHONEDEV(xpd).digital_inputs;
 		XBUS_NOTICE(xbus, "max xframe size = %d, disabling some PCM channels. no_pcm=0x%04X\n",
-				MAX_SEND_SIZE(xbus), xpd->no_pcm);
-	}
-	for (x = 0; x < xpd->channels; x++) {
-		if (!(xpd->chans[x] = kmalloc(sizeof(*xpd->chans[x]), GFP_KERNEL))) {
-			ERR("%s: Unable to allocate channel %d\n", __FUNCTION__, x);
-			goto err;
-		}
+				MAX_SEND_SIZE(xbus), PHONEDEV(xpd).no_pcm);
 	}
+	if (phonedev_init(xpd, proto_table, channels, no_pcm) < 0)
+		goto err;
 	xbus_xpd_bind(xbus, xpd, unit, subunit);
 	if(xpd_proc_create(xbus, xpd) < 0)
 		goto err;
@@ -577,11 +562,7 @@ __must_check xpd_t *xpd_alloc(xbus_t *xbus,
 err:
 	if(xpd) {
 		xpd_proc_remove(xbus, xpd);
-		for (x = 0; x < xpd->channels; x++) {
-			if (xpd->chans[x]) {
-				kfree(xpd->chans[x]);
-			}
-		}
+		phonedev_cleanup(xpd);
 		KZFREE(xpd);
 	}
 	return NULL;
@@ -606,7 +587,7 @@ void xbus_request_removal(xbus_t *xbus)
 			spin_lock_irqsave(&xpd->lock, flags);
 			xpd->card_present = 0;
 			xpd_setstate(xpd, XPD_STATE_NOHW);
-			xpd->span.alarms = DAHDI_ALARM_NOTOPEN;
+			PHONEDEV(xpd).span.alarms = DAHDI_ALARM_NOTOPEN;
 			spin_unlock_irqrestore(&xpd->lock, flags);
 		}
 	}
@@ -617,9 +598,9 @@ void xbus_request_removal(xbus_t *xbus)
 			if(SPAN_REGISTERED(xpd)) {
 				int j;
 
-				dahdi_alarm_notify(&xpd->span);
+				dahdi_alarm_notify(&PHONEDEV(xpd).span);
 				XPD_DBG(DEVICES, xpd, "Queuing DAHDI_EVENT_REMOVED on all channels to ask user to release them\n");
-				for (j=0; j<xpd->span.channels; j++) {
+				for (j=0; j<PHONEDEV(xpd).span.channels; j++) {
 					dahdi_qevent_lock(XPD_CHAN(xpd, j),DAHDI_EVENT_REMOVED);
 				}
 			}
@@ -632,19 +613,16 @@ void xbus_request_removal(xbus_t *xbus)
  */
 void xpd_remove(xpd_t *xpd)
 {
-	xbus_t	*xbus;
-
 	BUG_ON(!xpd);
-	xbus = xpd->xbus;
 	XPD_INFO(xpd, "Remove\n");
 	dahdi_unregister_xpd(xpd);
-	CALL_XMETHOD(card_remove, xbus, xpd);
+	CALL_XMETHOD(card_remove, xpd);
 	xpd_free(xpd);
 }
 
 void update_xpd_status(xpd_t *xpd, int alarm_flag)
 {
-	struct dahdi_span *span = &xpd->span;
+	struct dahdi_span *span = &PHONEDEV(xpd).span;
 
 	if(!SPAN_REGISTERED(xpd)) {
 		// XPD_NOTICE(xpd, "%s: XPD is not registered. Skipping.\n", __FUNCTION__);
@@ -660,7 +638,7 @@ void update_xpd_status(xpd_t *xpd, int alarm_flag)
 	}
 	if(span->alarms == alarm_flag)
 		return;
-	XPD_DBG(GENERAL, xpd, "Update XPD alarms: %s -> %02X\n", xpd->span.name, alarm_flag);
+	XPD_DBG(GENERAL, xpd, "Update XPD alarms: %s -> %02X\n", PHONEDEV(xpd).span.name, alarm_flag);
 	span->alarms = alarm_flag;
 	dahdi_alarm_notify(span);
 }
@@ -674,12 +652,12 @@ void oht_pcm(xpd_t *xpd, int pos, bool pass)
 {
 	if(pass) {
 		LINE_DBG(SIGNAL, xpd, pos, "OHT PCM: pass\n");
-		BIT_SET(xpd->oht_pcm_pass, pos);
+		BIT_SET(PHONEDEV(xpd).oht_pcm_pass, pos);
 	} else {
 		LINE_DBG(SIGNAL, xpd, pos, "OHT PCM: block\n");
-		BIT_CLR(xpd->oht_pcm_pass, pos);
+		BIT_CLR(PHONEDEV(xpd).oht_pcm_pass, pos);
 	}
-	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 }
 
 /*
@@ -689,12 +667,12 @@ void mark_offhook(xpd_t *xpd, int pos, bool to_offhook)
 {
 	if(to_offhook) {
 		LINE_DBG(SIGNAL, xpd, pos, "OFFHOOK\n");
-		BIT_SET(xpd->offhook_state, pos);
+		BIT_SET(PHONEDEV(xpd).offhook_state, pos);
 	} else {
 		LINE_DBG(SIGNAL, xpd, pos, "ONHOOK\n");
-		BIT_CLR(xpd->offhook_state, pos);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, pos);
 	}
-	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 }
 
 /*
@@ -730,117 +708,11 @@ void hookstate_changed(xpd_t *xpd, int pos, bool to_offhook)
 		 *
 		 * This bit will be cleared on the next tick.
 		 */
-		BIT_SET(xpd->silence_pcm, pos);
+		BIT_SET(PHONEDEV(xpd).silence_pcm, pos);
 	}
 	notify_rxsig(xpd, pos, (to_offhook) ? DAHDI_RXSIG_OFFHOOK : DAHDI_RXSIG_ONHOOK);
 }
 
-#ifdef CONFIG_PROC_FS
-#ifdef	OLD_PROC
-static int proc_xpd_ztregister_read(char *page, char **start, off_t off, int count, int *eof, void *data)
-{
-	int		len = 0;
-	unsigned long	flags;
-	xpd_t		*xpd = data;
-
-	BUG_ON(!xpd);
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] read from /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	spin_lock_irqsave(&xpd->lock, flags);
-	len += sprintf(page + len, "%d\n", SPAN_REGISTERED(xpd) ? xpd->span.spanno : 0);
-	spin_unlock_irqrestore(&xpd->lock, flags);
-	if (len <= off+count)
-		*eof = 1;
-	*start = page + off;
-	len -= off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-	return len;
-}
-
-static int proc_xpd_ztregister_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
-{
-	xpd_t		*xpd = data;
-	char		buf[MAX_PROC_WRITE];
-	int		dahdi_reg;
-	int		ret;
-
-	BUG_ON(!xpd);
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] wrote to /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	if(count >= MAX_PROC_WRITE)
-		return -EINVAL;
-	if(copy_from_user(buf, buffer, count))
-		return -EFAULT;
-	buf[count] = '\0';
-	ret = sscanf(buf, "%d", &dahdi_reg);
-	if(ret != 1)
-		return -EINVAL;
-	if(!XBUS_IS(xpd->xbus, READY))
-		return -ENODEV;
-	XPD_DBG(GENERAL, xpd, "%s\n", (dahdi_reg) ? "register" : "unregister");
-	if(dahdi_reg)
-		ret = dahdi_register_xpd(xpd);
-	else
-		ret = dahdi_unregister_xpd(xpd);
-	return (ret < 0) ? ret : count;
-}
-
-static int proc_xpd_blink_read(char *page, char **start, off_t off, int count, int *eof, void *data)
-{
-	int		len = 0;
-	unsigned long	flags;
-	xpd_t		*xpd = data;
-
-	BUG_ON(!xpd);
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] read from /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	spin_lock_irqsave(&xpd->lock, flags);
-	len += sprintf(page + len, "0x%lX\n", xpd->blink_mode);
-	spin_unlock_irqrestore(&xpd->lock, flags);
-	if (len <= off+count)
-		*eof = 1;
-	*start = page + off;
-	len -= off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-	return len;
-}
-
-static int proc_xpd_blink_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
-{
-	xpd_t		*xpd = data;
-	char		buf[MAX_PROC_WRITE];
-	char		*endp;
-	unsigned long	blink;
-
-
-	BUG_ON(!xpd);
-	XPD_NOTICE(xpd, "%s: DEPRECATED: %s[%d] wrote to /proc interface instead of /sys\n",
-		__FUNCTION__, current->comm, current->tgid);
-	if(count >= MAX_PROC_WRITE)
-		return -EINVAL;
-	if(copy_from_user(buf, buffer, count))
-		return -EFAULT;
-	buf[count] = '\0';
-	if(count > 0 && buf[count-1] == '\n')	/* chomp */
-		buf[count-1] = '\0';
-	blink = simple_strtoul(buf, &endp, 0);
-	if(*endp != '\0' || blink > 0xFFFF)
-		return -EINVAL;
-	XPD_DBG(GENERAL, xpd, "BLINK channels: 0x%lX\n", blink);
-	xpd->blink_mode = blink;
-	return count;
-}
-#endif
-
-#endif
-
-
 #define	XPP_MAX_LEN	512
 
 /*------------------------- Dahdi Interfaces -----------------------*/
@@ -879,13 +751,13 @@ int xpp_open(struct dahdi_chan *chan)
 		return -ENODEV;
 	}
 	spin_lock_irqsave(&xbus->lock, flags);
-	atomic_inc(&xpd->open_counter);
+	atomic_inc(&PHONEDEV(xpd).open_counter);
 	LINE_DBG(DEVICES, xpd, pos, "%s[%d]: open_counter=%d\n",
 		current->comm, current->pid,
-		atomic_read(&xpd->open_counter));
+		atomic_read(&PHONEDEV(xpd).open_counter));
 	spin_unlock_irqrestore(&xbus->lock, flags);
-	if(xpd->xops->card_open)
-		xpd->xops->card_open(xpd, pos);
+	if(PHONE_METHOD(card_open, xpd))
+		CALL_PHONE_METHOD(card_open, xpd, pos);
 	return 0;
 }
 
@@ -898,12 +770,12 @@ int xpp_close(struct dahdi_chan *chan)
 
 	spin_lock_irqsave(&xbus->lock, flags);
 	spin_unlock_irqrestore(&xbus->lock, flags);
-	if(xpd->xops->card_close)
-		xpd->xops->card_close(xpd, pos);
+	if(PHONE_METHOD(card_close, xpd))
+		CALL_PHONE_METHOD(card_close, xpd, pos);
 	LINE_DBG(DEVICES, xpd, pos, "%s[%d]: open_counter=%d\n",
 		current->comm, current->pid,
-		atomic_read(&xpd->open_counter));
-	atomic_dec(&xpd->open_counter);		/* from xpp_open() */
+		atomic_read(&PHONEDEV(xpd).open_counter));
+	atomic_dec(&PHONEDEV(xpd).open_counter);		/* from xpp_open() */
 	put_xpd(__FUNCTION__, xpd);					/* from xpp_open() */
 	return 0;
 }
@@ -935,8 +807,8 @@ int xpp_ioctl(struct dahdi_chan *chan, unsigned int cmd, unsigned long arg)
 	switch (cmd) {
 		default:
 			/* Some span-specific commands before we give up: */
-			if (xpd->xops->card_ioctl != NULL) {
-				return xpd->xops->card_ioctl(xpd, pos, cmd, arg);
+			if (PHONE_METHOD(card_ioctl, xpd)) {
+				return CALL_PHONE_METHOD(card_ioctl, xpd, pos, cmd, arg);
 			}
 			report_bad_ioctl(THIS_MODULE->name, xpd, pos, cmd);
 			return -ENOTTY;
@@ -955,7 +827,7 @@ int xpp_hooksig(struct dahdi_chan *chan, enum dahdi_txsig txsig)
 			__FUNCTION__, pos);
 		return -ENODEV;
 	}
-	if(!xpd->xops->card_hooksig) {
+	if(!PHONE_METHOD(card_hooksig, xpd)) {
 		LINE_ERR(xpd, pos,
 			"%s: No hooksig method for this channel. Ignore.\n",
 			__FUNCTION__);
@@ -964,7 +836,7 @@ int xpp_hooksig(struct dahdi_chan *chan, enum dahdi_txsig txsig)
 	xbus = xpd->xbus;
 	BUG_ON(!xbus);
 	DBG(SIGNAL, "Setting %s to %s (%d)\n", chan->name, txsig2str(txsig), txsig);
-	return CALL_XMETHOD(card_hooksig, xbus, xpd, pos, txsig);
+	return CALL_PHONE_METHOD(card_hooksig, xpd, pos, txsig);
 }
 EXPORT_SYMBOL(xpp_hooksig);
 
@@ -975,7 +847,8 @@ int xpp_setchunksize(struct dahdi_span *span, int chunksize);
 /* Enable maintenance modes */
 int xpp_maint(struct dahdi_span *span, int cmd)
 {
-	xpd_t		*xpd = container_of(span, struct xpd, span);
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	int		ret = 0;
 #if 0
 	char		loopback_data[] = "THE-QUICK-BROWN-FOX-JUMPED-OVER-THE-LAZY-DOG";
@@ -994,14 +867,10 @@ int xpp_maint(struct dahdi_span *span, int cmd)
 			break;
 		case DAHDI_MAINT_LOOPUP:
 			INFO("XXX Send loopup code XXX\n");
-			// CALL_XMETHOD(LOOPBACK_AX, xpd->xbus, xpd, loopback_data, ARRAY_SIZE(loopback_data));
 			break;
 		case DAHDI_MAINT_LOOPDOWN:
 			INFO("XXX Send loopdown code XXX\n");
 			break;
-		case DAHDI_MAINT_LOOPSTOP:
-			INFO("XXX Stop sending loop codes XXX\n");
-			break;
 		default:
 			ERR("XPP: Unknown maint command: %d\n", cmd);
 			ret = -EINVAL;
@@ -1027,6 +896,105 @@ static int xpp_watchdog(struct dahdi_span *span, int cause)
 }
 #endif
 
+/*
+ * Hardware Echo Canceller management
+ */
+static void echocan_free(struct dahdi_chan *chan,
+		struct dahdi_echocan_state *ec)
+{
+	xpd_t			*xpd;
+	xbus_t			*xbus;
+	int			pos = chan->chanpos - 1;
+	const struct echoops	*echoops;
+
+	xpd = chan->pvt;
+	xbus = xpd->xbus;
+	echoops = ECHOOPS(xbus);
+	if (!echoops)
+		return;
+	LINE_DBG(GENERAL, xpd, pos, "mode=0x%X\n", ec->status.mode);
+	CALL_EC_METHOD(ec_set, xbus, xpd, pos, 0);
+	CALL_EC_METHOD(ec_update, xbus, xbus);
+}
+
+static const struct dahdi_echocan_features xpp_ec_features = {
+};
+
+static const struct dahdi_echocan_ops xpp_ec_ops = {
+	.echocan_free = echocan_free,
+};
+
+const char *xpp_echocan_name(const struct dahdi_chan *chan)
+{
+	xpd_t	*xpd;
+	xbus_t	*xbus;
+	int	pos;
+
+	if (!chan) {
+		NOTICE("%s(NULL)\n", __func__);
+		return "XPP";
+	}
+	xpd = chan->pvt;
+	xbus = xpd->xbus;
+	pos = chan->chanpos - 1;
+	LINE_DBG(GENERAL, xpd, pos, "%s:\n", __func__);
+	if (!ECHOOPS(xbus))
+		return NULL;
+	/*
+	 * quirks and limitations
+	 */
+	if (xbus->quirks.has_fxo) {
+		if (
+			xbus->quirks.has_digital_span &&
+			xpd->type == XPD_TYPE_FXO) {
+			LINE_NOTICE(xpd, pos,
+				"quirk: give up HWEC on FXO: "
+				"AB has digital span\n");
+			return NULL;
+		} else if (
+			xbus->sync_mode != SYNC_MODE_AB &&
+			xpd->type == XPD_TYPE_FXS) {
+			LINE_NOTICE(xpd, pos,
+				"quirk: give up HWEC on FXS: "
+				"AB has FXO and is sync slave\n");
+			return NULL;
+		}
+	}
+	return "XPP";
+}
+EXPORT_SYMBOL(xpp_echocan_name);
+
+int xpp_echocan_create(struct dahdi_chan *chan,
+				struct dahdi_echocanparams *ecp,
+				struct dahdi_echocanparam *p,
+				struct dahdi_echocan_state **ec)
+{
+	xpd_t			*xpd;
+	xbus_t			*xbus;
+	int			pos;
+	struct phonedev		*phonedev;
+	const struct echoops	*echoops;
+	int			ret;
+
+	xpd = chan->pvt;
+	xbus = xpd->xbus;
+	pos = chan->chanpos - 1;
+	echoops = ECHOOPS(xbus);
+	if (!echoops)
+		return -ENODEV;
+	phonedev = &PHONEDEV(xpd);
+	*ec = phonedev->ec[pos];
+	(*ec)->ops = &xpp_ec_ops;
+	(*ec)->features = xpp_ec_features;
+	LINE_DBG(GENERAL, xpd, pos, "(tap=%d, param_count=%d)\n",
+		ecp->tap_length, ecp->param_count);
+	ret = CALL_EC_METHOD(ec_set, xbus, xpd, pos, 1);
+	CALL_EC_METHOD(ec_update, xbus, xbus);
+	return ret;
+}
+EXPORT_SYMBOL(xpp_echocan_create);
+
+
 /**
  * Unregister an xpd from dahdi and release related resources
  * @xpd The xpd to be unregistered
@@ -1045,6 +1013,11 @@ int dahdi_unregister_xpd(xpd_t *xpd)
 	BUG_ON(!xpd);
 	spin_lock_irqsave(&xpd->lock, flags);
 
+	if (!IS_PHONEDEV(xpd)) {
+		XPD_ERR(xpd, "Not a telephony device\n");
+		spin_unlock_irqrestore(&xpd->lock, flags);
+		return -EBADF;
+	}
 	if(!SPAN_REGISTERED(xpd)) {
 		XPD_NOTICE(xpd, "Already unregistered\n");
 		spin_unlock_irqrestore(&xpd->lock, flags);
@@ -1052,20 +1025,20 @@ int dahdi_unregister_xpd(xpd_t *xpd)
 	}
 	update_xpd_status(xpd, DAHDI_ALARM_NOTOPEN);
 	/* We should now have only a ref from the xbus (from create_xpd()) */
-	if(atomic_read(&xpd->open_counter)) {
-		XPD_NOTICE(xpd, "Busy (open_counter=%d). Skipping.\n", atomic_read(&xpd->open_counter));
+	if(atomic_read(&PHONEDEV(xpd).open_counter)) {
+		XPD_NOTICE(xpd, "Busy (open_counter=%d). Skipping.\n", atomic_read(&PHONEDEV(xpd).open_counter));
 		spin_unlock_irqrestore(&xpd->lock, flags);
 		return -EBUSY;
 	}
 	mdelay(2);	// FIXME: This is to give chance for transmit/receiveprep to finish.
 	spin_unlock_irqrestore(&xpd->lock, flags);
 	if(xpd->card_present)
-		xpd->xops->card_dahdi_preregistration(xpd, 0);
-	atomic_dec(&xpd->dahdi_registered);
+		CALL_PHONE_METHOD(card_dahdi_preregistration, xpd, 0);
+	atomic_dec(&PHONEDEV(xpd).dahdi_registered);
 	atomic_dec(&num_registered_spans);
-	dahdi_unregister(&xpd->span);
+	dahdi_unregister(&PHONEDEV(xpd).span);
 	if(xpd->card_present)
-		xpd->xops->card_dahdi_postregistration(xpd, 0);
+		CALL_PHONE_METHOD(card_dahdi_postregistration, xpd, 0);
 	return 0;
 }
 
@@ -1075,6 +1048,8 @@ static const struct dahdi_span_ops xpp_span_ops = {
 	.close = xpp_close,
 	.ioctl = xpp_ioctl,
 	.maint = xpp_maint,
+	.echocan_create = xpp_echocan_create,
+	.echocan_name = xpp_echocan_name,
 };
 
 static const struct dahdi_span_ops xpp_rbs_span_ops = {
@@ -1084,6 +1059,8 @@ static const struct dahdi_span_ops xpp_rbs_span_ops = {
 	.close = xpp_close,
 	.ioctl = xpp_ioctl,
 	.maint = xpp_maint,
+	.echocan_create = xpp_echocan_create,
+	.echocan_name = xpp_echocan_name,
 };
 
 int dahdi_register_xpd(xpd_t *xpd)
@@ -1092,33 +1069,34 @@ int dahdi_register_xpd(xpd_t *xpd)
 	xbus_t		*xbus;
 	int		cn;
 	int		i;
-	const xops_t	*xops;
 
 	BUG_ON(!xpd);
 
-	xops = xpd->xops;
 	xbus = xpd->xbus;
 
+	if (!IS_PHONEDEV(xpd)) {
+		XPD_ERR(xpd, "Not a telephony device\n");
+		return -EBADF;
+	}
 	if (SPAN_REGISTERED(xpd)) {
 		XPD_ERR(xpd, "Already registered\n");
 		return -EEXIST;
 	}
-	cn = xpd->channels;
+	cn = PHONEDEV(xpd).channels;
 	XPD_DBG(DEVICES, xpd, "Initializing span: %d channels.\n", cn);
-	memset(&xpd->span, 0, sizeof(struct dahdi_span));
+	memset(&PHONEDEV(xpd).span, 0, sizeof(struct dahdi_span));
 	for(i = 0; i < cn; i++) {
 		memset(XPD_CHAN(xpd, i), 0, sizeof(struct dahdi_chan));
 	}
 
-	span = &xpd->span;
+	span = &PHONEDEV(xpd).span;
 	snprintf(span->name, MAX_SPANNAME, "%s/%s", xbus->busname, xpd->xpdname);
 	span->deflaw = DAHDI_LAW_MULAW;	/* default, may be overriden by card_* drivers */
-	init_waitqueue_head(&span->maintq);
 	span->channels = cn;
-	span->chans = xpd->chans;
+	span->chans = PHONEDEV(xpd).chans;
 
 	span->flags = DAHDI_FLAG_RBS;
-	if(xops->card_hooksig)
+	if(PHONEDEV(xpd).phoneops->card_hooksig)
 		span->ops = &xpp_rbs_span_ops;	/* Only with RBS bits */
 	else
 		span->ops = &xpp_span_ops;
@@ -1156,17 +1134,17 @@ int dahdi_register_xpd(xpd_t *xpd)
 	 */
 	span->irq = 0;
 
-	snprintf(xpd->span.desc, MAX_SPANDESC, "Xorcom XPD #%02d/%1d%1d: %s",
+	snprintf(PHONEDEV(xpd).span.desc, MAX_SPANDESC, "Xorcom XPD #%02d/%1d%1d: %s",
 			xbus->num, xpd->addr.unit, xpd->addr.subunit, xpd->type_name);
-	XPD_DBG(GENERAL, xpd, "Registering span '%s'\n", xpd->span.desc);
-	xpd->xops->card_dahdi_preregistration(xpd, 1);
-	if(dahdi_register(&xpd->span, prefmaster)) {
+	XPD_DBG(GENERAL, xpd, "Registering span '%s'\n", PHONEDEV(xpd).span.desc);
+	CALL_PHONE_METHOD(card_dahdi_preregistration, xpd, 1);
+	if(dahdi_register(&PHONEDEV(xpd).span, prefmaster)) {
 		XPD_ERR(xpd, "Failed to dahdi_register span\n");
 		return -ENODEV;
 	}
 	atomic_inc(&num_registered_spans);
-	atomic_inc(&xpd->dahdi_registered);
-	xpd->xops->card_dahdi_postregistration(xpd, 1);
+	atomic_inc(&PHONEDEV(xpd).dahdi_registered);
+	CALL_PHONE_METHOD(card_dahdi_postregistration, xpd, 1);
 	/*
 	 * Update dahdi about our state:
 	 *   - Since asterisk didn't open the channel yet,
