@@ -24,7 +24,6 @@
 #include <linux/errno.h>
 #include <linux/module.h>
 #include <linux/init.h>
-#include <linux/errno.h>
 #include <linux/interrupt.h>
 #include <linux/delay.h>	/* for udelay */
 #include <linux/seq_file.h>
@@ -43,7 +42,7 @@
 #endif
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_usb.c 9676 2011-01-21 05:30:39Z sruffell $";
+static const char rcsid[] = "$Id: xpp_usb.c 9929 2011-06-02 20:00:36Z sruffell $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(int, usb1, 0, 0644, "Allow using USB 1.1 interfaces");
@@ -242,7 +241,7 @@ struct xusb {
 
 };
 
-static	spinlock_t	xusb_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(xusb_lock);
 static xusb_t *xusb_array[MAX_BUSES] = {};
 static unsigned bus_count = 0;
 
@@ -817,7 +816,6 @@ static void xusb_disconnect(struct usb_interface *interface)
 	struct usb_host_interface	*iface_desc = usb_altnum_to_altsetting(interface, 0);
 	xusb_t			*xusb;
 	xbus_t			*xbus;
-	int			minor;
 	int			i;
 
 	DBG(DEVICES, "CALLED on interface #%d\n", iface_desc->desc.bInterfaceNumber);
@@ -846,7 +844,6 @@ static void xusb_disconnect(struct usb_interface *interface)
 	xbus_disconnect(xbus);		// Blocking until fully deactivated!
 
 	down (&xusb->sem);
-	minor = xusb->minor;
 
 	/* give back our minor */
 	usb_deregister_dev(interface, &xusb_class);
@@ -925,7 +922,6 @@ static void xpp_receive_callback(USB_PASS_CB(urb))
 	xbus_t		*xbus = xbus_num(xusb->xbus_num);
 	size_t		size;
 	bool		do_resubmit = 1;
-	bool		is_inuse = 0;
 	struct timeval	now;
 
 	do_gettimeofday(&now);
@@ -934,7 +930,6 @@ static void xpp_receive_callback(USB_PASS_CB(urb))
 		XUSB_ERR(xusb, "Received URB does not belong to a valid xbus anymore...\n");
 		return;
 	}
-	is_inuse = 1;
 	if(!xusb->present) {
 		do_resubmit = 0;
 		goto err;
