@@ -28,7 +28,7 @@
 #include <linux/module.h>
 #include <linux/delay.h>
 
-static const char rcsid[] = "$Id: xproto.c 7244 2009-09-29 16:23:13Z tzafrir $";
+static const char rcsid[] = "$Id: xproto.c 9993 2011-06-28 18:23:00Z tzafrir $";
 
 extern	int debug;
 
@@ -414,17 +414,24 @@ const char *xproto_name(xpd_type_t xpd_type)
 	return proto_table->name;
 }
 
-#define	CHECK_XOP(f)	\
+#define	CHECK_XOP(xops, f)	\
 		if(!(xops)->f) { \
 			ERR("%s: missing xmethod %s [%s (%d)]\n", __FUNCTION__, #f, name, type);	\
 			return -EINVAL;	\
 		}
 
+#define	CHECK_PHONEOP(phoneops, f)	\
+		if(!(phoneops)->f) { \
+			ERR("%s: missing phone method %s [%s (%d)]\n", __FUNCTION__, #f, name, type);	\
+			return -EINVAL;	\
+		}
+
 int xproto_register(const xproto_table_t *proto_table)
 {
-	int		type;
-	const char	*name;
-	const xops_t	*xops;
+	int			type;
+	const char		*name;
+	const struct xops	*xops;
+	const struct phoneops	*phoneops;
 	
 	BUG_ON(!proto_table);
 	type = proto_table->type;
@@ -436,19 +443,25 @@ int xproto_register(const xproto_table_t *proto_table)
 	DBG(GENERAL, "%s (%d)\n", name, type);
 	if(xprotocol_tables[type])
 		NOTICE("%s: overriding registration of %s (%d)\n", __FUNCTION__, name, type);
-	xops = &proto_table->xops;
-	CHECK_XOP(card_new);
-	CHECK_XOP(card_init);
-	CHECK_XOP(card_remove);
-	CHECK_XOP(card_tick);
-	CHECK_XOP(card_pcm_recompute);
-	CHECK_XOP(card_pcm_fromspan);
-	CHECK_XOP(card_pcm_tospan);
-	CHECK_XOP(card_dahdi_preregistration);
-	CHECK_XOP(card_dahdi_postregistration);
-	// CHECK_XOP(card_ioctl);	// optional method -- call after testing
-	CHECK_XOP(card_register_reply);
-	CHECK_XOP(XPD_STATE);
+	xops = proto_table->xops;
+	CHECK_XOP(xops, card_new);
+	CHECK_XOP(xops, card_init);
+	CHECK_XOP(xops, card_remove);
+	CHECK_XOP(xops, card_tick);
+	CHECK_XOP(xops, card_register_reply);
+
+	phoneops = proto_table->phoneops;
+	if (phoneops) {
+		CHECK_PHONEOP(phoneops, card_pcm_recompute);
+		CHECK_PHONEOP(phoneops, card_pcm_fromspan);
+		CHECK_PHONEOP(phoneops, card_pcm_tospan);
+		CHECK_PHONEOP(phoneops, echocancel_timeslot);
+		CHECK_PHONEOP(phoneops, echocancel_setmask);
+		CHECK_PHONEOP(phoneops, card_dahdi_preregistration);
+		CHECK_PHONEOP(phoneops, card_dahdi_postregistration);
+		/* optional method -- call after testing: */
+		/*CHECK_PHONEOP(phoneops, card_ioctl);*/
+	}
 
 	xprotocol_tables[type] = proto_table;
 	return 0;
